@@ -14,6 +14,7 @@
 import logging
 logger = logging.getLogger(__name__)
 
+import os
 import traceback
 #~ from django.utils.translation import ugettext as _
 from django.utils.translation import ugettext_lazy as _
@@ -89,7 +90,7 @@ class Permittable(object):
     required = {}
     """
     A dict with permission requirements.
-    See :func:`lino.utils.auth.make_permission_handler`.
+    See :func:`lino.core.auth.make_permission_handler`.
     """
     
     workflow_state_field = None # internally needed for make_permission_handler
@@ -213,6 +214,11 @@ class Parametrizable(object):
     def get_window_layout(self,actor):
         return self.params_layout
         
+    def get_window_size(self,actor):
+        wl = self.get_window_layout(actor)
+        if wl is not None:
+            return wl.window_size
+        
         
 #~ class ActionMetaClass(type):
     #~ def __new__(meta, classname, bases, classDict):
@@ -232,12 +238,21 @@ class InstanceAction(object):
         self.instance = instance
         self.owner = owner
         
-    #~ def __call__(self,ar,**kw):
+    def run_from_code(self,ar,**kw):
+        ar.selected_rows = [self.instance]
+        return self.bound_action.action.run_from_code(ar)
+        
+    def run_from_ui(self,ar,**kw):
+        ar.selected_rows = [self.instance]
+        return self.bound_action.action.run_from_ui(ar)
+        
+        
     def run_from_session(self,ses,**kw):
         #~ print self,args, kw
         ar = self.bound_action.request(**kw)
         ar.setup_from(ses)
-        return self.bound_action.action.run_from_code(self.instance,ar)
+        ar.selected_rows = [self.instance]
+        return self.bound_action.action.run_from_code(ar)
 
     #~ def as_button(self,obj,request,label=None):
         #~ print "Foo"
@@ -245,6 +260,18 @@ class InstanceAction(object):
         #~ btn = settings.SITE.ui.row_action_button(obj,request,ba,label)
         #~ return E.tostring(btn)
         
+    def as_button_elem(self,request,label=None):
+        return settings.SITE.ui.row_action_button(self.instance,request,self.bound_action,label)
+        
+    #~ def as_button(self,request,label=None):
+    def as_button(self,*args,**kw):
+        """
+        Return a HTML chunk with a "button" which, when clicked, will 
+        execute this action on this instance.
+        This is being used in the :ref:`lino.tutorial.polls`.
+        """
+        #~ btn = settings.SITE.ui.row_action_button(self.instance,request,self.bound_action,label)
+        return E.tostring(self.as_button_elem(*args,**kw))
 
 class Action(Parametrizable,Permittable):
     """
@@ -257,28 +284,40 @@ class Action(Parametrizable,Permittable):
     
     icon_name = None 
     """
-    The class name of an icon to be used for this action.
+    The class name of an icon to be used for this action
+    when rendered as toolbar button.
     """
     
-    icon_file = None
-    """
-    The file name of an icon to be used for this action.
-    """
+    #~ icon_file = None
+    #~ """
+    #~ The file name of an icon to be used for this action
+    #~ when rendered by quick_add_buttons.
+    #~ """
     
     _layout_class = layouts.ActionParamsLayout
     
     hidden_elements = frozenset()
     
+    combo_group = None
+    """
+    The name of another action to which to "attach" this action.
+    Both actions will then be rendered as a single combobutton.
+    """
     
     sort_index = 90
     """
+    Determins the sort order in which the actions will be presented to 
+    the user.
     
-    Predefined sort_index values are:
+    List actions are negative and come first.
+    
+    Predefined `sort_index` values are:
     
     ===== =================================
     value action
     ===== =================================
-    10    :class:`insert <InsertRow>`
+    -1    :class:`as_pdf <lino.utils.appy_pod.PrinttableAction>`
+    10    :class:`insert <InsertRow>`, SubmitDetail
     11    :attr:`duplicate <lino.mixins.duplicable.Duplicable.duplicate>`
     20    :class:`detail <ShowDetailAction>`
     30    :class:`delete <DeleteSelected>`
@@ -343,13 +382,19 @@ class Action(Parametrizable,Permittable):
     The hotkey. Currently not used.
     """
     
-    callable_from = None
-    """
-    Either `None`(default) or a tuple of class 
-    objects (subclasses of :class:`Action`).
-    If specified, this action is available only within a window 
-    that has been opened by one of the given actions.
-    """
+    def is_callable_from(self,caller):
+        return isinstance(caller,(GridEdit,ShowDetailAction))
+        #~ if self.select_rows:
+            #~ return isinstance(caller,(GridEdit,ShowDetailAction))
+        #~ return isinstance(caller,GridEdit)
+    
+    #~ callable_from = None
+    #~ """
+    #~ Either `None`(default) or a tuple of class 
+    #~ objects (subclasses of :class:`Action`).
+    #~ If specified, this action is available only within a window 
+    #~ that has been opened by one of the given actions.
+    #~ """
     
     default_format = 'html'
     """
@@ -395,11 +440,19 @@ class Action(Parametrizable,Permittable):
     Whether this action should be displayed as a button in the bottom toolbar and the context menu.
     """
     
+    show_in_row_actions = False
+    """
+    No longer used.
+    Whether this action should be displayed as a button in the 
+    :meth:`action_buttons <lino.core.model.Model.action_buttons>`    
+    column.
+    """
+    
     show_in_workflow = False
     """
     Used internally.
     Whether this action should be displayed 
-    as the :meth:`workflow_buttons <lino.core.model.Model.workflow_buttons>`.
+    as the :meth:`workflow_buttons <lino.core.model.Model.workflow_buttons>` column.
     """
     
     custom_handler = False
@@ -408,6 +461,21 @@ class Action(Parametrizable,Permittable):
     (...)
     """
     
+    select_rows = True
+    """
+    Set this to False if this action is a list action, not a row action.
+    """
+    
+    http_method = 'GET'
+    """
+    HTTP method to use when this action is called using an AJAX call.
+    """
+    
+    preprocessor = 'null' # None
+    """
+    Name of a Javascript function to be invoked on the web client when
+    this action is called.
+    """
     
     #~ def __init__(self,label=None,url_action_name=None,required={},**kw):
     def __init__(self,label=None,**kw):
@@ -437,11 +505,14 @@ class Action(Parametrizable,Permittable):
             self.show_in_bbar = False
         else:
             self.show_in_bbar = True
+            
+        if self.show_in_row_actions:
+            self.custom_handler = True
         
         
         #~ self.set_required(**required)
-        assert self.callable_from is None or isinstance(
-            self.callable_from,(tuple,type)), "%s" % self
+        #~ assert self.callable_from is None or isinstance(
+            #~ self.callable_from,(tuple,type)), "%s" % self
             
         #~ if self.parameters is None:
             #~ assert self.params_layout is None
@@ -520,10 +591,14 @@ class Action(Parametrizable,Permittable):
             return self.defining_actor.actor_id + '.' + self.action_name
         return str(actor) + '.' + self.action_name
         
-    def as_button(self,obj,request,label=None):
-        ba = self.defining_actor.get_url_action(self.action_name)
-        btn = settings.SITE.ui.row_action_button(obj,request,ba,label)
-        return E.tostring(btn)
+    #~ def as_button(self,obj,request,label=None):
+        #~ """
+        #~ Return 
+        #~ This is being used in the :ref:`lino.tutorial.polls`.
+        #~ """
+        #~ ba = self.defining_actor.get_url_action(self.action_name)
+        #~ btn = settings.SITE.ui.row_action_button(obj,request,ba,label)
+        #~ return E.tostring(btn)
         
     def get_action_title(self,ar):
         return ar.get_title()
@@ -564,7 +639,7 @@ class Action(Parametrizable,Permittable):
             #~ raise Exception('20121003 %s %s' % (actor,name))
         if self.defining_actor is not None:
             # already defined in another actor
-            return
+            return True
         if self.action_name is not None:
             raise Exception("tried to attach named action %s.%s" % (actor,self.action_name))
         self.action_name = name
@@ -579,6 +654,7 @@ class Action(Parametrizable,Permittable):
         #~ if name == 'default_action':
             #~ print 20120527, self
         setup_params_choosers(self.__class__)
+        return True
             
     #~ def contribute_to_class(self,model,name):
         #~ ma = model.__dict__.get('_lino_model_actions',None)
@@ -605,15 +681,16 @@ class Action(Parametrizable,Permittable):
         
     def get_view_permission(self,profile):
         """
-        Overridden by :class:`lino_welfare.modlib.pcsw.models.BeIdReadCardAction`
+        Overridden e.g. by :class:`lino.mixins.beid.BeIdReadCardAction`
         to make it available only when :attr:`lino.Lino.use_eid_jslib` is True.
         """
         return True
         
-    def run_from_code(self,obj,ar,**kw):
-        return self.run_from_ui(obj,ar,**kw)
         
-    def run_from_ui(self,row,ar,**kw):
+    def run_from_code(self,ar,**kw):
+        return self.run_from_ui(ar,**kw)
+        
+    def run_from_ui(self,ar,**kw):
         """
         Execute the action on the given `row`. 
         `ar` is an :class:`ActionRequest <lino.core.requests.ActionRequest>` 
@@ -621,6 +698,11 @@ class Action(Parametrizable,Permittable):
         """
         raise NotImplementedError("%s has no run_from_ui() method" % self.__class__)
 
+    def run_from_session(self,ses,obj,*args,**kw): # 20130820
+        #~ ba = action.defining_actor.get_action_by_name(action.action_name)
+        ia = InstanceAction(self,self.defining_actor,obj,None)
+        return ia.run_from_session(ses,*args,**kw)
+        
     def action_param_defaults(self,ar,obj,**kw):
         """
         Same as Actor.param_defaults, except that here it is a instance method
@@ -646,41 +728,6 @@ class TableAction(Action):
     
     def get_action_title(self,ar):
         return ar.get_title()
-        
-
-class RowAction(Action):
-    """
-    Base class for actions that are executed server-side, 
-    either on an individual row (if `single_row` is True) 
-    or on a list.
-    TODO: rename RowAction to ServerSideAction or AjaxAction.
-    """
-    single_row = True
-    preprocessor = None
-    http_method = 'GET'
-    
-    def get_panel_btn_handler(self,actor):
-        if self.single_row:
-            h  = 'Lino.row_action_handler('
-        else:
-            h  = 'Lino.list_action_handler('
-            ls_url = '/' + actor.app_label + '/' + actor.__name__
-            h += "%r," % ls_url
-        h += "%r" % self.action_name
-        h += ",%r" % self.http_method
-        if self.preprocessor:
-            h += "," + self.preprocessor
-        h += ")"
-        return h 
-        
-    #~ def get_action_permission(self,user,obj):
-        #~ return self.actor.get_row_permission(self,ar,obj)
-            
-    #~ def attach_to_actor(self,actor,name):
-        #~ super(RowAction,self).attach_to_actor(actor,name)
-        #~ if not self.url_action_name:
-            #~ self.url_action_name = name 
-
 
 
 class RedirectAction(Action):
@@ -700,21 +747,28 @@ class GridEdit(TableAction):
     opens_a_window = True
     #~ icon_file = 'application_view_list.png' # used e.g. by quick_add_buttons()
 
-    callable_from = tuple()
+    #~ callable_from = tuple()
     action_name = 'grid'
     
+    def is_callable_from(self,caller):
+        return False
+    
     def attach_to_actor(self,actor,name):
         #~ self.label = actor.button_label or actor.label
         self.label = actor.label
-        super(GridEdit,self).attach_to_actor(actor,name)
+        return super(GridEdit,self).attach_to_actor(actor,name)
 
     def get_window_layout(self,actor):
         #~ return self.actor.list_layout
         return None
         
+    def get_window_size(self,actor):
+        return actor.window_size
+        
     def as_html(self,ar):
         t = xghtml.Table()
-        settings.SITE.ui.ar2html(ar,t,ar.sliced_data_iterator)
+        #~ settings.SITE.ui.ar2html(ar,t,ar.sliced_data_iterator)
+        ar.dump2html(t,ar.sliced_data_iterator)
         #~ return t.as_element() # 20130418
         buttons = []
         if ar.limit is None:
@@ -772,33 +826,26 @@ class GridEdit(TableAction):
         
 
 
-class BeIdReadCardAction(RowAction):
-    preprocessor = 'Lino.beid_read_card_processor'
-    http_method = 'POST'
-    
-    def get_button_label(self,actor):
-        return self.label 
-        
-    def get_view_permission(self,profile):
-        if not settings.SITE.use_eid_jslib:
-            return False
-        return super(BeIdReadCardAction,self).get_view_permission(profile)
-  
 
 
 
-
-class ShowDetailAction(RowAction):
+class ShowDetailAction(Action):
     """
     An action that opens the Detail Window of its actor.
     """
-    icon_name = 'x-tbar-detail'
+    icon_name = 'application_form'
     #~ icon_file = 'application_form.png'
     opens_a_window = True
     show_in_workflow = False
+    #~ show_in_row_actions = True
+    
     
     sort_index = 20
-    callable_from = (GridEdit,)
+    #~ callable_from = (GridEdit,)
+    
+    def is_callable_from(self,caller):
+        return isinstance(caller,GridEdit)
+    
     #~ show_in_detail = False
     #~ needs_selection = True
     action_name = 'detail'
@@ -808,6 +855,10 @@ class ShowDetailAction(RowAction):
     def get_window_layout(self,actor):
         return actor.detail_layout
         
+    def get_window_size(self,actor):
+        wl = self.get_window_layout(actor)
+        return wl.window_size
+        
     #~ def get_elem_title(self,elem):
         #~ return _("%s (Detail)")  % unicode(elem)
         
@@ -871,7 +922,7 @@ class ShowDetailAction(RowAction):
         
         
 
-RowAction.callable_from = (GridEdit,ShowDetailAction)
+#~ Action.callable_from = (GridEdit,ShowDetailAction)
 
 class InsertRow(TableAction):
     """
@@ -879,9 +930,12 @@ class InsertRow(TableAction):
     The new row will be actually created only when this 
     window gets submitted.
     """
+    #~ debug_permissions = 20130924
+
     label = _("New")
-    icon_name = 'x-tbar-new' # if action rendered as toolbar button
-    icon_file = 'add.png' # if action rendered by quick_add_buttons
+    #~ icon_name = 'x-tbar-new' # if action rendered as toolbar button
+    icon_name = 'add' # if action rendered as toolbar button
+    #~ icon_file = 'add.png' # if action rendered by quick_add_buttons
     show_in_workflow = False
     opens_a_window = True
     hide_navigator = True
@@ -890,7 +944,7 @@ class InsertRow(TableAction):
     help_text = _("Insert a new record")
     #~ readonly = False # see blog/2012/0726
     required = dict(user_level='user')
-    callable_from = (GridEdit,ShowDetailAction)
+    #~ callable_from = (GridEdit,ShowDetailAction)
     action_name = 'insert'
     #~ label = _("Insert")
     key = INSERT # (ctrl=True)
@@ -902,6 +956,10 @@ class InsertRow(TableAction):
     def get_window_layout(self,actor):
         return actor.insert_layout or actor.detail_layout
 
+    def get_window_size(self,actor):
+        wl = self.get_window_layout(actor)
+        return wl.window_size
+
     def get_action_permission(self,ar,obj,state):
         # see blog/2012/0726
         if ar.get_user().profile.readonly: 
@@ -911,52 +969,55 @@ class InsertRow(TableAction):
 
 
 
-class DuplicateRow(RowAction):
-    opens_a_window = True
-  
-    readonly = False
-    required = dict(user_level='user')
-    callable_from = (GridEdit,ShowDetailAction)
-    action_name = 'duplicate'
-    label = _("Duplicate")
-
+#~ class DuplicateRow(Action):
+    #~ opens_a_window = True
+  #~ 
+    #~ readonly = False
+    #~ required = dict(user_level='user')
+    #~ callable_from = (GridEdit,ShowDetailAction)
+    #~ action_name = 'duplicate'
+    #~ label = _("Duplicate")
+#~ 
 
 class ShowEmptyTable(ShowDetailAction):
     use_param_panel = True
-    callable_from = tuple()
+    #~ callable_from = tuple()
     action_name = 'show' 
     default_format = 'html'
     #~ hide_top_toolbar = True
     hide_navigator = True
     icon_name = None
     
+    def is_callable_from(self,caller):
+        return True
+    
     def attach_to_actor(self,actor,name):
         self.label = actor.label
-        ShowDetailAction.attach_to_actor(self,actor,name)
-        #~ print 20120523, actor, name, 'setup', unicode(self.label)
+        return super(ShowEmptyTable,self).attach_to_actor(actor,name)
         
     def as_html(self,ar):
         return super(ShowEmptyTable,self).as_html(ar,'-99998')
 
-class UpdateRowAction(RowAction):
+class UpdateRowAction(Action):
     show_in_workflow = False
     readonly = False
     required = dict(user_level='user')
     
-
-class DeleteSelected(RowAction):
+        
+class DeleteSelected(Action):
     """
     Delete the row on which it is being executed.
     """
     #~ debug_permissions = "20130222"
-    icon_name = 'x-tbar-delete'
+    #~ icon_name = 'x-tbar-delete'
+    icon_name = 'delete'
     help_text = _("Delete this record")
     auto_save = False
     sort_index = 30
     readonly = False
     show_in_workflow = False
     required = dict(user_level='user')
-    callable_from = (GridEdit,ShowDetailAction)
+    #~ callable_from = (GridEdit,ShowDetailAction)
     #~ needs_selection = True
     label = _("Delete")
     #~ url_action_name = 'delete'
@@ -964,11 +1025,12 @@ class DeleteSelected(RowAction):
     #~ client_side = True
     
         
-class SubmitDetail(RowAction):
+class SubmitDetail(Action):
     #~ debug_permissions = 20130128
     sort_index = 10
     switch_to_detail = False
-    icon_name = 'x-tbar-save'
+    #~ icon_name = 'x-tbar-save'
+    icon_name = 'disk'
     help_text = _("Save changes in this form")
     label = _("Save")
     auto_save = False
@@ -978,10 +1040,13 @@ class SubmitDetail(RowAction):
     readonly = False
     required = dict(user_level='user')
     #~ url_action_name = 'SubmitDetail'
-    callable_from = (ShowDetailAction,)
+    #~ callable_from = (ShowDetailAction,)
+    
+    def is_callable_from(self,caller):
+        return isinstance(caller,ShowDetailAction)
+    
     
 class SubmitInsert(SubmitDetail):
-    sort_index = 10
     switch_to_detail = True
     icon_name = None # don't inherit 'x-tbar-save' from Submitdetail 
     #~ url_action_name = 'SubmitInsert'
@@ -989,7 +1054,12 @@ class SubmitInsert(SubmitDetail):
     action_name = 'post'
     help_text = _("Create the record and open a detail window on it")
     #~ label = _("Insert")
-    callable_from = (InsertRow,)
+    #~ callable_from = (InsertRow,)
+    
+    def is_callable_from(self,caller):
+        return isinstance(caller,InsertRow)
+    
+    
     
 class SubmitInsertAndStay(SubmitInsert):
     sort_index = 11
@@ -999,8 +1069,48 @@ class SubmitInsertAndStay(SubmitInsert):
     help_text = _("Don't open a detail window on the new record")
 
     
+
+
+
+class ShowSlaveTable(Action):
+    TABLE2ACTION_ATTRS = tuple('sort_index help_text icon_name label'.split())
+    #~ label = "ShowSlaveTable"
+    #~ show_in_row_actions = True
+    show_in_bbar = True
     
-class NotifyingAction(RowAction):
+    def __init__(self,slave_table,**kw):
+        self.slave_table = slave_table
+        #~ kw.setdefault('label',slave_table.label)
+        super(ShowSlaveTable,self).__init__(**kw)
+
+    @classmethod
+    def get_actor_label(self):
+        return self._label or self.slave_table.label
+        
+    def attach_to_actor(self,actor,name):
+        if isinstance(self.slave_table,basestring):
+            self.slave_table = settings.SITE.modules.resolve(self.slave_table)
+        for k in self.TABLE2ACTION_ATTRS:
+            setattr(self,k,getattr(self.slave_table,k))
+        #~ self.label = self.slave_table.label
+        #~ self.help_text = self.slave_table.help_text
+        #~ self.icon_name = self.slave_table.icon_name
+        #~ self.icon_file = self.slave_table.icon_file
+        return super(ShowSlaveTable,self).attach_to_actor(actor,name)
+        
+    def run_from_ui(self,ar,**kw):
+        obj = ar.selected_rows[0]
+        #~ ba = self.slave_table.default_action
+        sar = ar.spawn(self.slave_table,master_instance=obj)
+        js = ar.renderer.request_handler(sar)
+        #~ js = ar.renderer.action_call(ar.requesting_panel,ba,{})
+        kw.update(eval_js=js)
+        return kw
+
+    
+class NotifyingAction(Action):
+    
+    show_in_row_actions = True
     
     parameters = dict(
         notify_subject = models.CharField(_("Summary"),blank=True,max_length=200),
@@ -1038,9 +1148,10 @@ class NotifyingAction(RowAction):
             if s is not None: kw.update(notify_body=s)
         return kw
         
-    def run_from_ui(self,obj,ar,**kw):
+    def run_from_ui(self,ar,**kw):
+        obj = ar.selected_rows[0]
         kw.update(message=ar.action_param_values.notify_subject)
-        kw.update(alert=True)
+        #~ kw.update(alert=True)
         kw.update(refresh=True)
         kw.update(success=True)
         #~ kw = super(NotifyingAction,self).run_from_ui(obj,ar,**kw)
@@ -1075,9 +1186,13 @@ def action(*args,**kw):
     """
     def decorator(fn):
         kw.setdefault('custom_handler',True)
-        a = RowAction(*args,**kw)
+        #~ kw.setdefault('show_in_row_actions',True)
+        a = Action(*args,**kw)
         #~ a.run = curry(fn,a)
-        a.run_from_ui = fn
+        def wrapped(ar):
+            obj = ar.selected_rows[0]
+            return fn(obj,ar)
+        a.run_from_ui = wrapped
         return a
     return decorator
         
