@@ -1,3 +1,4 @@
+# -*- coding: UTF-8 -*-
 ## Copyright 2009-2013 Luc Saffre
 ## This file is part of the Lino project.
 ## Lino is free software; you can redistribute it and/or modify 
@@ -44,6 +45,9 @@ from lino.utils import curry, AttrDict
 #~ from lino.core import perms
 from lino.utils import jsgen
 from lino.utils import uncamel
+from lino.utils import join_elems
+#~ from lino.utils import class_dict_items
+from lino.utils.xmlgen.html import E
 
 
 actor_classes = []
@@ -155,7 +159,7 @@ class BoundAction(object):
             else:
                 raise Exception("debug_permissions for %r (required=%s)",self,required)
             
-        from lino.utils.auth import make_permission_handler, make_view_permission_handler
+        from lino.core.auth import make_permission_handler, make_view_permission_handler
         self.allow_view = curry(make_view_permission_handler(
             self,action.readonly,debug_permissions,**required),action)
         self._allow = curry(make_permission_handler(
@@ -168,6 +172,9 @@ class BoundAction(object):
     def get_window_layout(self):
         return self.action.get_window_layout(self.actor)
         
+    def get_window_size(self):
+        return self.action.get_window_size(self.actor)
+        
     def full_name(self):
         return self.action.full_name(self.actor)
         #~ if self.action.action_name is None:
@@ -182,27 +189,30 @@ class BoundAction(object):
     def get_button_label(self,*args):
         return self.action.get_button_label(self.actor,*args)
         
-    def get_panel_btn_handler(self,*args):
-        return self.action.get_panel_btn_handler(self.actor,*args)
+    #~ def get_panel_btn_handler(self,*args):
+        #~ return self.action.get_panel_btn_handler(self.actor,*args)
         
     def setup_action_request(self,*args):
         return self.action.setup_action_request(self.actor,*args)
         
+    def get_row_permission(self,ar,obj,state):
+        #~ if self.actor is None: return False
+        return self.actor.get_row_permission(obj,ar,state,self)
+        
     def get_bound_action_permission(self,ar,obj,state):
         if not self.action.get_action_permission(ar,obj,state):
             #~ if self.action.action_name == 'wf7':
                 #~ logger.info("20130424 actors.BoundAction.get_bound_action_permission")
             return False
-        if self.action.action_name == 'wf7':
-            rv = self._allow(ar.get_user(),obj,state)
-            #~ logger.info("20130424 actors.BoundAction.get_bound_action_permission called %s and got %s",self._allow,rv)
-            return rv
         return self._allow(ar.get_user(),obj,state)
         
     def get_view_permission(self,profile):
         """
         20130511
         """
+        #~ if self.action.__class__.__name__ == 'ShowSlaveTable':
+            #~ logger.info("20130820 BoundAction.get_view_permission()")
+            #~ raise Exception(20130820)
         if not self.actor.get_view_permission(profile):
             return False
         if not self.action.get_view_permission(profile):
@@ -212,8 +222,18 @@ class BoundAction(object):
     def __repr__(self):
         return "<%s(%s,%r)>" % (self.__class__.__name__,self.actor,self.action)
     
+        
+#~ class ClassProperty(property):
+    #~ def __get__(self, cls, owner):
+        #~ return self.fget.__get__(None, owner)()
+#~ 
 
 
+def field_getter(name):
+    def func(cls,obj,ar):
+        #~ print 20130910, repr(obj),name
+        return getattr(obj,name)
+    return func
 
 class ActorMetaClass(type):
     def __new__(meta, classname, bases, classDict):
@@ -225,14 +245,33 @@ class ActorMetaClass(type):
         attributes that are never inherited from base classes:
         """
         # classDict.setdefault('name',classname)
-        classDict.setdefault('label',None)
+        #~ classDict.setdefault('label',None) # 20130906
         #~ classDict.setdefault('related_name',None)
         #~ classDict.setdefault('button_label',None)
         classDict.setdefault('title',None)
         classDict.setdefault('help_text',None)
+        classDict.setdefault('abstract',False)
+        
+        declared_label = classDict.pop('label',None)
+        if declared_label is not None:
+            classDict.update(_label=declared_label)
+        declared_known_values = classDict.pop('known_values',None)
+        if declared_known_values is not None:
+            classDict.update(_known_values=declared_known_values)
+        declared_editable = classDict.pop('editable',None)
+        if declared_editable is not None:
+            classDict.update(_editable=declared_editable)
         
         cls = type.__new__(meta, classname, bases, classDict)
         
+        
+        #~ if not classDict.has_key('label'):
+            #~ cls.label = ClassProperty(cls.get_actor_label)
+            #~ meta.label = property(cls.get_actor_label.im_func) # 20130906
+        #~ if not classDict.has_key('known_values'):
+            #~ cls.known_values = ClassProperty(cls.get_known_values)
+            #~ meta.known_values = property(cls.get_known_values.im_func) # 20130906
+        
         #~ if cls.is_abstract():
             #~ actions.register_params(cls)
         
@@ -269,13 +308,13 @@ class ActorMetaClass(type):
             bd = getattr(b,'virtual_fields',None)
             if bd:
                 cls.virtual_fields.update(bd)
-            
-        for k,v in classDict.items():
-            if isinstance(v,fields.Constant):
-                cls.add_constant(k,v)
-            if isinstance(v,fields.VirtualField): # 20120903b
-                #~ logger.warning("20120903 VirtualField %s on Actor %s" % (k,cls.actor_id))
-                cls.add_virtual_field(k,v)
+                
+        if True: # (20130817) tried to move this to a later moment
+            for k,v in classDict.items():
+                cls.register_class_attribute(k,v)
+                    
+        #~ if classname == 'Tasks':
+            #~ logger.info("20130817 no longer added actor vfs")
                 
                 
         #~ cls.params = []
@@ -288,6 +327,9 @@ class ActorMetaClass(type):
         
                 
         #~ cls.install_params_on_actor()
+        
+        
+        
                 
         if classname not in (
             'Table','AbstractTable','VirtualTable',
@@ -304,9 +346,22 @@ class ActorMetaClass(type):
             #~ logger.debug("ActorMetaClass.__new__(%s)", cls)
         return cls
 
-    def __str__(self):
-        return self.actor_id 
+    def __str__(cls):
+        return cls.actor_id
   
+    @property
+    def label(cls):
+        return cls.get_actor_label()
+        
+    @property
+    def known_values(cls):
+        return cls.get_known_values()
+        
+    @property
+    def editable(cls):
+        return cls.get_actor_editable()
+        
+
 
 class Actor(actions.Parametrizable):
     """
@@ -322,8 +377,84 @@ class Actor(actions.Parametrizable):
     
     _layout_class = layouts.ParamsLayout
     
+    #~ @property
+    #~ def known_values(cls):
+        #~ return cls.get_known_values()
+    
+    
+    
+    sort_index = None 
+    """
+    The sort_index to be used for a ShowSlaveTable action on this actor.
+    """
+    
+    icon_name = None 
+    """
+    The icon_name to be used for a ShowSlaveTable action on this actor.
+    """
+    
+    #~ icon_file = None
+    #~ """
+    #~ The icon_file to be used for a ShowSlaveTable action on this actor.
+    #~ """
+    #~ 
+    
+    
     hidden_elements = frozenset()
     
+    @classmethod
+    def apply_cell_format(self,ar,row,col,recno,td):
+        """
+        Actor-level hook for overriding the formating when rendering 
+        this table as plain html.
+        
+        For example :class:`lino.modlib.cal.models.Events`
+        overrides this.
+        """
+        pass
+        
+    
+    @classmethod
+    def register_class_attribute(cls,k,v):
+        if isinstance(v,fields.Constant):
+            cls.add_constant(k,v)
+        elif isinstance(v,fields.VirtualField): # 20120903b
+            cls.add_virtual_field(k,v)
+        elif isinstance(v,models.Field): # 20130910
+            #~ print "20130910 add virtual field " ,k, cls
+            vf = fields.VirtualField(v,field_getter(k))
+            cls.add_virtual_field(k,vf)
+    
+    @classmethod
+    def inject_field(cls,name,fld):
+        # called from auth.add_user_group()
+        setattr(cls,name,fld)
+        cls.register_class_attribute(name,fld)
+        
+        
+    get_welcome_messages = None
+    """
+    If not None, this must be a class method which takes an ActionRequest 
+    and returns or yields a list of messages to displayed in the welcome 
+    screen.
+    """
+    
+    get_row_classes = None
+    """
+    If not None, this must be a class method which takes a table row
+    and an ActionRequest and returns either None or a string 
+    "red", "green" or "blue" (todo: add more colors 
+    and styles). Example::
+    
+        @classmethod
+        def get_row_classes(cls,obj,ar):
+            if obj.client_state == pcsw.ClientStates.newcomer:
+                return 'green'
+    
+    It will cause an additional special field RowClassStoreField
+    
+    """
+    
     app_label = None
     """
     Specify this if you want to "override" an existing actor.
@@ -414,6 +545,11 @@ class Actor(actions.Parametrizable):
     Set this to True if you don't want Lino to display sums in a table view.
     """
     
+    insert_layout_width = 60
+    """
+    When specifying an :attr:`insert_layout` using a simple a multline 
+    string, then Lino will instantiate a FormPanel with this width.
+    """
     
     workflow_state_field = None 
     """
@@ -428,9 +564,28 @@ class Actor(actions.Parametrizable):
     """
     
     
+    @classmethod
+    def get_pk_field(self):
+        """Return the Django field object used to represent the primary key 
+        when filling selected_pks
+        """
+        return None
+        
+    @classmethod
+    def get_row_by_pk(self,pk):
+        """
+        REturn the data row identified by the given primary key.
+        """
+        return None
+        
     
     
     
+    #~ @classmethod
+    #~ def get_default_table(cls):
+        #~ "For InstanceAction "
+        #~ return cls
+        
     @classmethod
     def disabled_fields(cls,obj,ar):
         """
@@ -496,7 +651,9 @@ class Actor(actions.Parametrizable):
     :class:`lino.modlib.users.models.Mysettings`.
     """
     
-    known_values = {}
+    _label = None
+    _editable = None
+    _known_values = {}
     """
     A `dict` of `fieldname` -> `value` pairs that specify "known values".
     Requests will automatically be filtered to show only existing records 
@@ -581,10 +738,15 @@ class Actor(actions.Parametrizable):
     def make_params_layout_handle(self,ui):
         return actions.make_params_layout_handle(self,ui)
         
-        
+    abstract = False
+    """
+    Set this to True to prevent Lino from generating useless JavaScript 
+    if this is just an abstract base class to be inherited by other actors.
+    """
+    
     @classmethod
-    def is_abstract(self):
-        return False
+    def is_abstract(cls):
+        return cls.abstract
         
             
     @classmethod
@@ -612,8 +774,7 @@ class Actor(actions.Parametrizable):
         pass
         
     @classmethod
-    def summary_row(cls,ar,obj,**kw):
-        return obj.summary_row(ar,**kw)
+    def summary_row(cls,ar,obj,**kw): return obj.summary_row(ar,**kw)
     
     @classmethod
     def get_handle(self):
@@ -665,10 +826,15 @@ class Actor(actions.Parametrizable):
     
     #~ submit_action = actions.SubmitDetail()
     
+    _class_init_done = False
+    
     @classmethod
     def class_init(cls):
-        #~ if str(cls) == 'jobs.JobsOverview':
-            #~ logger.info("20121023 Actor.class_init() %r",cls)
+        """
+        Called internally a site startup. Don't override.
+        """
+        #~ if str(cls) == 'contacts.Persons':
+            #~ logger.info("20130804 Actor.class_init() %r",cls)
         #~ if cls.__name__ == 'Home':
             #~ print "20120524",cls, "class_init()", cls.__bases__
         #~ 20121008 cls.default_action = cls.get_default_action()
@@ -683,13 +849,28 @@ class Actor(actions.Parametrizable):
                 #~ raise Exception("%r has both detail_template and detail_layout" % cls)
             #~ dl = dt
             
+        #~ if cls.is_abstract() : return
+        
+        if False: # (20130817) tried to move this from an earlier moment
+            for k,v in class_dict_items(cls):
+            #~ for k,v in cls.__dict__.items():
+                if isinstance(v,fields.Constant):
+                    cls.add_constant(k,v)
+                if isinstance(v,fields.VirtualField): # 20120903b
+                    cls.add_virtual_field(k,v)
+
+        #~ if str(cls) == 'cal.Tasks':
+            #~ logger.info("20130817 gonna add actor vfs")
+
+
         master = getattr(cls,'master',None)
         if master is not None:
             cls.master = resolve_model(master)
                 
-                
-        #~ dl = classDict.get('detail_layout',None)
-        dl = getattr(cls,'detail_layout',None)
+        #~ dl = getattr(cls,'detail_layout',None) 
+        dl = cls.__dict__.get('detail_layout',None) # see 20130804
+        if dl is None and not cls._class_init_done:
+            dl = cls.detail_layout
         if dl is not None:
             if isinstance(dl,basestring):
                 cls.detail_layout = layouts.FormLayout(dl,cls)
@@ -697,22 +878,16 @@ class Actor(actions.Parametrizable):
                 dl.set_datasource(cls)
                 cls.detail_layout = dl
             elif not issubclass(cls,dl._datasource):
-                raise Exception("Cannot reuse %r detail_layout for %r" % (dl._datasource,cls))
-            #~ else:
-                #~ raise Exception("Cannot reuse detail_layout owned by another table")
-                #~ logger.debug("Note: %s uses layout owned by %s",cls,dl._datasource)
+                raise Exception("Cannot use detail_layout of %r for %r" % (dl._datasource,cls))
             
-        # the same for insert_template and insert_layout:
-        #~ dt = classDict.get('insert_template',None)
-        dt = getattr(cls,'insert_template',None)
-        if dt is not None:
-            raise Exception("Please rename insert_template to insert_layout")
-            
-        #~ dl = classDict.get('insert_layout',None)
-        dl = getattr(cls,'insert_layout',None)
+        #~ dl = getattr(cls,'insert_layout',None)
+        dl = cls.__dict__.get('insert_layout',None)
+        if dl is None and not cls._class_init_done:
+            dl = cls.insert_layout
         if dl is not None:
             if isinstance(dl,basestring):
-                cls.insert_layout = layouts.FormLayout(dl,cls)
+                cls.insert_layout = layouts.FormLayout(dl,cls,
+                    window_size=(cls.insert_layout_width,'auto'))
             elif dl._datasource is None:
                 dl.set_datasource(cls)
                 cls.insert_layout = dl
@@ -720,9 +895,9 @@ class Actor(actions.Parametrizable):
                 raise Exception("Cannot reuse %r insert_layout for %r" % (dl._datasource,cls))
                 #~ logger.debug("Note: %s uses layout owned by %s",cls,dl._datasource)
                 
-        if cls.label is None:
-            #~ self.label = capfirst(self.model._meta.verbose_name_plural)
-            cls.label = cls.get_actor_label()
+        # 20130906
+        #~ if cls.label is None: 
+            #~ cls.label = cls.get_actor_label()
             
         #~ if cls.parameters is not None:
             #~ for name,f in cls.parameters.items():
@@ -756,16 +931,23 @@ class Actor(actions.Parametrizable):
                     #~ for k,v in bd.items():
                         #~ cls._actions_dict[k] = cls.add_action(copy.deepcopy(v),k)
                         
+        _class_init_done = True
                         
+    @classmethod
+    def get_known_values(self):
+        return self._known_values
+        
     @classmethod
     def get_actor_label(self):
         """
         Compute the label of this actor. 
-        Called only if `label` is not set, and only once during site startup.
         """
-        return self.__name__
+        return self._label or self.__name__
+                        
+    @classmethod
+    def get_actor_editable(self):
+        return self._editable
                         
-        
     @classmethod
     def hide_elements(self,*names):
         for name in names:
@@ -868,15 +1050,15 @@ class Actor(actions.Parametrizable):
                 #~ setattr(cls,name,fn())
                 setattr(cls,a.action_name,a) 
 
-        #~ if cls.__name__.startswith('OutboxBy'):
-            #~ print '20120524 collect_actions',cls, cls.insert_action, cls.detail_action, cls.editable
+        # bind all my actions, including those inherited from parent actors:
         for b in cls.mro():
             for k,v in b.__dict__.items():
+                v = cls.__dict__.get(k,v) # 20130820 disable inherited actions
                 if isinstance(v,actions.Action):
                     if not cls._actions_dict.has_key(k):
                         #~ cls._attach_action(k,v)
-                        v.attach_to_actor(cls,k)
-                        cls.bind_action(v)
+                        if v.attach_to_actor(cls,k):
+                            cls.bind_action(v)
         
                         
                     
@@ -900,7 +1082,23 @@ class Actor(actions.Parametrizable):
             self._actions_dict.define(a.action_name,ba)
         self._actions_list.append(ba)
         return ba
+        
+    #~ @classmethod
+    #~ def unbind_action(self,k):
+        #~ self._actions_dict.pop(k,None)
+        #~ self._actions_list = [ba for ba in self._actions_list if ba.action.action_name != k]
+        #~ logger.info("20130820 unbind_action %s",self._actions_list)
 
+    #~ @classmethod
+    #~ def get_row_actions(self,ar,obj):
+        #~ state = self.get_row_state(obj)
+        #~ profile = ar.get_user().profile
+        #~ for ba in self.get_actions(ar.bound_action.action):
+            #~ if ba.action.show_in_row_actions:
+                    #~ if ba.get_view_permission(profile):
+                        #~ if self.get_row_permission(obj,ar,state,ba):
+                            #~ yield ba
+                
     @classmethod
     def get_workflow_actions(self,ar,obj):
         """
@@ -1116,11 +1314,12 @@ class Actor(actions.Parametrizable):
         if not self.is_abstract():
             actions.setup_params_choosers(self)
             
+        #~ logger.info("20130906 Gonna Actor.do_setup() on %s", self)
         self.do_setup()
         #~ self.setup_permissions()
         self._setup_doing = False
         self._setup_done = True
-        #~ logger.debug("20120103 Actor.setup() done: %s, default_action is %s", 
+        #~ logger.info("20130906 Actor.after_site_setup() done: %s, default_action is %s", 
             #~ self.actor_id,self.default_action)
         return True
         
@@ -1143,13 +1342,15 @@ class Actor(actions.Parametrizable):
             #~ raise Exception("Tried to %s.get_actions() with empty _actions_list" % self)
         if callable_from is None:
             return self._actions_list
-        return [ba for ba in self._actions_list 
-          if ba.action.callable_from is None or isinstance(callable_from,ba.action.callable_from)]
+        #~ cf = ba.action.callable_from
+        #~ return [ba for ba in self._actions_list 
+            #~ if cf is None or isinstance(callable_from,cf)]
+        return [ba for ba in self._actions_list if ba.action.is_callable_from(callable_from)]
     
     @classmethod
     def get_data_elem(self,name):
         """
-        Find data element ni this actor by name.
+        Find data element in this actor by name.
         """
         c = self._constants.get(name,None)
         if c is not None:
@@ -1256,7 +1457,7 @@ class Actor(actions.Parametrizable):
             ar = self.request(master,request=ar.request,param_values={})
             ar.renderer = settings.SITE.ui.default_renderer
             #~ s = ui.table2xhtml(ar).tostring()
-            return settings.SITE.ui.table2xhtml(ar)
+            return ar.table2xhtml()
             #~ s = etree.tostring(ui.table2xhtml(ar))
             #~ return s
         return meth
@@ -1287,7 +1488,7 @@ class Actor(actions.Parametrizable):
         """
         """
         #~ settings.SITE.startup()
-        return xghtml.E.tostring(self.request(**kw).table2xhtml())
+        return E.tostring(self.request(**kw).table2xhtml())
         #~ return self.request(**kw).table2xhtml()
         
     #~ def show(self,*args,**kw):
@@ -1313,3 +1514,35 @@ class Actor(actions.Parametrizable):
         For example :class:`lino_faggio.models.InvoiceItems`
         """
         return obj.get_choices_text(request,self,field)
+
+    #~ @fields.displayfield(_("Actions"))
+    #~ def action_buttons(self,obj,ar):
+        #~ return E.p(*join_elems([
+            #~ settings.SITE.ui.row_action_button(obj,ar.request,ba)
+                #~ for ba in ar.actor.get_row_actions(ar,obj)]))
+      
+    @fields.displayfield(_("Workflow"))
+    def workflow_buttons(self,obj,ar):
+        """
+        Displays the workflow buttons for the given row `obj` and the request `ar`.
+        """
+        #~ logger.info('20120930 workflow_buttons %r', obj)
+        actor = ar.actor
+        #~ print 20120621 , actor,  self
+        #~ print 20120618, ar
+        l = []
+        state = actor.get_row_state(obj)
+        if state:
+            #~ l.append(E.b(unicode(state),style="vertical-align:middle;"))
+            l.append(E.b(unicode(state)))
+            #~ l.append(u" » ")
+            #~ l.append(u" \u25b8 ")
+            #~ l.append(u" \u2192 ")
+        #~ sep = u" \u25b8 "
+        sep = u" \u2192 "
+        for ba in ar.actor.get_workflow_actions(ar,obj):
+            l.append(sep)
+            l.append(ar.action_button(ba,obj))
+            sep = ' '
+        return E.p(*l)
+        
