@@ -33,6 +33,7 @@ import datetime
 from appy import Object
 
 from django.conf import settings
+from django.db.models.fields import NOT_PROVIDED
 #~ from django.utils.translation import ugettext as _
 from django.utils.translation import ugettext_lazy as _
 from django.utils.text import capfirst
@@ -78,7 +79,7 @@ from lino.core.dbutils import get_model_report
 from lino.core.tables import AbstractTable, TableRequest, VirtualTable
 #~ from lino.core.tables import GridEdit #, ComputedColumn
 
-
+from lino.utils.xmlgen.html import E
 #~ from lino.modlib import field_choices
 
         
@@ -101,9 +102,10 @@ def wildcard_data_elems(model):
     #~ for f in meta.virtual_fields: yield f.name
     for f in meta.fields: 
         #~ if f.editable:
-        if not isinstance(f,fields.VirtualField):
-            if not getattr(f,'_lino_babel_field',False):
-                yield f
+        if not isinstance(f,fields.RichTextField):
+            if not isinstance(f,fields.VirtualField):
+                if not getattr(f,'_lino_babel_field',False):
+                    yield f
     for f in meta.many_to_many: yield f
     for f in meta.virtual_fields: 
         if not isinstance(f,fields.VirtualField):
@@ -113,45 +115,32 @@ def wildcard_data_elems(model):
     #~ for de in data_elems(self.model): yield de
     
 
-
-#~ def summary_row(obj,ui,rr,**kw):
-def summary_row(obj,ar,**kw):
-    return obj.summary_row(ar,**kw)
-    #~ m = getattr(obj,'summary_row',None)
-    #~ if m:
-        #~ return m(ar,**kw)
-    #~ return ar.renderer.href_to(obj)
-    #~ return ar.ui.ext_renderer.href_to(obj)
+#~ removed 20130810
+#~ def summary_row(obj,ar,**kw):
+    #~ return obj.summary_row(ar,**kw)
   
 
-#~ def summary(ui,rr,separator=', ',max_items=5,before='',after='',**kw):
-def summary(ar,objects,separator=', ',max_items=5,before='',after='',**kw):
+def comma(): return ', '
+
+#~ def summary(ar,objects,separator=', ',max_items=5,before='',after='',**kw):
+def qs2summary(ar,objects,separator=comma,max_items=5,**kw):
     """
     Returns this table as a unicode string.
     
     :param max_items: don't include more than the specified number of items.
     """
-    #~ if format is None:
-        #~ def format(rr,obj):
-            #~ return unicode(obj)
-    s = u''
+    elems = []
     n = 0
-    #~ for i in rr.data_iterator:
     for i in objects:
         if n:
-            s += separator
-        else:
-            s += before
+            elems.append(separator())
         n += 1
-        #~ s += summary_row(i,ui,rr,**kw)
-        s += ar.summary_row(i,**kw)
-        #~ s += i.summary_row(ui,rr,**kw)
+        elems += list(ar.summary_row(i,**kw))
         if n >= max_items:
-            s += separator + '...' + after
-            return s
-    if n:
-        return s + after
-    return s
+            elems += [ separator(),'...' ]
+            break
+            #~ return E.p(*elems)
+    return E.p(*elems)
 
 #~ def default_summary_row(obj,rr):
     #~ return u'<a href="%s" target="_blank">%s</a>' % (rr.get_request_url(str(obj.pk),fmt='detail'),unicode(obj))
@@ -341,8 +330,7 @@ def discover():
       
     - For each model we want to find out the "default table".
       The "choices table" for a foreignkey field is also currently 
-      simply the pointed model's
-      model_table.
+      simply the pointed model's default table.
       :modattr:`_lino_default_table`
 
     """
@@ -540,7 +528,12 @@ class Table(AbstractTable):
             #~ if vf.name == name:
                 #~ return vf
         #~ return cls.model._meta.get_field(name)
+        
     
+    @classmethod
+    def get_pk_field(self):
+        return self.model._meta.pk
+        
     @classmethod
     def get_row_by_pk(self,pk):
         try:
@@ -586,8 +579,9 @@ class Table(AbstractTable):
     @classmethod
     def get_actor_label(self):
         if self.model is None:
-            return self.__name__
-        return self.model._meta.verbose_name_plural
+            #~ return self._label or self.__name__
+            return super(Table,self).get_actor_label()
+        return self._label or self.model._meta.verbose_name_plural
         
     @classmethod
     def class_init(self):
@@ -604,8 +598,7 @@ class Table(AbstractTable):
             self.hidden_columns = self.hidden_columns | self.model.hidden_columns
             self.hidden_elements = self.hidden_elements | self.model.hidden_elements
             
-            
-        if self.model is not None:
+            #~ if self.model is not None:
               
             #~ for b in self.model.mro():
                 #~ for k,v in b.__dict__.items():
@@ -615,18 +608,20 @@ class Table(AbstractTable):
                 for k,v in b.__dict__.items():
                     if isinstance(v,actions.Action):
                         #~ print "20130326 %s.%s = action %s from %s" % (self,k,v,b)
-                        existing_value = self.__dict__.get(k,None)
-                        if existing_value is not None:
-                            if not isinstance(existing_value,actions.Action):
-                                raise Exception(
-                                    "%s cannot get model action %s because name is already used for %r" %
-                                    self,k,existing_value)
-                        else:
+                        existing_value = self.__dict__.get(k,NOT_PROVIDED) 
+                        if existing_value is NOT_PROVIDED:
                             setattr(self,k,v)
-                      
-          
-                    
-              
+                        else:
+                            if existing_value is None: # 20130820
+                                pass
+                                #~ logger.info("%s disables model action '%s'",self,k)
+                                #~ self.unbind_action(k)
+                            else:
+                                if not isinstance(existing_value,actions.Action):
+                                    raise Exception(
+                                        "%s cannot install model action %s because name is already used for %r" %
+                                        self,k,existing_value)
+
             for name in ('workflow_state_field','workflow_owner_field'):
                 if getattr(self,name) is None:
                     setattr(self,name,getattr(self.model,name))
@@ -718,7 +713,7 @@ class Table(AbstractTable):
             or self.model._meta.abstract:
             #~ logger.info('20120621 %s : no real table',h)
             return True
-        return False
+        return self.abstract
           
     #~ @classmethod
     #~ def setup_permissions(self):
@@ -737,6 +732,8 @@ class Table(AbstractTable):
         and the given user.
         """
         #~ logger.info("20121020 dbtables.Table.get_row_permission %s",unicode(ba.action.label))
+        if obj is None: 
+            return True
         return obj.get_row_permission(ar,state,ba)
         
     @classmethod
@@ -777,7 +774,7 @@ class Table(AbstractTable):
             
             
         # logger.info("20120202 Table.get_data_elem found nothing")
-        return fields.get_data_elem(self.model,name)
+        return self.model.get_data_elem(name)
         #~ de = get_data_elem(self.model,name)
         #~ if de is not None: 
             #~ return de
@@ -807,7 +804,7 @@ class Table(AbstractTable):
               details=title,
               master=ar.master_instance)
         return title
-        
+   
     @classmethod
     def get_queryset(self):
         """
@@ -885,12 +882,7 @@ class Table(AbstractTable):
     def create_instance(self,ar,**kw):
         #~ print 20120630, "Actor.create_instance", kw
         instance = self.model(**kw)
-        #~ self.on_create(instance,ar)
-        
         instance.on_create(ar)
-        #~ m = getattr(instance,'on_create',None)
-        #~ if m:
-            #~ m(ar)
         return instance
         
     @classmethod
@@ -904,7 +896,7 @@ class Table(AbstractTable):
             #~ print 20120715, ar
             ar = ar.spawn(self,master_instance=master)
             #~ ar = self.request(ui,None,master_instance=master)
-            s = summary(ar,ar.data_iterator,row_separator)
+            s = qs2summary(ar,ar.data_iterator,row_separator)
             return s
         return meth
         
