@@ -16,20 +16,21 @@
 Defines classes :class:`Frame` and :class:`FrameHandle`
 """
 
+import logging
+logger = logging.getLogger(__name__)
+
+import datetime
+from decimal import Decimal
+
 
 from django.conf import settings
 from django.db import models
 from django.utils.translation import ugettext_lazy as _
 from django.core.exceptions import ValidationError
 from django.contrib.contenttypes import generic
-
-import datetime
-
-from decimal import Decimal
+from django.db.models.fields import NOT_PROVIDED
 
 
-import logging
-logger = logging.getLogger(__name__)
 
 from djangosite.dbutils import full_model_name
 from djangosite.dbutils import obj2str
@@ -44,7 +45,6 @@ from lino.core.dbutils import resolve_model, UnresolvedModel
 from lino.utils import IncompleteDate, d2iso
 #~ from lino.utils.quantities import Duration
 from lino.utils import quantities 
-from lino.utils import get_class_attr
 
 class PasswordField(models.CharField):
     """Stored as plain text in database, but not displayed in user interface."""
@@ -126,7 +126,8 @@ class MonthField(models.DateField):
   
 class PriceField(models.DecimalField):
     """
-    Deserves more documentation.
+    A Decimalfield with default values for decimal_places, max_length and max_digits.
+    
     """
     def __init__(self, *args, **kwargs):
         defaults = dict(
@@ -181,12 +182,14 @@ class NullCharField(models.CharField): #subclass the CharField
        else:
             return value # otherwise, just pass the value        
 
+
 class FakeField(object):
     """
     Base class for 
     :class:`RemoteField`
     :class:`DisplayField`
     """
+    model = None
     choices = None
     primary_key = False
     editable = False
@@ -197,17 +200,19 @@ class FakeField(object):
     preferred_height = 3
     max_digits = None
     decimal_places = None
+    default = NOT_PROVIDED
     
     def is_enabled(self,lh):
         """
         Overridden by mti.EnableChild
         """
-        return False
+        #~ return False
         #~ return True
-        #~ return not self.editable
+        return self.editable
         
     def has_default(self):
-        return False
+        return self.default is not NOT_PROVIDED
+        
         
 
 #~ class NullField(FakeField):
@@ -336,6 +341,9 @@ class VirtualField(FakeField): # (Field):
               return self.total_excl + self.total_vat
         """
             
+    def override_getter(self,get):
+        self.get = get
+        
     def attach_to_model(self,model,name):
         self.model = model
         self.name = name
@@ -346,6 +354,9 @@ class VirtualField(FakeField): # (Field):
         model._meta.add_virtual_field(self)
         #~ logger.info('20120831 VirtualField %s.%s',full_model_name(model),name)
         
+    def __repr__(self):
+        return "%s %s.%s" % (self.__class__.__name__,self.model,self.name)
+        
     def lino_resolve_type(self):
         """
         Unlike attach_to_model, this is also called on virtual 
@@ -363,8 +374,8 @@ class VirtualField(FakeField): # (Field):
         #~ self.return_type.name = self.name
         if isinstance(self.return_type,models.ForeignKey):
             f = self.return_type
+            f.rel.to = resolve_model(f.rel.to)
             if f.verbose_name is None:
-                f.rel.to = resolve_model(f.rel.to)
                 #~ if f.name is None:
                 f.verbose_name = f.rel.to._meta.verbose_name
                     #~ from lino.core.kernel import set_default_verbose_name
@@ -387,6 +398,12 @@ class VirtualField(FakeField): # (Field):
         #~ self._lino_atomizer = store.create_field(self,self.name)
         store.get_atomizer(self,self.name)
         
+    def get_default(self):
+        return self.return_type.get_default()
+        #~ 
+    def has_default(self):
+        return self.return_type.has_default()
+        
       
     def unused_contribute_to_class(self, cls, name):
         ## if defined in abstract base class, called once on each submodel
@@ -950,73 +967,6 @@ class RecurrenceField(models.CharField):
         models.CharField.__init__(self,*args,**kw)
       
 
-def get_data_elem(model,name):
-    #~ logger.info("20120202 get_data_elem %r,%r",model,name)
-    if not name.startswith('__'):
-        parts = name.split('__')
-        if len(parts) > 1:
-            """It's going to be a RemoteField
-            """
-            # logger.warning("20120406 RemoteField %s in %s",name,self)
-            #~ model = self.model
-
-            from lino.ui import store
-            
-            field_chain = []
-            for n in parts:
-                assert model is not None
-                #~ 20130508 model.get_default_table().get_handle() # make sure that all atomizers of those fields get created.
-                fld = get_data_elem(model,n)
-                if fld is None:
-                    # raise Exception("Part %s of %s got None" % (n,model))
-                    raise Exception(
-                        "Invalid RemoteField %s.%s (no field %s in %s)" % 
-                        (full_model_name(model),name,n,full_model_name(model)))
-                store.get_atomizer(fld,fld.name) # make sure that the atomizer gets created.
-                field_chain.append(fld)
-                if fld.rel:
-                    model = fld.rel.to
-                else:
-                    model = None
-            def func(obj,ar=None):
-                #~ print '20130422',name,obj, [fld.name for fld in field_chain]
-                try:
-                    for fld in field_chain:
-                        #~ obj = fld.value_from_object(obj)
-                        obj = fld._lino_atomizer.full_value_from_object(obj,ar)
-                    #~ for n in parts:
-                        #~ obj = getattr(obj,n)
-                    #~ print '20130422 %s --> %r', fld.name,obj
-                    return obj
-                except Exception,e:
-                    if False: # only for debugging
-                        logger.exception(e)
-                        return str(e) 
-                    return None
-            return RemoteField(func,name,fld)
-    
-    try:
-        return model._meta.get_field(name)
-    except models.FieldDoesNotExist,e:
-        pass
-        
-    #~ s = name.split('.')
-    #~ if len(s) == 1:
-        #~ mod = import_module(model.__module__)
-        #~ rpt = getattr(mod,name,None)
-    #~ elif len(s) == 2:
-        #~ mod = getattr(settings.SITE.modules,s[0])
-        #~ rpt = getattr(mod,s[1],None)
-    #~ else:
-        #~ raise Exception("Invalid data element name %r" % name)
-    
-    v = get_class_attr(model,name)
-    if v is not None: return v
-    
-    for vf in model._meta.virtual_fields:
-        if vf.name == name:
-            return vf
-
 
 def fields_list(model,field_names):
     """
@@ -1039,12 +989,11 @@ def fields_list(model,field_names):
     #~ return [get_field(model,n).name for n in field_names.split()]
     lst = []
     for name in field_names.split():
-        e = get_data_elem(model,name)
+        e = model.get_data_elem(name)
         if e is None:
             raise models.FieldDoesNotExist("No data element %r in %s" % (name,model))
         lst.append(e.name)
     return lst
-    #~ return [get_data_elem(model,n).name for n in field_names.split()]
 
 
 def ForeignKey(othermodel,*args,**kw):
