@@ -50,6 +50,7 @@ from django.db.models.fields.related import SingleRelatedObjectDescriptor
 #~ from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes import generic
 
+from lino.utils.xmlgen.html import E
 
 from lino.core import constants # as ext_requests
 
@@ -575,6 +576,9 @@ add_tabpanel() on %s horizontal 'main' panel %r."""
             #~ print 20120525, self, self.detail_layout._element_options
             
             
+    #~ def before_setup_handle(self):
+        #~ pass
+
             
     def get_layout_handle(self,ui):
         """
@@ -595,6 +599,7 @@ add_tabpanel() on %s horizontal 'main' panel %r."""
         # we do not want any inherited handle
         h = self.__dict__.get(hname,None)
         if h is None:
+            #~ self.before_setup_handle()
             #~ print 20130404, self._handle_class
             h = self._handle_class(self)
             setattr(self,hname,h)
@@ -613,17 +618,26 @@ add_tabpanel() on %s horizontal 'main' panel %r."""
           self._datasource.__name__,
           field.name,**kw)
         
-
         
             
-class FormLayout(BaseLayout):
+class FieldLayout(BaseLayout):
+    pass
+    
+    #~ def before_setup_handle(self):
+        #~ logger.info("20130804 before_setup_handle %s",self)
+        #~ ds = settings.SITE.modules.resolve(str(self._datasource))
+        #~ if self._datasource is not ds:
+            #~ logger.info("20130804 override _datasource of %r by %r",self._datasource,ds)
+            #~ self._datasource = ds
+        
+    
+class FormLayout(FieldLayout):
     """
     A Layout description for the main panel of a DetailWindow or InsertWindow.
     """
     join_str = "\n"
     
-        
-class ListLayout(BaseLayout):
+class ListLayout(FieldLayout):
     """
     A Layout description for the columns of a GridPanel.
     """
@@ -642,7 +656,7 @@ class ParamsLayout(BaseLayout):
     url_param_name = constants.URL_PARAM_PARAM_VALUES
     params_store = None
 
-    def get_data_elem(self,name): 
+    def get_data_elem(self,name):
         return self._datasource.get_param_elem(name)
         
     def setup_handle(self,lh):
@@ -832,7 +846,7 @@ def create_layout_element(lh,name,**kw):
                 field.help_text = de.help_text
                 field.name = de.__name__
                 #~ field._return_type_for_method = de.slave_as_summary_meth(self,'<br>')
-                field._return_type_for_method = de.slave_as_summary_meth('<br>')
+                field._return_type_for_method = de.slave_as_summary_meth(E.br)
                 lh.add_store_field(field)
                 e = ext_elems.HtmlBoxElement(lh,field,**kw)
                 return e
