@@ -31,9 +31,11 @@ from django.core.exceptions import ValidationError
 from lino.core import fields
 from lino.core import signals
 from lino.core import dbutils
+from lino.core.actions import InstanceAction
+#~ from lino.core.actions import PdfAction
 from djangosite.dbutils import obj2str, full_model_name
-from lino.utils.xmlgen import html as xghtml
-E = xghtml.E
+from lino.utils.xmlgen.html import E
+from lino.utils import get_class_attr
 
 
 class Model(models.Model):
@@ -46,6 +48,8 @@ class Model(models.Model):
     class Meta:
         abstract = True
         
+    #~ as_pdf = PdfAction()
+        
     allow_cascaded_delete = []
     """
     A list of names of ForeignKey fields of this model 
@@ -77,6 +81,8 @@ class Model(models.Model):
     
     """
     
+    #~ help_text = None
+    
     quick_search_fields = None
     """
     When quick_search text is given for a table on this model, 
@@ -158,7 +164,76 @@ class Model(models.Model):
     def as_list_item(self,ar):
         return E.li(unicode(self))
         
+    @classmethod
+    def get_data_elem(model,name):
+        #~ logger.info("20120202 get_data_elem %r,%r",model,name)
+        if not name.startswith('__'):
+            parts = name.split('__')
+            if len(parts) > 1:
+                """It's going to be a RemoteField
+                """
+                # logger.warning("20120406 RemoteField %s in %s",name,self)
+                #~ model = self.model
+
+                from lino.ui import store
+                
+                field_chain = []
+                for n in parts:
+                    assert model is not None
+                    #~ 20130508 model.get_default_table().get_handle() # make sure that all atomizers of those fields get created.
+                    fld = model.get_data_elem(n)
+                    if fld is None:
+                        # raise Exception("Part %s of %s got None" % (n,model))
+                        raise Exception(
+                            "Invalid RemoteField %s.%s (no field %s in %s)" % 
+                            (full_model_name(model),name,n,full_model_name(model)))
+                    store.get_atomizer(fld,fld.name) # make sure that the atomizer gets created.
+                    field_chain.append(fld)
+                    if fld.rel:
+                        model = fld.rel.to
+                    else:
+                        model = None
+                def func(obj,ar=None):
+                    #~ if ar is None: raise Exception(20130802)
+                    #~ print '20130422',name,obj, [fld.name for fld in field_chain]
+                    try:
+                        for fld in field_chain:
+                            #~ obj = fld.value_from_object(obj)
+                            obj = fld._lino_atomizer.full_value_from_object(obj,ar)
+                        #~ for n in parts:
+                            #~ obj = getattr(obj,n)
+                        #~ print '20130422 %s --> %r', fld.name,obj
+                        return obj
+                    except Exception,e:
+                        #~ if False: # only for debugging
+                        if True: # see 20130802
+                            logger.exception(e)
+                            return str(e) 
+                        return None
+                return fields.RemoteField(func,name,fld)
         
+        try:
+            return model._meta.get_field(name)
+        except models.FieldDoesNotExist,e:
+            pass
+            
+        #~ s = name.split('.')
+        #~ if len(s) == 1:
+            #~ mod = import_module(model.__module__)
+            #~ rpt = getattr(mod,name,None)
+        #~ elif len(s) == 2:
+            #~ mod = getattr(settings.SITE.modules,s[0])
+            #~ rpt = getattr(mod,s[1],None)
+        #~ else:
+            #~ raise Exception("Invalid data element name %r" % name)
+        
+        v = get_class_attr(model,name)
+        if v is not None: return v
+        
+        for vf in model._meta.virtual_fields:
+            if vf.name == name:
+                return vf
+
     
     def get_choices_text(self,request,actor,field):
         """
@@ -225,7 +300,7 @@ class Model(models.Model):
         They remain in the database but are not visible in the user interface.
         """
         for name in names:
-            if fields.get_data_elem(self,name) is None:
+            if self.get_data_elem(name) is None:
                 raise Exception("%s has no element '%s'" % (self,name))
         self.hidden_elements = self.hidden_elements | set(names)
         
@@ -383,42 +458,7 @@ class Model(models.Model):
         pass
   
     def summary_row(self,ar,**kw):
-        return ar.href_to(self)
-        
-    #~ @fields.displayfield(_("ACtions"))
-    #~ def action_buttons(obj,ar):
-        #~ l = []
-        #~ for a in ar.actor.get_custom_actions(ar,obj):
-            #~ l.append(ar.renderer.action_button(obj,ar,a))
-            #~ l.append(' ')
-        #~ return xghtml.E.p(*l)
-      
-    @fields.displayfield(_("Workflow"))
-    def workflow_buttons(obj,ar):
-        """
-        Displays the workflow buttons for this row and this user.
-        """
-        #~ logger.info('20120930 workflow_buttons %r', obj)
-        actor = ar.actor
-        #~ print 20120621 , actor,  self
-        #~ print 20120618, ar
-        l = []
-        state = actor.get_row_state(obj)
-        if state:
-            #~ l.append(xghtml.E.b(unicode(state),style="vertical-align:middle;"))
-            l.append(E.b(unicode(state)))
-            #~ l.append(u" » ")
-            #~ l.append(u" \u25b8 ")
-            #~ l.append(u" \u2192 ")
-        #~ sep = u" \u25b8 "
-        sep = u" \u2192 "
-        for a in ar.actor.get_workflow_actions(ar,obj):
-            l.append(sep)
-            l.append(ar.renderer.action_button(obj,ar,a))
-            sep = ' '
-            #~ l.append(' ')
-        #~ return ', '.join(l)
-        return E.p(*l)
+        yield ar.obj2html(self)
         
         
     def __repr__(self):
@@ -466,7 +506,7 @@ class Model(models.Model):
             
     def to_html(self,**kw):
         import lino.ui.urls # hack: trigger ui instantiation
-        actor=self.get_default_table()
+        actor = self.get_default_table()
         kw.update(renderer=settings.SITE.ui.text_renderer)
         #~ ar = settings.SITE.ui.text_renderer.request(**kw)
         ar = actor.request(**kw)
@@ -478,12 +518,14 @@ class Model(models.Model):
         """
         Used when implementing :ref:`polymorphism`.
         """
-        assert model is self.__class__
+        #~ assert model is self.__class__
         return self
         
         
     LINO_MODEL_ATTRIBS = (
+              #~ 'as_pdf',
               'get_row_permission',
+              'get_data_elem',
               'after_ui_save',
               #~ 'update_system_note',
               'preferred_foreignkey_width',
@@ -503,6 +545,7 @@ class Model(models.Model):
               'get_system_note_recipients',
               'get_system_note_type',
               'quick_search_fields',
+              #~ 'help_text',
               'change_watcher_spec',
               #~ 'site_setup',
               'on_analyze',
