@@ -20,7 +20,8 @@ logger = logging.getLogger(__name__)
 import traceback
 #~ from django.utils.translation import ugettext as _
 from django.utils.translation import ugettext_lazy as _
-from north.dbutils import set_language
+from django.utils import translation
+#~ from north.dbutils import set_language
 from django.utils.encoding import force_unicode
 from django.conf import settings
 from django import http
@@ -80,7 +81,7 @@ class BaseRequest(object):
     
     This holds information like the current user and renderer.
     A bare BaseRequest instance is returned as a "session" by 
-    :meth:`login <lino.ui.Site.login>`.
+    :meth:`login <lino.site.Site.login>`.
     """
     renderer = None
     
@@ -142,6 +143,11 @@ class BaseRequest(object):
         kw.update(user=request.user)
         kw.update(subst_user=request.subst_user)
         kw.update(requesting_panel=request.requesting_panel)
+        kw.update(current_project=rqdata.get(ext_requests.URL_PARAM_PROJECT,None))
+        
+        selected = rqdata.getlist(ext_requests.URL_PARAM_SELECTED)
+        kw.update(selected_rows = [self.actor.get_row_by_pk(pk) for pk in selected])
+        
         #~ if settings.SITE.user_model:
             #~ username = rqdata.get(ext_requests.URL_PARAM_SUBST_USER,None)
             #~ if username:
@@ -155,10 +161,14 @@ class BaseRequest(object):
     def setup(self,
             user=None,
             subst_user=None,
+            current_project=None,
+            selected_rows=None,
             requesting_panel=None,
             renderer=None):
         self.requesting_panel = requesting_panel
         self.user = user 
+        self.current_project = current_project
+        self.selected_rows = selected_rows
         if renderer is not None:
             self.renderer = renderer
         #~ if self.actor.parameters:
@@ -245,10 +255,11 @@ class BaseRequest(object):
         msg.send()
         logger.info("System note '%s' from %s has been sent to %s",subject,sender,recipients)
         
-    def spawn(self,spec,language=None,**kw):
+    def spawn(self,spec,**kw):
         """
         Create a new ActionRequest, taking an "action specificator" 
-        and using default values from this one (via :meth:setup_from`).
+        and using default values from this one.
+        
         """
         if isinstance(spec,ActionRequest):
             for k,v in kw.items():
@@ -257,15 +268,16 @@ class BaseRequest(object):
             #~ if kw:
                 #~ ar = ar.spawn(**kw)
                 #~ raise Exception(20130327)
+            spec.setup_from(self)
         else:
             from lino.core.menus import create_item
             mi = create_item(spec)
             #~ print 20130425, __file__, mi.bound_action
+            kw.setdefault('user',self.user)
+            kw.setdefault('subst_user',self.subst_user)
+            kw.setdefault('renderer',self.renderer)
+            kw.setdefault('requesting_panel',self.requesting_panel)
             spec = mi.bound_action.request(**kw)
-        spec.setup_from(self)
-        if language:
-            spec.set_language(language)
-        
         #~ ar.user = self.user
         #~ ar.subst_user = self.subst_user
         #~ ar.renderer = self.renderer
@@ -273,46 +285,57 @@ class BaseRequest(object):
         
         
 
-    def old_spawn(self,actor,*args,**kw):
+        
+    def run(self,thing,*args,**kw):
         """
-        Create a new ActionRequest, taking default values from this one.
+        The first parameter `thing` may be an InstanceAction or a 
+        Model instance.
         """
-        kw.setdefault('user',self.user)
-        kw.setdefault('subst_user',self.subst_user)
-        kw.setdefault('requesting_panel',self.requesting_panel)
-        kw.setdefault('renderer',self.renderer)
-        #~ kw.setdefault('request',self.request) 
-        # removed 20120702 because i don't want to inherit quick_search from spawning request
-        # and because i couldn't remember why 'request' was passed to the spawned request.
-        #~ return self.ui.request(actor,**kw)
-        return actor.request(*args,**kw)
+        return thing.run_from_session(self,*args,**kw)
         
-    #~ def show(self,*args,**kw):
-        #~ print self.to_rst(*args,**kw)
+    #~ def set_language(self,*args):
+        #~ set_language(*args)
         
-    def run(self,ia,*args,**kw):
-        return ia.run_from_session(self,*args,**kw)
-        
-    def set_language(self,*args):
-        set_language(*args)
-        
-        
-    def show(self,spec,column_names=None,**kw):
+    def show(self,spec,master_instance=None,column_names=None,language=None,**kw):
+        # 20130905 added master_instance positional arg. but finally didn't use it.
+        if master_instance is not None:
+            kw.update(master_instance=master_instance)
         ar = self.spawn(spec,**kw)
         #~ print ar.to_rst(column_names)
-        return ar.renderer.show(ar,column_names)
+        if language:
+            #~ spec.set_language(language)
+            with translation.override(language):
+                return ar.renderer.show(ar,column_names=column_names)
+        return ar.renderer.show(ar,column_names=column_names)
         
-    def summary_row(self,obj,**kw):
-        return obj.summary_row(self,**kw)
-            
+    def summary_row(self,obj,**kw): return obj.summary_row(self,**kw)
     def instance_handler(self,*args,**kw): return self.renderer.instance_handler(self,*args,**kw)
-    def href_to(self,*args,**kw): return self.renderer.href_to(self,*args,**kw)
+    #~ def href_to(self,*args,**kw): return self.renderer.href_to(self,*args,**kw)
     def pk2url(self,*args,**kw): return self.renderer.pk2url(self,*args,**kw)
     def get_request_url(self,*args,**kw): return self.renderer.get_request_url(self,*args,**kw)
     def obj2html(self,*args,**kw): return self.renderer.obj2html(self,*args,**kw)
     def href_to_request(self,*args,**kw): return self.renderer.href_to_request(self,*args,**kw)
     def window_action_button(self,*args,**kw): return self.renderer.window_action_button(self,*args,**kw)
     def row_action_button(self,obj,a,*args,**kw): return self.renderer.row_action_button(obj,self.request,a,*args,**kw)
+    def instance_action_button(self,ai,*args,**kw): 
+        return self.renderer.row_action_button(ai.instance,self.request,ai.bound_action,*args,**kw)
+    def action_button(self,ba,obj,*args,**kw): return self.renderer.action_button(obj,self,ba,*args,**kw)
+    def insert_button(self,*args,**kw): return self.renderer.insert_button(self,*args,**kw)
+    
+    def as_button(self,*args,**kw): 
+        """
+        Return a button which when activated would execute (a copy of) this request.
+        """
+        return self.renderer.action_button(None,self,self.bound_action,*args,**kw)
+    
+    def goto_instance(self,obj,**kw):
+        #~ kw.update(refresh=True)
+        #~ kw.update(goto_record=('sales.Invoices',invoice.pk))
+        #~ kw.update(goto_record=invoice)
+        js = self.instance_handler(obj)
+        kw.update(eval_js=js)
+        return kw
+    
 
       
 class ActionRequest(BaseRequest):
@@ -334,7 +357,11 @@ class ActionRequest(BaseRequest):
     limit = None
     order_by = None
     
-    def __init__(self,actor=None,request=None,action=None,renderer=None,param_values=None,**kw):
+    def __init__(self,actor=None,
+        request=None,action=None,renderer=None,
+        param_values=None,
+        action_param_values=None,
+        **kw):
         """
         An ActionRequest is instantiated from different shortcut methods:
         
@@ -404,17 +431,15 @@ class ActionRequest(BaseRequest):
         if self.bound_action.action.parameters is not None:
             apv = self.bound_action.action.action_param_defaults(self,None)
             if request is not None:
-                #~ pv.update(self.ui.parse_params(self.ah,request))
-                #~ pv.update(self.ah.store.parse_params(request))
                 apv.update(self.bound_action.action.params_layout.params_store.parse_params(request))
             #~ logger.info("20130122 action_param_defaults() returned %s",apv)
+            if action_param_values is not None:
+                for k in action_param_values.keys(): 
+                    if not apv.has_key(k):
+                        raise Exception("Invalid key '%s' in action_param_values of %s request (possible keys are %s)" % (k,actor,apv.keys()))
+                apv.update(action_param_values)
             self.action_param_values = AttrDict(**apv)
             
-            #~ if param_values:
-                #~ # logger.info("20120608 param_values is %s",param_values)
-                #~ for k,v in param_values.items():
-                    #~ self.param_values.define(k,v)
-                    
         self.bound_action.setup_action_request(self)
                 
         
@@ -497,23 +522,25 @@ class ActionRequest(BaseRequest):
     #~ def get_request_url(self,*args,**kw):
         #~ return self.ui.get_request_url(self,*args,**kw)
 
-    def get_action_status(self,ba,obj,**kw):
-        #~ logger.info("get_action_status %s",ba.full_name())
-        if ba.action.parameters:
-            if ba.action.params_layout.params_store is None:
-                raise Exception("20121016 %s has no store" % ba.action.params_layout)
-            kw.update(field_values=ba.action.params_layout.params_store.pv2dict(
-                settings.SITE.ui,ba.action.action_param_defaults(self,obj)))
-        return kw
       
-      
-    def get_status(self,ui,**kw):
+    def get_status(self,**kw):
         if self.actor.parameters:
             #~ kw.update(param_values=self.ah.store.pv2dict(ui,self.param_values))
             #~ lh = self.actor.params_layout.get_layout_handle(ui)
-            kw.update(param_values=self.actor.params_layout.params_store.pv2dict(ui,self.param_values))
+            kw.update(param_values=self.actor.params_layout.params_store.pv2dict(self.param_values))
+            
+        if self.bound_action.action.parameters is not None:
+            pv = self.bound_action.action.params_layout.params_store.pv2dict(self.action_param_values)
+            kw.update(field_values=pv)
             
         bp = kw.setdefault('base_params',{})
+        
+        if self.current_project is not None:
+            bp[ext_requests.URL_PARAM_PROJECT] = self.current_project
+            
+        #~ if self.selected_rows is not None:
+            #~ bp[ext_requests.URL_PARAM_SELECTED] = [obj.pk for obj in self.selected_rows if obj is not None]
+            
         if self.subst_user is not None:
             #~ bp[ext_requests.URL_PARAM_SUBST_USER] = self.subst_user.username
             bp[ext_requests.URL_PARAM_SUBST_USER] = self.subst_user.id
@@ -539,9 +566,10 @@ class ActionRequest(BaseRequest):
         location = ar.get_request_url()
         return self.request.build_absolute_uri(location)
         
-        
     def to_rst(self,*args,**kw):
         """
         Returns a string representing this request in reStructuredText markup.
         """
         return self.actor.to_rst(self,*args,**kw)
+
+
