@@ -77,15 +77,18 @@ from lino.core.requests import ActionRequest
 
 from lino.ui import base
 
-from lino.utils.appy_pod import Renderer
+#~ from lino.utils.appy_pod import Renderer
+from lino.utils import jsgen
 
+from lino.utils.xmlgen import html as xghtml
+from lino.utils.xmlgen.html import E
+from lino.utils.appy_pod import PrintTableAction, PortraitPrintTableAction
 
 class InvalidRequest(Exception):
     pass
 
 from lino.utils.xmlgen.html import RstTable
 
-MAX_ROW_COUNT = 900
 
 if False: # 20130710
 
@@ -153,7 +156,6 @@ if False: # 20130710
         
         
 
-
 WARNINGS_LOGGED = dict()
 
 
@@ -193,7 +195,21 @@ class TableRequest(ActionRequest):
     def execute(self):
         #~ if self.actor.parameters:
             #~ logger.info("20121203 TableRequest.execute() %s",self.param_values)
-        self._data_iterator = self.get_data_iterator()
+            
+        try:
+            self._data_iterator = self.get_data_iterator()
+        except Warning,e:
+            #~ logger.info("20130809 Warning %s",e)
+            self.no_data_text = unicode(e)
+            self._data_iterator = []
+        except Exception,e:
+            self.no_data_text = unicode(e)
+            w = WARNINGS_LOGGED.get(str(e))
+            if w is None:
+                WARNINGS_LOGGED[str(e)] = True
+                logger.exception(e)
+            self._data_iterator = []
+            
         self._sliced_data_iterator = self._data_iterator
         if self.offset is not None:
             self._sliced_data_iterator = self._sliced_data_iterator[self.offset:]
@@ -224,24 +240,12 @@ class TableRequest(ActionRequest):
     def get_data_iterator(self):
         if self.actor.get_data_rows is not None:
             l = []
-            try:
-                rows = self.actor.get_data_rows(self)
-                for row in rows:
-                    #~ if len(l) > 300:
-                        #~ raise Exception("20120521 More than 300 items in %s" % 
-                            #~ unicode(rows))
-                    group = self.actor.group_from_row(row)
-                    group.process_row(l,row)
-            except Warning,e:
-                self.no_data_text = unicode(e)
-                return []
-            except Exception,e:
-                self.no_data_text = unicode(e)
-                w = WARNINGS_LOGGED.get(str(e))
-                if w is None:
-                    WARNINGS_LOGGED[str(e)] = True
-                    logger.exception(e)
-                return []
+            for row in self.actor.get_data_rows(self):
+                #~ if len(l) > 300:
+                    #~ raise Exception("20120521 More than 300 items in %s" % 
+                        #~ unicode(rows))
+                group = self.actor.group_from_row(row)
+                group.process_row(l,row)
             return l
         #~ logger.info("20120914 tables.get_data_iterator %s",self)
         #~ logger.info("20120914 tables.get_data_iterator %s",self.actor)
@@ -361,7 +365,7 @@ class TableRequest(ActionRequest):
         if offset:
             kw.update(offset=int(offset))
         #~ limit = rqdata.get(constants.URL_PARAM_LIMIT,None)
-        limit = rqdata.get(constants.URL_PARAM_LIMIT,15) 
+        limit = rqdata.get(constants.URL_PARAM_LIMIT,self.actor.preview_limit) 
         if limit:
             kw.update(limit=int(limit))
         
@@ -448,9 +452,73 @@ class TableRequest(ActionRequest):
         if limit is not None:
             self.limit = limit
             
+    def table2xhtml(self,**kw):
+        t = xghtml.Table()
+        self.dump2html(t,self.sliced_data_iterator,**kw)
+        return t.as_element()
     
-    def table2xhtml(self):
-        return settings.SITE.ui.table2xhtml(self)
+    #~ def table2xhtml(self):
+        #~ return settings.SITE.ui.table2xhtml(self)
+
+    def dump2html(ar,tble,data_iterator,column_names=None):
+        """
+        Render this to html
+        """
+        tble.attrib.update(cellspacing="3px",bgcolor="#ffffff", width="100%")
+        #~ tble.attrib.update(cellspacing="3px",bgcolor="#d0def0", width="100%")
+        
+        grid = ar.ah.list_layout.main
+        columns = grid.columns
+        fields, headers, cellwidths = ar.get_field_info(column_names)
+        columns = fields
+        #~ print 20130330, cellwidths
+          
+        if ar.renderer.is_interactive and ar.master_instance is None:
+            #~ print 20130527, ar.order_by
+            for i,e in enumerate(columns):
+                if e.sortable and ar.order_by != [e.name]:
+                    kw = {constants.URL_PARAM_SORT:e.name}
+                    url = ar.renderer.get_request_url(ar,**kw)
+                    if url is not None:
+                        headers[i] = xghtml.E.a(headers[i],href=url)
+        
+        #~ cellattrs = dict(align="center",valign="middle",bgcolor="#eeeeee")
+        cellattrs = dict(align="left",valign="top",bgcolor="#eeeeee")
+        #~ cellattrs = dict(align="left",valign="top",bgcolor="#d0def0")
+        #~ cellattrs = dict()
+        
+        headers = [x for x in grid.headers2html(ar,columns,headers,**cellattrs)]
+        sums  = [fld.zero for fld in columns]
+        #~ hr = tble.add_header_row(*headers,**cellattrs)
+        if cellwidths:
+            for i,td in enumerate(headers): 
+                td.attrib.update(width=str(cellwidths[i]))
+        tble.head.append(xghtml.E.tr(*headers))
+        #~ print 20120623, ar.actor
+        recno = 0
+        for obj in data_iterator:
+            cells = ar.row2html(recno,columns,obj,sums,**cellattrs)
+            if cells is not None:
+                recno += 1
+                #~ ar.actor.apply_row_format(tr,recno)
+                tble.body.append(xghtml.E.tr(*cells))
+            
+        if recno == 0:
+            tble.clear()
+            tble.body.append(ar.no_data_text)
+        
+        if not ar.actor.hide_sums:
+            has_sum = False
+            for i in sums:
+                if i:
+                    has_sum = True
+                    break
+            if has_sum:
+                cells = ar.sums2html(columns,sums,**cellattrs)
+                tble.body.append(xghtml.E.tr(*cells))
+                #~ tble.add_body_row(*ar.ah.store.sums2html(ar,fields,sums,**cellattrs))
+            
+        
         
     def get_field_info(ar,column_names=None):
         """
@@ -460,6 +528,10 @@ class TableRequest(ActionRequest):
         Also apply the tables's :meth:`override_column_headers 
         <lino.core.actors.Actor.override_column_headers>` method.
         """
+        u = ar.get_user()
+        if u is not None:
+            jsgen.set_for_user_profile(u.profile)
+        
         if ar.request is None:
             columns = None
         else:
@@ -502,8 +574,14 @@ class TableRequest(ActionRequest):
                 #~ ah = ar.actor.get_handle(self.extjs_ui)
                 ah = ar.actor.get_request_handle(ar)
                 columns = ah.list_layout.main.columns
-                #~ columns = ar.ah.list_layout.main.columns
-            #~ fields = ar.ah.store.list_fields
+                
+            # render them so that babelfields in hidden_languages get hidden:
+            for e in columns: 
+                e.value = e.ext_options()
+                #~ e.js_value() 
+                
+            columns = [e for e in columns if not e.value.get('hidden',False)]
+            
             headers = [unicode(col.label or col.name) for col in columns]
             widths = ["%d" % (col.width or col.preferred_width) for col in columns]
             #~ 20130415 widths = ["%d%%" % (col.width or col.preferred_width) for col in columns]
@@ -520,42 +598,56 @@ class TableRequest(ActionRequest):
             
         return fields, headers, widths
         
-        
-    def appy_render(ar,target_file):
-        
-        from lino.utils.config import find_config_file
-        
-        if ar.get_total_count() > MAX_ROW_COUNT:
-            raise Exception(_("List contains more than %d rows") % MAX_ROW_COUNT)
-    
-        tpl_leaf = "Table.odt" 
-        tplgroup = None
-        tplfile = find_config_file(tpl_leaf,tplgroup)
-        if not tplfile:
-            raise Exception("No file %s / %s" % (tplgroup,tpl_leaf))
+    def row2html(self,recno,columns,row,sums,**cellattrs):
+        #~ logger.info("20130123 row2html %s",fields)
+        #~ for i,fld in enumerate(self.list_fields):
+        has_numeric_value = False
+        cells = []
+        for i,col in enumerate(columns):
+            #~ if fld.name == 'person__gsm':
+            #~ logger.info("20120406 Store.row2list %s -> %s", fld, fld.field)
+            #~ import pdb; pdb.set_trace()
+            v = col.field._lino_atomizer.full_value_from_object(row,self)
+            if v is None:
+                td = E.td(**cellattrs)
+            else:
+                nv = col.value2num(v)
+                if nv != 0:
+                    try:
+                        sums[i] += nv
+                    except TypeError as e:
+                        raise Exception("Cannot compute %r + %r" % (sums[i],nv))
+                    has_numeric_value = True
+                td = col.value2html(self,v,**cellattrs)
+            col.apply_cell_format(td)
+            self.actor.apply_cell_format(self,row,col,recno,td)
+            cells.append(td)
+        if self.actor.hide_zero_rows and not has_numeric_value:
+            return None
+        return cells
+            
+    def row2text(self,fields,row,sums):
+        for i,fld in enumerate(fields):
+            if fld.field is not None:
+                try: # was used to find bug 20130422
+                    v = fld.field._lino_atomizer.full_value_from_object(row,self)
+                except Exception as e:
+                    v = "%s:\n%s" % (fld.field,e)
+                if v is None:
+                    yield ''
+                else:
+                    sums[i] += fld.value2num(v)
+                    yield fld.format_value(self,v)
+                
+    def sums2html(self,columns,sums,**cellattrs):
+        #~ return [fld.format_sum(self,sums,i)
+          #~ for i,fld in enumerate(fields)]
+        return [fld.sum2html(self,sums,i,**cellattrs)
+          for i,fld in enumerate(columns)]
+      
+        #~ return [fld.sum2html(self.ui,sums[i])
+          #~ for i,fld in enumerate(fields)]
             
-        ar.renderer = settings.SITE.ui.ext_renderer # 20120624
-        
-        context = dict(
-            ar=ar,
-            title=unicode(ar.get_title()),
-            dtos=dbutils.dtos,
-            dtosl=dbutils.dtosl,
-            dtomy=dbutils.dtomy,
-            babelattr=dbutils.babelattr,
-            babelitem=dbutils.babelitem,
-            tr=dbutils.babelitem,
-            settings=settings,
-            _ = _,
-            #~ knowledge_text=fields.knowledge_text,
-            )
-        if os.path.exists(target_file):
-            os.remove(target_file)
-        logger.info(u"appy.pod render %s -> %s (params=%s",
-            tplfile,target_file,settings.SITE.appy_params)
-        renderer = Renderer(ar,tplfile, context, target_file,**settings.SITE.appy_params)
-        renderer.run()
-        
         
     def get_title(self):
         if self.title is not None:  
@@ -564,8 +656,8 @@ class TableRequest(ActionRequest):
         
         
         
-    def get_status(self,ui,**kw):
-        kw = ActionRequest.get_status(self,ui,**kw)
+    def get_status(self,**kw):
+        kw = ActionRequest.get_status(self,**kw)
         bp = kw['base_params']
         if self.quick_search:
             bp[constants.URL_PARAM_FILTER] = self.quick_search
@@ -575,13 +667,13 @@ class TableRequest(ActionRequest):
                 if self.actor.known_values.get(k,None) != v:
                     bp[k] = v
         if self.master_instance is not None:
-            if self.master is not None:
+            if self.master is None:
+                bp[constants.URL_PARAM_MASTER_PK] = self.master_instance
+            else:
                 bp[constants.URL_PARAM_MASTER_PK] = self.master_instance.pk
                 if ContentType._meta.installed:
                     mt = ContentType.objects.get_for_model(self.master_instance.__class__).pk
                     bp[constants.URL_PARAM_MASTER_TYPE] = mt
-            else:
-                bp[constants.URL_PARAM_MASTER_PK] = self.master_instance
         return kw
         
         
@@ -701,16 +793,25 @@ class AbstractTable(actors.Actor):
     Used internally to store :class:`groups <Group>` defined by this Table.
     """
     
-    preview_limit = None
+    #~ preview_limit = 15
+    preview_limit = settings.SITE.preview_limit
     """
     The LIMIT to use when this is being used in "preview mode", 
     e.g. as a slave table in a detail window.
-    If this is None, preview requests for this table will request all rows.
-    Since preview tables usually have no paging toolbar, that's what we want.
-    But if there are many rows, this can lead to waste of performance.
     
-    Not tested. This was one idea while trying to solve the 20130107 problem 
-    "AvailableCoaches slave table shows only 5 records".    
+    If you set this to `None`, preview requests for this table will request all rows.
+    Since preview tables usually have no paging toolbar, that's 
+    theoretically what we want (but can 
+    lead to waste of performance if there are many rows).
+    
+    The default value for this is the  
+    :attr:`preview_limit <lino.site.Site.preview_limit>`
+    class attribute of your :class:`Site <lino.site.Site>`,
+    which itself has a hard-coded default value of 15
+    and which you can override in your :xfile:`settings.py`.
+    
+    Test case and description  in :ref:`cosi.tested`.
+    
     """
     
     get_data_rows = None
@@ -767,11 +868,15 @@ class AbstractTable(actors.Actor):
     page_length = 20
     """
     Number of rows to display per page.
+    Used to control the height of a combobox of a ForeignKey 
+    pointing to this model 
     """
     
-    cell_edit = True 
+    cell_edit = True
     """
     `True` to use ExtJS CellSelectionModel, `False` to use RowSelectionModel.
+    When True, the users cannot select multiple rows.
+    When False, the users cannot select and edit individual cells.
     """
     
     show_detail_navigator = False
@@ -842,6 +947,11 @@ class AbstractTable(actors.Actor):
     
     """
     
+    if settings.SITE.is_installed('system'):
+        
+        as_pdf = PrintTableAction()
+        as_pdf_p = PortraitPrintTableAction()
+            
     
     def __init__(self,*args,**kw):
         raise NotImplementedError("20120104")
@@ -866,7 +976,7 @@ class AbstractTable(actors.Actor):
         return actions.GridEdit()
         
     @classmethod
-    def class_init(self):
+    def unused_class_init(self):
         """
         Sets table-specific default values for certain attributes.
         """
@@ -878,12 +988,14 @@ class AbstractTable(actors.Actor):
             #~ self.show_detail_navigator = False
             
         """
-        :class:`lino_welfare.modellib.debts.models.DistByBudget` 
+        :mod:`lino_welfare.modlib.debts.models`,
+        :class:`DistByBudget` 
         defines its own `get_data_rows` but inherits from 
-        :class:`lino_welfare.modellib.debts.models.EntriesByBudget` 
+        :class:`EntriesByBudget` 
         which has no `get_data_rows`.
         """
-        if self.__dict__.get('editable') is None:
+        if self.editable is None:
+        #~ if self.__dict__.get('editable') is None:
             self.editable = (self.get_data_rows is None)
             #~ logger.info("20130204 %s editable = %s",self.__name__,self.editable)
         #~ if self.editable is None:
@@ -893,6 +1005,13 @@ class AbstractTable(actors.Actor):
         super(AbstractTable,self).class_init()
       
       
+    @classmethod
+    def get_actor_editable(self):
+        if self._editable is None:
+            return (self.get_data_rows is None)
+        return self._editable
+                        
+        
         
       
     #~ @classmethod
@@ -1064,7 +1183,6 @@ class AbstractTable(actors.Actor):
         Not yet stable. Used by print_tx25.py.
         To be combined with the `show` management command.
         """
-        from north import dbutils
         dbutils.set_language(None)
         settings.SITE.startup()
         #~ settings.SITE.ui
@@ -1077,9 +1195,9 @@ class AbstractTable(actors.Actor):
             an = self.default_list_action_name
         ba = self.get_action_by_name(an)
         #~ print ba
-        ar = self.request(action=ba)
+        ar = self.request(action=ba,selected_rows=[elem])
         #~ ar = TableRequest(None,self,None,ba)
-        kw = ba.action.run_from_ui(elem,ar)
+        kw = ba.action.run_from_ui(ar)
         #~ kw = self.check_action_response(kw)
         msg = kw.get('message')
         if msg: 
@@ -1095,8 +1213,7 @@ class AbstractTable(actors.Actor):
         #~ settings.SITE.startup() 
         #~ settings.SITE.resolve_virtual_fields()
 
-        
-        grid = ar.ah.list_layout.main
+        #~ grid = ar.ah.list_layout.main
                     
         sums  = [fld.zero for fld in fields]
         rows = []  
@@ -1104,7 +1221,7 @@ class AbstractTable(actors.Actor):
         #~ for row in ar:
         for row in ar.sliced_data_iterator:
             recno += 1
-            rows.append([x for x in grid.row2text(ar,fields,row,sums)])
+            rows.append([x for x in ar.row2text(fields,row,sums)])
             #~ rows.append([x for x in grid.row2html(ar,fields,row,sums)])
                 
         has_sum = False
@@ -1114,7 +1231,7 @@ class AbstractTable(actors.Actor):
                 has_sum = True
                 break
         if has_sum:
-            rows.append([x for x in grid.sums2html(ar,fields,sums)])
+            rows.append([x for x in ar.sums2html(fields,sums)])
               
         t = RstTable(headers,**kwargs)
         return t.to_rst(rows)
