@@ -98,19 +98,21 @@ Signals:
 
 Actions:
 
-- :class:`AuthorRowAction <lino.mixins.AuthorRowAction>`
-- :class:`RowAction <lino.core.actions.RowAction>`
-- :class:`AjaxAction <lino.core.actions.RowAction>`
+- :class:`AuthorAction <lino.mixins.AuthorAction>`
+- :class:`Action <lino.core.actions.Action>`
 - :class:`ChangeStateAction <lino.core.workflows.ChangeStateAction>`
 - :class:`NotifyingAction <lino.core.actions.NotifyingAction>`
 - :class:`MergeAction <lino.core.merge.MergeAction>`
+- :class:`ShowSlaveTable <lino.core.actions.ShowSlaveTable>`
+- :class:`PrintTableAction <lino.utils.appy_pod.PrintTableAction>`
+- :class:`PrintLabelsAction <lino.utils.appy_pod.PrintLabelsAction>`
 
 Permissions:
 
-- :class:`UserProfiles <lino.utils.auth.UserProfiles>`
-- :class:`UserGroups <lino.utils.auth.UserGroups>`
-- :class:`UserLevels <lino.utils.auth.UserLevels>`
-- :func:`add_user_group` 
+- :class:`UserProfiles <lino.core.auth.UserProfiles>`
+- :class:`UserGroups <lino.core.auth.UserGroups>`
+- :class:`UserLevels <lino.core.auth.UserLevels>`
+- :func:`add_user_group <lino.core.auth.add_user_group>` 
 
 
 Workflows:
@@ -119,6 +121,22 @@ Workflows:
 - :class:`Workflow <lino.core.workflows.Workflow>`
 - :class:`State <lino.core.workflows.State>`
 
+Model mixins:
+
+- :class:`lino.mixins.ProjectRelated`
+- :class:`lino.mixins.UserAuthored`
+- :class:`lino.mixins.Sequenced`
+- :class:`lino.mixins.Hierarizable`
+- :class:`lino.mixins.printable.BasePrintable`
+- :class:`lino.mixins.printable.Printable`
+- :class:`lino.mixins.printable.PrintableType`
+- :class:`lino.mixins.printable.CachedPrintable`
+- :class:`lino.mixins.printable.TypedPrintable`
+- :class:`lino.mixins.uploadable.Uploadable`
+- :class:`lino.mixins.human.Human`
+- :class:`lino.mixins.human.Born`
+- :class:`Addressable <lino.mixins.addressable.Addressable>`
+
 Miscellaneous:
 
 - :class:`Genders <lino.core.choicelists.Genders>`
@@ -159,7 +177,7 @@ from django.conf import settings
 #~ from lino.core import table
 #~ Table = table.Table
 
-from lino.core.dbtables import summary, summary_row
+#~ from lino.core.dbtables import summary, summary_row
 
 from lino.core.frames import Frame
 from lino.core.tables import VentilatingTable
@@ -167,10 +185,12 @@ from lino.core.tables import VentilatingTable
 
 from lino.core.actions import action
 #~ from lino.core.actions import Action
-from lino.core.actions import RowAction
-AjaxAction = RowAction
+from lino.core.actions import Action
+#~ RowAction = CustomAction
+#~ Action = CustomAction
+from lino.core.actions import ShowSlaveTable
 
-from lino.mixins import AuthorRowAction
+from lino.mixins import AuthorAction
 #~ from lino.core.actions import ListAction
 from lino.core.actions import GridEdit, ShowDetailAction
 from lino.core.actions import InsertRow, DeleteSelected
@@ -208,19 +228,24 @@ from lino.core.fields import BabelTextField
 from lino.core.fields import CharField
 
 from lino.mixins import BabelNamed
+from lino.utils.appy_pod import PrintTableAction
+from lino.utils.appy_pod import PrintLabelsAction
+
+from lino.mixins.addressable import Addressable
+
 
 from north.dbutils import BabelCharField, LanguageField
 from north.dbutils import babelkw
 from north.dbutils import babelattr
 from north.dbutils import babel_values # alias for babelkw for backward compat
-from north.dbutils import set_language
+#~ from north.dbutils import set_language
 #~ from north.dbutils import BabelCharField, BabelTextField, BabelNamed, LanguageField
 #~ from lino.core.fields import MethodField
 
 from lino.utils.choosers import chooser
 from lino.utils.mti import EnableChild
 
-from lino.utils.auth import UserLevels, UserProfiles, UserGroups
+from lino.core.auth import UserLevels, UserProfiles, UserGroups, add_user_group
 
 #~ from lino.base.utils import UserLevels, UserGroups, UserProfiles
 
@@ -249,17 +274,17 @@ from lino.core.signals import pre_ui_build
 from lino.core.signals import post_ui_build
 
 from django.db.models.signals import pre_save, post_save
-from django.db.backends.signals import connection_created
-
-
+from django.db.models.signals import pre_init, post_init
 from django.db.models.signals import class_prepared
-from django.db.models.fields import NOT_PROVIDED
 
+from django.db.backends.signals import connection_created
 
 from django.dispatch import receiver
 #~ from lino.core import signals
 
 
+from django.db.models.fields import NOT_PROVIDED
+
 #~ class Module(object):
     #~ pass
 
@@ -270,21 +295,6 @@ from lino.core.inject import do_when_prepared, when_prepared
  
   
     
-def add_user_group(name,label):
-    """
-    Add a user group to the :class:`UserGroups <lino.core.perms.UserGroups>` 
-    choicelist. If a group with that name already exists, add `label` to the 
-    existing group.
-    """
-    #~ logging.info("add_user_group(%s,%s)",name,label)
-    #~ print "20120705 add_user_group(%s,%s)" % (name,unicode(label))
-    g = UserGroups.items_dict.get(name)
-    if g is None:
-        UserGroups.add_item(name,label)
-    else:
-        if g.text != label:
-            g.text += " & " + unicode(label)
-    
 from lino.core.actors import get_default_required as required
 Required = required
     
@@ -304,16 +314,21 @@ class PseudoRequest:
 
 from lino.utils import IncompleteDate
 
-from north.dbutils import dtosl
-from north.dbutils import dtos as dtos_
+from north.dbutils import fds, fdm, fdl, fdf, fdmy
+from north.dbutils import fds as fds_
 
-def dtos(d):
+def fds(d):
     """
-    Also supports IncompleteDate
+    Adds support for :class:`lino.fields.IncompleteDate`.
     """
     if isinstance(d,IncompleteDate):
-        return dtos_(d.as_date())
-    return dtos_(d)
+        return fds_(d.as_date())
+    return fds_(d)
+    
+# backward compatibility
+dtos = fds 
+from north.dbutils import dtosl
+#~ from north.dbutils import dtos as dtos_
 
     
 #~ from lino.mixins import Born
@@ -321,10 +336,10 @@ def dtos(d):
 from lino.mixins import (ProjectRelated, UserAuthored, 
     Sequenced, Hierarizable)
 
-from lino.mixins import Printable, PrintableType, CachedPrintable, TypedPrintable, DirectPrintAction
+from lino.mixins.printable import BasePrintable, Printable, PrintableType, CachedPrintable, TypedPrintable, DirectPrintAction
 #~ from lino.mixins import SimplyPrintable
 from lino.mixins.uploadable import Uploadable
-from lino.mixins.human import Human, Born, Genders
+from lino.mixins.human import Human, Born
 
 #~ from lino.utils.screenshots import register_screenshot
 
