@@ -13,7 +13,11 @@
 ## along with Lino; if not, see <http://www.gnu.org/licenses/>.
 
 """
-Calls `initdb` with :attr:`demo_fixtures <lino.ui.Site.demo_fixtures>`.
+Calls :mod:`initdb <djangosite.management.commands.initdb>` with 
+:attr:`demo_fixtures <lino.site.Site.demo_fixtures>`.
+
+Introduction see :ref:`lino.tutorial.quickstart`.
+
 """
 
 from django.conf import settings
@@ -25,7 +29,7 @@ class Command(BaseCommand):
 
     def handle(self, *args, **options):
         if len(args) > 0:
-            raise CommandError("This command takes no arguments")
+            raise CommandError("This command takes no arguments (got %r)" % args)
             
         args = settings.SITE.demo_fixtures
         if isinstance(args,basestring):
