@@ -53,6 +53,10 @@ from lino.utils.appy_pod import Renderer
 from lino.core.model import Model
 from lino.mixins.duplicable import Duplicable
 
+from lino.utils.media import MediaFile
+from lino.utils.media import TmpMediaFile
+from lino.utils.pdf import merge_pdfs
+
 
 def decfmt(v,places=2,**kw):
     """
@@ -87,8 +91,8 @@ except ImportError:
 #~ def filename_root(elem):
     #~ return elem._meta.app_label + '.' + elem.__class__.__name__
 
-def model_group(elem):
-    return elem._meta.app_label + '/' + elem.__class__.__name__
+def model_group(model):
+    return model._meta.app_label + '/' + model.__name__
 
 
 
@@ -151,24 +155,31 @@ class BuildMethod:
     def __unicode__(self):
         return unicode(self.label)
         
-    def get_target_parts(self,action,elem):
+    def get_target(self,action,elem):
         "used by `get_target_name`"
-        #~ return [self.cache_name, self.name, action.actor.elem_filename_root(elem) + self.target_ext]
-        return [self.cache_name, self.name, elem.filename_root() + self.target_ext]
-        #~ return [self.cache_name, self.name, filename_root(elem) + '-' + str(elem.pk) + self.target_ext]
+        return MediaFile(self.use_webdav,self.cache_name, self.name, elem.filename_root() + self.target_ext)
+        
         
     def get_target_name(self,action,elem):
-        "return the output filename to generate on the server"
-        if self.use_webdav and settings.SITE.use_davlink:
-            return os.path.join(settings.SITE.webdav_root,*self.get_target_parts(action,elem))
-        return os.path.join(settings.MEDIA_ROOT,*self.get_target_parts(action,elem))
-        
+        return self.get_target(action,elem).name
     def get_target_url(self,action,elem):
-        "return the url that points to the generated filename on the server"
-        if self.use_webdav and settings.SITE.use_davlink:
-            return settings.SITE.webdav_url + "/".join(self.get_target_parts(action,elem))
-        #~ return settings.MEDIA_URL + "/".join(self.get_target_parts(action,elem))
-        return settings.SITE.build_media_url(*self.get_target_parts(action,elem))
+        return self.get_target(action,elem).url
+        
+    #~ def get_target_parts(self,action,elem):
+        #~ "used by `get_target_name`"
+        #~ return [self.cache_name, self.name, elem.filename_root() + self.target_ext]
+        #~ 
+    #~ def get_target_name(self,action,elem):
+        #~ "return the output filename to generate on the server"
+        #~ if self.use_webdav and settings.SITE.use_davlink:
+            #~ return os.path.join(settings.SITE.webdav_root,*self.get_target_parts(action,elem))
+        #~ return os.path.join(settings.MEDIA_ROOT,*self.get_target_parts(action,elem))
+        #~ 
+    #~ def get_target_url(self,action,elem):
+        #~ "return the url that points to the generated filename on the server"
+        #~ if self.use_webdav and settings.SITE.use_davlink:
+            #~ return settings.SITE.webdav_url + "/".join(self.get_target_parts(action,elem))
+        #~ return settings.SITE.build_media_url(*self.get_target_parts(action,elem))
             
     def build(self,ar,action,elem):
         raise NotImplementedError
@@ -177,7 +188,9 @@ class BuildMethod:
         #~ raise NotImplementedError
         
 class DjangoBuildMethod(BuildMethod):
-
+    """
+    Using Django's templating engine.
+    """
     def get_template(self,action,elem):
         tpls = action.get_print_templates(self,elem)
         if len(tpls) == 0:
@@ -188,8 +201,8 @@ class DjangoBuildMethod(BuildMethod):
         except TemplateDoesNotExist,e:
             raise Warning("No template found for %s (%s)" % (e,tpls))
 
-    def render_template(self,elem,tpl): # ,MEDIA_URL=settings.MEDIA_URL):
-        context = dict(
+    def render_template(self,elem,tpl,**context): # ,MEDIA_URL=settings.MEDIA_URL):
+        context.update(
           instance=elem,
           title = unicode(elem),
           MEDIA_URL = settings.MEDIA_ROOT.replace('\\','/') + '/',
@@ -199,6 +212,7 @@ class DjangoBuildMethod(BuildMethod):
 class PisaBuildMethod(DjangoBuildMethod):
     """
     Generates .pdf files from .html templates.
+    Usage example see :ref:`lino.tutorials.pisa`.
     """
     name = 'pisa'
     target_ext = '.pdf'
@@ -210,7 +224,8 @@ class PisaBuildMethod(DjangoBuildMethod):
         filename = action.before_build(self,elem)
         if filename is None:
             return
-        html = self.render_template(elem,tpl) # ,MEDIA_URL=url)
+        #~ html = self.render_template(elem,tpl,request=ar.request)
+        html = self.render_template(elem,tpl,ar=ar)
         html = html.encode("utf-8")
         file(filename+'.html','w').write(html)
         
@@ -246,7 +261,7 @@ class SimpleBuildMethod(BuildMethod):
               "%s.get_print_templates() must return exactly 1 template (got %r)" % (
                 elem.__class__.__name__,tpls))
         tpl_leaf = tpls[0]
-        lang = elem.get_print_language(self)
+        lang = elem.get_print_language()
         if lang != settings.SITE.DEFAULT_LANGUAGE.django_code:
             name = tpl_leaf[:-len(self.template_ext)] + "_" + lang + self.template_ext
             from lino.utils.config import find_config_file
@@ -267,7 +282,7 @@ class SimpleBuildMethod(BuildMethod):
         tpl_leaf = self.get_template_leaf(action,elem)
         tplfile = find_config_file(tpl_leaf,self.get_group(elem))
         if not tplfile:
-            raise Warning("No file %s / %s" % (self.get_group(elem),tpl_leaf))
+            raise Warning("No file %s/%s" % (self.get_group(elem),tpl_leaf))
         #~ tplfile = os.path.normpath(os.path.join(self.templates_dir,tpl_leaf))
         return tplfile
         
@@ -303,32 +318,14 @@ class AppyBuildMethod(SimpleBuildMethod):
         When the source string contains non-ascii characters, then 
         we must convert it to a unicode string.
         """
-        def translate(s):
-            return _(s.decode('utf8'))
-        from lino import dd
-        context = dict(self=elem,
-            dtos=dd.dtos,
-            dtosl=dbutils.dtosl,
-            dtomy=dbutils.dtomy,
-            babelattr=dbutils.babelattr,
-            babelitem=dbutils.babelitem,
-            tr=dbutils.babelitem,
-            iif=iif,
-            mtos=decfmt,
-            settings=settings,
-            ar=ar,
-            #~ restify=restify,
-            #~ site_config = get_site_config(),
-            site_config=settings.SITE.site_config,
-            _ = translate,
-            #~ knowledge_text=fields.knowledge_text,
-            )
-        lang = str(elem.get_print_language(self))
+        lang = str(elem.get_print_language())
         #~ savelang = dbutils.get_language()
         #~ dbutils.set_language(lang)
         logger.info(u"appy.pod render %s -> %s (language=%r,params=%s",
             tpl,target,lang,settings.SITE.appy_params)
         def f():
+            context = elem.get_printable_context(ar)
+            context.update(self=elem)
             Renderer(ar,tpl, context, target,**settings.SITE.appy_params).run()
         dbutils.run_with_language(lang,f)
         #~ dbutils.set_language(savelang)
@@ -446,25 +443,6 @@ def get_template_choices(elem,bmname):
     from lino.utils.config import find_template_config_files
     return find_template_config_files(bm.template_ext,bm.get_group(elem))
     
-    #~ files = find_config_files('*' + bm.template_ext,bm.get_group(elem))
-    #~ l = []
-    #~ for name in files.keys():
-        #~ # ignore babel variants: 
-        #~ # e.g. ignore "foo_fr.odt" if "foo.odt" exists 
-        #~ # but don't ignore "my_template.odt"
-        #~ basename = name[:-len(bm.template_ext)]
-        #~ chunks = basename.split('_')
-        #~ if len(chunks) > 1:
-            #~ basename = '_'.join(chunks[:-1])
-            #~ if files.has_key(basename + bm.template_ext):
-                #~ continue
-        #~ l.append(name)
-    #~ l.sort()
-    #~ if not l:
-        #~ logger.warning("get_template_choices() : no matches for (%r,%r)",'*' + bm.template_ext,bm.get_group(elem))
-    #~ return l
-
-    
 def get_build_method(elem):
     bmname = elem.get_build_method()
     if not bmname:
@@ -486,7 +464,7 @@ def get_build_method(elem):
         
 
 #~ class PrintAction(actions.RedirectAction):
-class BasePrintAction(actions.RowAction):
+class BasePrintAction(actions.Action):
     """
     Base class for all "Print" actions.
     """
@@ -500,7 +478,7 @@ class BasePrintAction(actions.RowAction):
     
     #~ def __init__(self,rpt,*args,**kw):
         #~ self.actor = rpt
-        #~ actions.RowAction.__init__(self,*args,**kw)
+        #~ actions.Action.__init__(self,*args,**kw)
     
     def get_print_templates(self,bm,elem):
         return elem.get_print_templates(bm,self)
@@ -509,6 +487,7 @@ class BasePrintAction(actions.RowAction):
         """Return the target filename if a document needs to be built,
         otherwise return ``None``.
         """
+        elem.before_printable_build(bm)
         filename = bm.get_target_name(self,elem)
         if not filename:
             return
@@ -529,33 +508,60 @@ class CachedPrintAction(BasePrintAction):
     For the user they are synonyms as long as 
     Lino doesn't support server-side printing.
     """
-    
-    icon_name = 'x-tbar-print'
+    select_rows = False
+    http_method = 'POST'
+    icon_name = 'printer'
     
     def before_build(self,bm,elem):
         if elem.build_time:
             return
         return BasePrintAction.before_build(self,bm,elem)
             
-    def run_from_ui(self,elem,ar,**kw):
-        bm = get_build_method(elem)
-        if not elem.build_time:
-            t = bm.build(ar,self,elem)
-            if t is None:
-                raise Exception("%s : build() returned None?!")
-            elem.build_time = datetime.datetime.fromtimestamp(t)
-            elem.save()
+    def run_from_ui(self,ar,**kw):
+        #~ obj = ar.selected_rows[0]        
+        #~ assert obj is None
+        if len(ar.selected_rows) == 1:
+            obj = ar.selected_rows[0]
+            bm = get_build_method(obj)
+            mf = bm.get_target(self,obj)
+            
+            if obj.build_time is None:
+                obj.build_target(ar)
+                kw.update(message=_("%s printable has been built.") % obj)
+            else:
+                kw.update(message=_("Reused %s printable from cache.") % obj)
             kw.update(refresh=True)
-            kw.update(message=_("%s printable has been built.") % elem)
-        else:
-            kw.update(message=_("Reused %s printable from cache.") % elem)
-        url = bm.get_target_url(self,elem)
-        if bm.use_webdav and settings.SITE.use_davlink:
-            kw.update(open_davlink_url=ar.request.build_absolute_uri(url))
-        else:
-            kw.update(open_url=url)
-        return ar.success(**kw)
-        
+            #~ kw.update(open_url=mf.url)
+            if bm.use_webdav and settings.SITE.use_davlink and ar.request is not None:
+                kw.update(open_davlink_url=ar.request.build_absolute_uri(mf.url))
+            else:
+                kw.update(open_url=mf.url)
+            return ar.success(**kw)
+
+        def ok():
+            #~ qs = [ar.actor.get_row_by_pk(pk) for pk in ar.selected_pks]
+            mf = self.print_multiple(ar,ar.selected_rows)
+            kw.update(open_url=mf.url)
+            #~ kw.update(refresh_all=True)
+            return kw
+        msg = _("This will print %d rows.") % len(ar.selected_pks)
+        return ar.confirm(ok, msg, _("Are you sure?"))
+
+    def print_multiple(self,ar,qs):
+        pdfs = []
+        for obj in qs:
+            #~ assert isinstance(obj,CachedPrintable)
+            #~ obj.register(ar) # moved call to register to Registrable.before_printable_build
+            if obj.build_time is None:
+                obj.build_target(ar)
+            pdf = obj.get_target_name()
+            assert pdf is not None
+            pdfs.append(pdf)
+                
+        mf = TmpMediaFile(ar,'pdf')
+        settings.SITE.makedirs_if_missing(os.path.dirname(mf.name))
+        merge_pdfs(pdfs,mf.name)
+        return mf
 
 
 class EditTemplate(BasePrintAction):
@@ -564,7 +570,8 @@ class EditTemplate(BasePrintAction):
     label = _('Edit Print Template')
     required = dict(user_level='manager')
     
-    def run_from_ui(self,elem,ar,**kw):
+    def run_from_ui(self,ar,**kw):
+        elem = ar.selected_rows[0]
         bm = get_build_method(elem)
         tplfile = bm.get_template_file(ar,self,elem)
         kw.update(message=_("Template file: %s ") % tplfile)
@@ -583,7 +590,7 @@ class DirectPrintAction(BasePrintAction):
     A Print Action that uses a hard-coded template and no cache.
     """
     url_action_name = None
-    icon_name='x-tbar-print'
+    icon_name='printer'
     
     #~ def __init__(self,rpt,name,label,bmname,tplname):
     #~ def __init__(self,label=None,tplname='Default',build_method=None,**kw):
@@ -609,7 +616,8 @@ class DirectPrintAction(BasePrintAction):
         #~ return super(DirectPrintAction,self).get_print_templates(bm,elem)
         
         
-    def run_from_ui(self,elem,ar,**kw):
+    def run_from_ui(self,ar,**kw):
+        elem = ar.selected_rows[0]
         bm =  bm_dict.get(
             self.build_method or 
             settings.SITE.site_config.default_build_method)
@@ -627,7 +635,7 @@ class DirectPrintAction(BasePrintAction):
             kw.update(open_url=url)
         return ar.success(**kw)
     
-#~ class EditTemplateAction(dd.RowAction):
+#~ class EditTemplateAction(dd.Action):
     #~ name = 'tpledit'
     #~ label = _('Edit template')
     
@@ -636,7 +644,7 @@ class DirectPrintAction(BasePrintAction):
         #~ target = bm.get_template_url(self,elem)
         #~ return rr.ui.success_response(open_url=target,**kw)
     
-class ClearCacheAction(actions.RowAction):
+class ClearCacheAction(actions.Action):
     """
     Defines the :guilabel:`Clear cache` button on a Printable record.
     
@@ -650,7 +658,7 @@ class ClearCacheAction(actions.RowAction):
     label = _('Clear cache')
     #~ debug_permissions = 20121127
     
-    icon_name = 'x-tbar-clearcache'
+    icon_name = 'printer_delete'
     
     #~ def disabled_for(self,obj,request):
         #~ if not obj.build_time:
@@ -663,7 +671,8 @@ class ClearCacheAction(actions.RowAction):
             return False
         return super(ClearCacheAction,self).get_action_permission(ar,obj,state)
     
-    def run_from_ui(self,elem,ar):
+    def run_from_ui(self,ar):
+        elem = ar.selected_rows[0]
         def doit():
             elem.clear_cache()
             return ar.success("%s printable cache has been cleared." % elem,refresh=True)
@@ -679,7 +688,7 @@ class ClearCacheAction(actions.RowAction):
             #~ logger.info("%r == %r : no confirmation", elem.get_cache_mtime(),elem.build_time)
         return doit()
         
-        
+    
     
 class PrintableType(Model):
     """
@@ -715,7 +724,7 @@ class PrintableType(Model):
     #~ build_method = models.CharField(max_length=20,choices=mixins.build_method_choices())
     #~ template = models.CharField(max_length=200)
     
-    @classmethod
+    #~ @classmethod
     def get_templates_group(cls):
         #~ return cls.templates_group or cls._meta.app_label
         return cls.templates_group # or full_model_name(cls)
@@ -733,11 +742,14 @@ class BasePrintable(object):
     """
     Common base for :class:`Printable`.and :class:`CachedPrintable`.
     """
-    def get_print_language(self,pm):
+    def before_printable_build(self,bm):
+        pass
+        
+    def get_print_language(self):
         return settings.SITE.DEFAULT_LANGUAGE.django_code
         
     def get_templates_group(self):
-        return model_group(self)
+        return model_group(self.__class__)
         
     def filename_root(self):
         return self._meta.app_label + '.' + self.__class__.__name__ + '-' + str(self.pk)
@@ -762,6 +774,48 @@ class BasePrintable(object):
         #~ return settings.SITE.preferred_build_method 
         #~ return 'pisa'
         
+    def get_printable_context(self,ar,**kw):
+        """
+        Defines the following names in your template context:
+        
+        this
+            The printable object instance
+            
+        mtos
+            "amount to string" using :func:`decfmt`
+        fds
+            "format date short", see :func:`north.dbutils.fds`
+        fdm
+            "format date medium", see :func:`north.dbutils.fdm`
+        
+        
+        :class:`lino.modlib.notes.models.Note` extends this.
+        """
+        def translate(s):
+            return _(s.decode('utf8'))
+        from lino import dd
+        kw.update(this=self,
+            dtos=dd.dtos, # obsolete
+            dtosl=dbutils.dtosl, # obsolete
+            dtomy=dbutils.dtomy, # obsolete
+            mtos=decfmt,
+            fds=dbutils.fds,
+            fdm=dbutils.fdm,
+            fdl=dbutils.fdl,
+            fdf=dbutils.fdf,
+            fdmy=dbutils.fdmy,
+            babelattr=dbutils.babelattr,
+            babelitem=dbutils.babelitem,
+            tr=dbutils.babelitem,
+            iif=iif,
+            settings=settings,
+            ar=ar,
+            site_config=settings.SITE.site_config,
+            _ = translate,
+            )
+        kw.update(language=self.get_print_language())
+        return kw
+            
 class Printable(BasePrintable):
     """
     Mixin for Models whose instances can "print" (generate a printable document).
@@ -776,8 +830,27 @@ class CachedPrintable(Duplicable,BasePrintable):
     Mixin for Models that generate a unique external file at a 
     determined place when being printed.
     
+    Adds a "Print" button, a "Clear cache" button and a `build_time` 
+    field.
+    
+    The "Print" button of a 
+    :class:`CachedPrintable <lino.mixins.printable.CachedPrintable>`
+    transparently handles the case when multiple rows are selected. 
+    If multiple rows are selected (which is possible only when 
+    :attr:`cell_edit <lino.core.tables.AbstractTable.cell_edit>` is True),
+    then it will automatically:
+    
+    - build the cached printable for those objects who don't yet have 
+      one
+      
+    - generate a single temporary pdf file which is a merge of these 
+      individual cached printable docs
+    
+    
     """
     do_print = CachedPrintAction()
+    do_clear_cache = ClearCacheAction()
+    #~ print_all = PrintAll()
     
     #~ must_build = models.BooleanField(_("must build"),default=True,editable=False)
     build_time = models.DateTimeField(_("build time"),null=True,editable=False)
@@ -786,15 +859,14 @@ class CachedPrintable(Duplicable,BasePrintable):
     if no build hasn't been called yet.
     """
     
-    do_clear_cache = ClearCacheAction()
     
     
     class Meta:
         abstract = True
         
       
-    def print_from_posting(self,posting,ar,**kw):
-        return self.do_print.run_from_ui(self,ar,**kw)
+    #~ def print_from_posting(self,posting,ar,**kw):
+        #~ return self.do_print.run_from_session(ar,**kw)
         
     def on_duplicate(self,ar,master):
         super(CachedPrintable,self).on_duplicate(ar,master)
@@ -821,6 +893,13 @@ class CachedPrintable(Duplicable,BasePrintable):
         self.build_time = None
         self.save()
         
+    def build_target(elem,ar):
+        bm = get_build_method(elem)
+        t = bm.build(ar,elem.__class__.do_print,elem)
+        if t is None:
+            raise Exception("%s : build() returned None?!")
+        elem.build_time = datetime.datetime.fromtimestamp(t)
+        elem.save()
 
 class TypedPrintable(CachedPrintable):
     """
