@@ -1,4 +1,4 @@
-## Copyright 2012 Luc Saffre
+## Copyright 2012-2013 Luc Saffre
 ## This file is part of the Lino project.
 ## Lino is free software; you can redistribute it and/or modify 
 ## it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@ import inspect
 import types
 import datetime
 
-from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext_lazy as _
 from django.db import models
 from django.conf import settings
 
@@ -55,14 +55,13 @@ class Models(dd.VirtualTable):
     def get_data_rows(self,ar):
         #~ profile = ar.get_user().profile
         for model in models.get_models():
-            #~ if model._lino_default_table.get_view_permission(profile):
             if True: 
                 #~ print model
                 yield model
                 
     @classmethod
     def summary_row(cls,ar,obj,**kw):
-        return E.p(unicode(obj._meta.verbose_name_plural))
+        return [unicode(obj._meta.verbose_name_plural)]
                 
                 
     @dd.displayfield(_("app_label"))
@@ -76,10 +75,9 @@ class Models(dd.VirtualTable):
     #~ @dd.displayfield(_("Detail Action"))
     @dd.displayfield()
     def detail_action(self,obj,ar):
-        #~ return str(obj._lino_default_table.detail_action)
-        if obj._lino_default_table.detail_action is None:
+        if obj.get_default_table().detail_action is None:
             return ''
-        return obj._lino_default_table.detail_action.full_name()
+        return obj.get_default_table().detail_action.full_name()
         
     #~ @dd.displayfield(_("verbose name"))
     #~ def vebose_name(self,obj,ar):
@@ -92,8 +90,7 @@ class Models(dd.VirtualTable):
         
     @dd.requestfield(_("Rows"))
     def rows(self,obj,ar):
-        #~ if obj._lino_default_table.get_view_permission(ar.get_user()):
-        return obj._lino_default_table.request(ar.ui,
+        return obj.get_default_table().request(ar.ui,
           user=ar.get_user(),renderer=ar.renderer)
 
 
@@ -260,6 +257,7 @@ class About(mixins.EmptyTable):
     #~ detail_layout = AboutDetail()
     detail_layout = dd.FormLayout("""
     about_html
+    server_status
     """,window_size = (60,20))
     
     #~ versions = dd.Constant(lino.welcome_html())
@@ -278,12 +276,7 @@ class About(mixins.EmptyTable):
         
     @dd.constant()
     def about_html(cls):
-    #~ def about_html(cls,ui):
       
-    #~ @dd.displayfield()
-    #~ def about_html(cls,obj,ar):
-        #~ ui = ar.ui
-        #~ return settings.SITE.welcome_html(ui)
         body = []
         
         body.append(settings.SITE.welcome_html())
@@ -313,10 +306,15 @@ class About(mixins.EmptyTable):
         for label,value in times:
             items.append(E.li(unicode(label),' : ',E.b(dtfmt(value))))
         body.append(E.ul(*items))
-        
         return E.div(*body,class_='htmlText')
         
         
+    @dd.displayfield(_("Server status"))
+    def server_status(cls,obj,ar):
+        body = []
+        body.append(E.p(_("%s pending threads") % len(settings.SITE.ui.pending_threads)))
+        return E.div(*body,class_='htmlText')
+        
     #~ @dd.displayfield(_("Versions"))
     #~ def versions(self,obj,ar):
         #~ return lino.welcome_html(ar.ui)
