@@ -116,8 +116,8 @@ class Account(dd.BabelNamed,mixins.Sequenced):
                 self.ref = str(self.chart.account_set.count()+1)
             if not self.name:
                 self.name = self.group.name
-            if not self.type:
-                self.type = self.group.account_type
+            #~ if not self.type:
+            self.type = self.group.account_type
             #~ if not self.chart:
                 #~ self.chart = self.group.chart
         super(Account,self).full_clean(*args,**kw)
@@ -142,28 +142,32 @@ class AccountsByGroup(Accounts):
     required = dd.required()
     master_key = 'group'
     column_names = "ref name *"
-
+    
+    
+    
     
 
-def customize_products():
-    dd.inject_field('products.Product',
-        'sales_account',
-        models.ForeignKey('accounts.Account',
-            verbose_name=_("Sales account"),
-            blank=True,null=True,
-            related_name="products_sales",
-            help_text=_("The account to move when this product is used in a sales invoice.")
-        ))
-    dd.inject_field('products.Product',
-        'purchases_account',
-        models.ForeignKey('accounts.Account',
-            verbose_name=_("Purchases account"),
-            blank=True,null=True,
-            related_name="products_purchases",
-            help_text=_("The account to move when this product is used in a purchases invoice.")
-        ))
 
-customize_products()
+#~ 
+#~ def customize_products():
+    #~ dd.inject_field('products.Product',
+        #~ 'sales_account',
+        #~ models.ForeignKey('accounts.Account',
+            #~ verbose_name=_("Sales account"),
+            #~ blank=True,null=True,
+            #~ related_name="products_sales",
+            #~ help_text=_("The account to move when this product is used in a sales invoice.")
+        #~ ))
+    #~ dd.inject_field('products.Product',
+        #~ 'purchases_account',
+        #~ models.ForeignKey('accounts.Account',
+            #~ verbose_name=_("Purchases account"),
+            #~ blank=True,null=True,
+            #~ related_name="products_purchases",
+            #~ help_text=_("The account to move when this product is used in a purchases invoice.")
+        #~ ))
+#~ 
+#~ customize_products()
 
 
 
