@@ -31,7 +31,6 @@ from babel.dates import format_datetime, format_date
 from north import to_locale
 
 from lino.core import actions
-#~ from lino.core.actors import 
 from lino import dd
 
 
@@ -115,7 +114,8 @@ class CalendarAction(actions.Action):
     opens_a_window = True
     action_name = 'grid' # because...
     default_format = 'html'
-    icon_name = 'x-tbar-calendar'
+    #~ icon_name = 'x-tbar-calendar'
+    icon_name = 'calendar'
 
 
 
