@@ -13,7 +13,11 @@
 ## along with Lino; if not, see <http://www.gnu.org/licenses/>.
 
 """
+
+The :xfile:`models.py` module for the :mod:`lino.modlib.contacs` app.
+
 This module defines the tables 
+
 - :class:`Partner` (and their specializations :class:`Person` and :class:`Company`)
 - :class:`Role` and :class:`RoleType`
 
@@ -47,6 +51,7 @@ from lino import dd
 #~ from lino import fields
 
 from lino import mixins
+
 from lino.utils import join_words
 from lino.utils.choosers import chooser
 #~ from lino.models import get_site_config
@@ -56,19 +61,66 @@ from lino.utils.choosers import chooser
 #~ from lino.modlib.countries.models import CountryCity
 from lino.modlib.countries.models import CountryRegionCity
 
-#~ from lino.modlib.contacts.utils import get_salutation
-#~ from lino.modlib.contacts.utils import GENDER_CHOICES, get_salutation
+from lino.modlib.contacts import App
+
 
 
 from lino.utils import mti
 
 
-from lino.modlib.contacts import MODULE_LABEL
+#~ from lino.modlib.contacts import MODULE_LABEL
 
 
 PARTNER_NUMBERS_START_AT = 100 # used for generating demo data and tests
 
 
+class AddressFormatter(object):
+    """
+    Format used in BE, DE, FR, NL...
+    """
+    def get_city_lines(me,self):
+        if self.city is not None:
+            s = join_words(self.zip_code or self.city.zip_code,self.city)
+            if s:
+                yield s 
+            
+class EstonianAddressFormatter(AddressFormatter):
+    """
+    Format used in Estonia.
+    Not ready and not tested.
+    """
+    def get_city_lines(me,self):
+        #lines = [self.name,street,self.addr1,self.addr2]
+        if self.region: # 
+            if self.city:
+                join_words(self.zip_code or self.city.zip_code,self.city)
+                if self.city_zip_code:
+                    yield unicode(self.city)
+                    yield unicode(self.city)
+            s = join_words(self.zip_code,self.region)
+        else: 
+            s = join_words(self.zip_code,self.city)
+        if s:
+            yield s 
+            
+
+            
+ADDRESS_FORMATTERS = dict()
+ADDRESS_FORMATTERS[None] = AddressFormatter()
+ADDRESS_FORMATTERS['EE'] = EstonianAddressFormatter()
+            
+def get_address_formatter(country):
+    if country and country.isocode:
+        af = ADDRESS_FORMATTERS.get(country.isocode,None)
+        if af is not None: 
+            return af
+    return ADDRESS_FORMATTERS.get(None)
+            
+            
+            
+
+
+
 
 class CompanyType(dd.BabelNamed):
     """
@@ -93,7 +145,7 @@ class CompanyTypes(dd.Table):
 
 
 #~ class Contact(mti.MultiTableBase,CountryCity):
-class Partner(mti.MultiTableBase,CountryRegionCity):
+class Partner(mti.MultiTableBase,CountryRegionCity,dd.Addressable):
     """
     
     A :class:`Partner` is anything that can act as a business partner.
@@ -165,6 +217,12 @@ class Partner(mti.MultiTableBase,CountryRegionCity):
         verbose_name=_("is Company"),
         help_text=_("Whether this Partner is a Company."))
         
+    print_labels = dd.PrintLabelsAction()
+        
+    def on_create(self,ar):
+        self.language = ar.get_user().language
+        super(Partner,self).on_create(ar)
+        
     def save(self,*args,**kw):
         if self.id is None:
             sc = settings.SITE.site_config # get_site_config()
@@ -210,43 +268,20 @@ but e.g. :class:`PersonMixin` overrides this.
               self.street_no,self.street_box)
         if self.addr2:
             yield self.addr2
-        #lines = [self.name,street,self.addr1,self.addr2]
-        if self.region: # format used in Estonia
-            if self.city:
-                yield unicode(self.city)
-            s = join_words(self.zip_code,self.region)
-        else: 
-            s = join_words(self.zip_code,self.city)
-        if s:
-            yield s 
-        #~ foreigner = True # False
-        #~ if self.id == 1:
-            #~ foreigner = False
-        #~ else:
-            #~ foreigner = (self.country != self.objects.get(pk=1).country)
+           
+        af = get_address_formatter(self.country)
+        for ln in af.get_city_lines(self):
+            yield ln
+            
         if self.country is not None:
             sc = settings.SITE.site_config # get_site_config()
             #~ print 20130228, sc.site_company_id
-            if not sc.site_company or self.country != sc.site_company.country: 
+            if sc.site_company is None or self.country != sc.site_company.country: 
                 # (if self.country != sender's country)
                 yield unicode(self.country)
             
         #~ logger.debug('%s : as_address() -> %r',self,lines)
         
-    def address_lines(self):
-        for ln in self.address_person_lines() : yield ln
-        for ln in self.address_location_lines() : yield ln
-          
-    #~ def address(self,linesep="\n<br/>"):
-    def address(self,linesep="\n"):
-        """
-        The plain text full postal address (person and location). 
-        Lines are separated by `linesep`.
-        """
-        #~ return linesep.join(self.address_lines())
-        return linesep.join(list(self.address_person_lines()) + list(self.address_location_lines()))
-    address.return_type = models.TextField(_("Address"))
-    
     def address_location(self,linesep="\n"):
         """
         The plain text postal address location part. 
@@ -306,7 +341,7 @@ class PartnerDetail(dd.FormLayout):
     
     
 class Partners(dd.Table):
-    required = dd.required(user_level='user')
+    required = dd.Required(user_level='user',user_groups='office')
     model = 'contacts.Partner'
     column_names = "name email * id" 
     order_by = ['name','id']
@@ -349,8 +384,9 @@ class PersonMixin(mixins.Human):
         abstract = True
         
     title = models.CharField(max_length=200,blank=True,
-      verbose_name=_('Title'))
-    """Text to print as part of the first address line in front of first_name."""
+        verbose_name=_('Title'),
+        help_text=_("Text to print before first_name as part of the first address line."))
+    
         
   
 class Person(PersonMixin,Partner):
@@ -365,6 +401,7 @@ class Person(PersonMixin,Partner):
         abstract = settings.SITE.is_abstract_model('contacts.Person')
         verbose_name = _("Person")
         verbose_name_plural = _("Persons")
+        ordering = ['last_name','first_name']
 
     def address_person_lines(self,*args,**kw):
         "Deserves more documentation."
@@ -525,7 +562,7 @@ class RoleTypes(dd.Table):
 
 
 #~ class Contact(dd.Model):
-class Role(dd.Model):
+class Role(dd.Model,dd.Addressable):
     """
     
     A Contact (historical model name :class:`Role`) 
@@ -573,17 +610,19 @@ class Role(dd.Model):
         #~ else:
             #~ for ln in self.person.address_location_lines():
                 #~ yield ln
-    def address_lines(self):
-        for ln in self.person.address_person_lines():
-            yield ln
+    def address_person_lines(self):
+        #~ yield self.name
         if self.company:
             for ln in self.company.address_person_lines():
                 yield ln
-            for ln in self.company.address_location_lines():
-                yield ln
+        for ln in self.person.address_person_lines():
+            yield ln
+        
+    def address_location_lines(self):
+        if self.company:
+            return self.company.address_location_lines()
         else:
-            for ln in self.person.address_location_lines():
-                yield ln
+            return self.person.address_location_lines()
 
 #~ class ContactsByCompany(dd.Table):
     #~ model = 'contacts.RoleOccurence'
@@ -655,34 +694,23 @@ class PartnerDocument(dd.Model):
                 yield p
         
         
-    #~ def summary_row(self,ui,rr,**kw):
-        #~ if self.person:
-            #~ if self.company:
-                #~ # s += ": " + ui.href_to(self.person) + " / " + ui.href_to(self.company)
-                #~ return ui.href_to(self.company) + ' ' + ugettext("attn:") + ' ' + ui.href_to(self.person)
-            #~ else:
-                #~ return ui.href_to(self.person)
-        #~ elif self.company:
-            #~ return ui.href_to(self.company)
-            
-    #~ def summary_row(self,ui,rr,**kw):
     def summary_row(self,ar,**kw):
         """
         A :meth:`lino.core.model.Model.summary_row` 
         method for partner documents.
         """
-        href_to = ar.href_to
+        href_to = ar.obj2html
         #~ href_to = ui.ext_renderer.href_to
-        s = href_to(self)
+        s = [href_to(self)]
         #~ if self.person and not dd.has_fk(rr,'person'):
         if self.person:
             if self.company:
-                s += " (" + href_to(self.person) \
-                    + "/" + href_to(self.company) + ")"
+                s += [" (", href_to(self.person), 
+                    "/",href_to(self.company),")"]
             else:
-                s += " (" + href_to(self.person) + ")"
+                s += [" (",href_to(self.person), ")"]
         elif self.company:
-            s += " (" + href_to(self.company) + ")"
+            s += [" (", href_to(self.company),")"]
         return s
             
     def update_owned_instance(self,other):
@@ -767,16 +795,34 @@ class ContactRelated(dd.Model):
       
     @chooser()
     def contact_person_choices(cls,company):
+        """
+        chooser method for the `contact_person` field.
+        """
         if company is not None:
             return cls.contact_person_choices_queryset(company)
-        return []
+        return settings.SITE.modules.contacts.Person.objects.order_by('last_name','first_name')
+        
         
     def get_contact(self):
-        roles = Role.objects.filter(company=self.company,person=self.contact_person)
-        #~ print '20120929 get_contact', roles
-        if roles.count() == 1:
-            return roles[0]
+        if self.contact_person is not None:
+            if self.company is not None:
+                roles = Role.objects.filter(company=self.company,person=self.contact_person)
+                #~ print '20120929 get_contact', roles
+                if roles.count() == 1:
+                    return roles[0]
         
+    def get_recipient(self):
+        contact = self.get_contact()
+        if contact is not None: 
+            return contact
+        if self.contact_person is not None:
+            if self.company is not None:
+                return Role(company=self.company,person=self.contact_person)
+            return self.contact_person
+        return self.company
+        
+    recipient = property(get_recipient)
+
     def contact_person_changed(self,ar):
         #~ print '20120929 contact_person_changed'
         if self.company and not self.contact_person_id:
@@ -788,18 +834,22 @@ class ContactRelated(dd.Model):
       
     @classmethod
     def contact_person_choices_queryset(cls,company):
+        """
+        Return a queryset of candidate Person objects allowed 
+        in `contact_person` for a given `company`.
+        """
         return settings.SITE.modules.contacts.Person.objects.filter(rolesbyperson__company=company).distinct()
 
     def full_clean(self,*args,**kw):
         if not settings.SITE.loading_from_dump:
-            if self.company and self.contact_person is None:
+            if self.company is not None and self.contact_person is None:
                 qs = self.contact_person_choices_queryset(self.company)
                 #~ qs = self.company.rolesbyparent.all()
                 if qs.count() == 1:
                     self.contact_person = qs[0]
                 else:
                     #~ print "20120227 clear contact!"
-                    self.contact = None
+                    self.contact_person = None
             contact = self.get_contact()
             if contact is not None:
                 self.contact_role = contact.type
@@ -884,7 +934,7 @@ def company_tables_alias(sender,**kw):
 
 
 def setup_main_menu(site,ui,profile,m):
-    m = m.add_menu("contacts",MODULE_LABEL)
+    m = m.add_menu("contacts",App.verbose_name)
     #~ actors = (Persons,Companies,Partners)
     #~ for m in (Person,Company,Partner):
         #~ if m._meta.abstract: 
@@ -897,14 +947,11 @@ def setup_main_menu(site,ui,profile,m):
     for a in ('contacts.Persons','contacts.Companies','contacts.Partners'):
         m.add_action(a)
 
-def setup_my_menu(site,ui,profile,m): 
-    pass
-  
 def setup_master_menu(site,ui,profile,m): 
     pass
     
-def setup_config_menu(site,ui,profile,m): 
-    config_contacts = m.add_menu("contacts",MODULE_LABEL)
+def setup_config_menu(site,ui,profile,m):
+    config_contacts = m.add_menu("contacts",App.verbose_name)
     config_contacts.add_action(CompanyTypes)
     config_contacts.add_action(RoleTypes)
     #~ config_contacts.add_action(site.modules.countries.Countries)
@@ -915,7 +962,7 @@ def setup_config_menu(site,ui,profile,m):
     #~ m.add_action('contacts.RoleTypes')
   
 def setup_explorer_menu(site,ui,profile,m):
-    m = m.add_menu("contacts",MODULE_LABEL)
+    m = m.add_menu("contacts",App.verbose_name)
     m.add_action(site.modules.contacts.Roles)
     #~ m.add_action(site.modules.countries.Cities)
   
