@@ -27,6 +27,7 @@ from __future__ import unicode_literals
 from pprint import pprint
 
 from django.conf import settings
+from django.utils import translation
 
 from djangosite.utils.djangotest import RemoteAuthTestCase
 from django.test.utils import override_settings
@@ -95,10 +96,10 @@ class QuickTest(RemoteAuthTestCase):
         and not properly reset.
         """
         
-        #~ if 'en' in settings.SITE.AVAILABLE_LANGUAGES:
         if settings.SITE.get_language_info('en'):
-            dbutils.set_language('en')
-            self.assertEquals(luc.address(), u'''\
+          with translation.override('en'):
+            #~ dbutils.set_language('en')
+            self.assertEquals(luc.address, u'''\
 Mr Luc SAFFRE
 Uus 1
 Vana-Vigala küla
@@ -106,28 +107,19 @@ Vana-Vigala küla
 Estonia''')
 
         if settings.SITE.get_language_info('de'):
-            dbutils.set_language('de')
-            self.assertEquals(luc.address(), u'''\
+            with translation.override('de'):
+                self.assertEquals(luc.address, u'''\
 Herrn Luc SAFFRE
 Uus 1
 Vana-Vigala küla
 78003 Vigala
 Estland''')
-        dbutils.set_language(None)
-        
-        
-            
-        #~ def test02(self):
-            #~ """
-            #~ """
+                self.assertEquals(luc.address_html, '''\
+<p>Herrn Luc SAFFRE<br />Uus 1<br />Vana-Vigala k&#252;la<br />78003 Vigala<br />Estland</p>''')
             
         u = create_and_get(settings.SITE.user_model,
             username='root',language='',profile=dd.UserProfiles.admin)
             
-        #~ lang = u.language
-        #~ print 20120729, repr(u.language)
-        
-        #~ settings.SITE.never_build_site_cache = True
         
         """
         disable SITE.is_imported_partner() otherwise 
