@@ -15,7 +15,7 @@
 from __future__ import unicode_literals
 
 """
-Deserves a docstring
+The :xfile:`models.py` module for the :mod:`lino.modlib.courses` app.
 """
 
 
@@ -29,6 +29,8 @@ import os
 import cgi
 import datetime
 from decimal import Decimal
+ZERO = Decimal()
+ONE = Decimal(1)
 
 from django.db import models
 from django.db.models import Q
@@ -54,20 +56,6 @@ from lino import dd
 #~ from lino import layouts
 #~ from lino.utils import printable
 from lino import mixins
-#~ from lino import actions
-#~ from lino import fields
-#~ from lino.modlib.contacts import models as contacts
-#~ from lino.modlib.notes import models as notes
-#~ from lino.modlib.links import models as links
-#~ from lino.modlib.uploads import models as uploads
-#~ from lino.modlib.cal import models as cal
-#~ from lino.modlib.users import models as users
-#~ from lino.modlib.contacts.utils import Gender
-#~ from lino.modlib.properties.models import HowWell
-#~ from lino.utils.choicelists import HowWell, Gender
-#~ from lino.utils.choicelists import ChoiceList
-#~ from lino.modlib.properties.utils import KnowledgeField #, StrengthField
-#~ from lino.modlib.uploads.models import UploadsByPerson
 from lino.utils.choosers import chooser
 from lino.utils import mti
 from lino.mixins.printable import DirectPrintAction, Printable
@@ -76,21 +64,6 @@ from lino.core.dbutils import obj2str
 
 from north.dbutils import day_and_month
 
-#~ from lino.modlib.countries.models import CountryCity
-#~ from lino.modlib.cal import models as cal
-#~ from lino.modlib.contacts.models import Partner
-
-#~ # not used here, but these modules are required in INSTALLED_APPS, 
-#~ # and other code may import them using 
-
-#~ from lino.modlib.properties.models import Property
-#~ # from lino.modlib.notes.models import NoteType
-#~ from lino.modlib.countries.models import Country, City
-
-#~ if settings.SITE.user_model:
-    #~ User = dd.resolve_model(settings.SITE.user_model,strict=True)
-
-#~ try:
 
 users = dd.resolve_app('users')
 cal = dd.resolve_app('cal')
@@ -100,7 +73,7 @@ contacts = dd.resolve_app('contacts')
 #~ print '20130219 lino.modlib.courses 2'  
 
 """
-Here we must use `resolve_model` with strict=True
+Here we must use `resolve_model` with `strict=True`
 because we want the concrete model 
 and we don't know whether it is overridden
 by this application.
@@ -125,7 +98,6 @@ Person = dd.resolve_model('contacts.Person',strict=True)
 #~ class PresenceStatuses(dd.Table):
     #~ model = PresenceStatus
     
-    
 class StartEndTime(dd.Model):
     class Meta:
         abstract = True
@@ -247,7 +219,6 @@ class TeachersByType(Teachers):
 
 
 
-
 class PupilType(dd.BabelNamed,dd.Printable):
     class Meta:
         verbose_name = _("Pupil type")
@@ -343,7 +314,7 @@ class CourseStates(dd.Workflow):
 
 add = CourseStates.add_item
 add('10', _("Draft"),'draft')
-add('20', _("Scheduled"),'scheduled')
+add('20', _("Published"),'published')
 add('30', _("Started"),'started')
 add('40', _("Ended"),'ended')
 add('50', _("Cancelled"),'cancelled')
@@ -385,7 +356,7 @@ class Course(contacts.ContactRelated,cal.EventGenerator,cal.RecurrenceSet,dd.Pri
         #~ blank=True,null=True)
         
         
-    tariff = models.ForeignKey('products.Product',
+    tariff = dd.ForeignKey('products.Product',
         blank=True,null=True,
         verbose_name=_("Tariff"),
         related_name='courses_by_tariff')
@@ -409,18 +380,19 @@ class Course(contacts.ContactRelated,cal.EventGenerator,cal.RecurrenceSet,dd.Pri
     def update_cal_until(self):
         return self.end_date
         
-    def update_cal_calendar(self,i):
+    def update_cal_calendar(self):
         return self.calendar
         
     def update_cal_subject(self,i):
-        return _("Lesson %d") % i
+        return "%s %s" % (dd.babelattr(self.calendar,'event_label'),i)
+        #~ return _("Lesson %d") % i
         
     @dd.displayfield(_("Info"))
     def info(self,ar):
         return ar.obj2html(self)
         
     @dd.displayfield(_("Events"))
-    def events_text(self,ar):
+    def events_text(self,ar=None):
         return ', '.join([day_and_month(e.start_date)
             for e in self.cal_event_set_by_project.order_by('start_date')])
         
@@ -450,7 +422,7 @@ def setup_event_from_course(sender=None,instance=None,**kw):
     if settings.SITE.loading_from_dump: return
     event = instance
     if event.is_user_modified(): return
-    if not event.is_editable_state(): return
+    if event.is_fixed_state(): return
     if not isinstance(event.owner,Course): return
     course = event.owner
     event.project = course
@@ -473,7 +445,7 @@ if Course.FILL_EVENT_GUESTS:
         if settings.SITE.loading_from_dump: return
         event = instance
         if event.is_user_modified(): return
-        if not event.is_editable_state(): return
+        if event.is_fixed_state(): return
         if not isinstance(event.owner,Course): return
         course = event.owner
         if event.guest_set.count() > 0: return
@@ -487,13 +459,13 @@ class CourseDetail(dd.FormLayout):
     #~ end = "end_date end_time"
     #~ freq = "every every_unit"
     #~ start end freq
-    main = "general cal.EventsByController"
+    main = "general courses.EnrolmentsByCourse"
     general = dd.Panel("""
     line teacher start_date start_time room #slot state id:8
     max_places max_events end_date end_time every_unit every 
     monday tuesday wednesday thursday friday saturday sunday
     company contact_person user calendar 
-    courses.EnrolmentsByCourse
+    cal.EventsByController
     """,label=_("General"))
     
     #~ def setup_handle(self,dh):
@@ -593,13 +565,15 @@ class ActiveCourses(Courses):
     
     
 class EnrolmentStates(dd.Workflow):
+    verbose_name_plural = _("Enrolment states")
     required = dd.required(user_level='admin')
+    invoiceable = models.BooleanField(_("invoiceable"),default=True)
 
 add = EnrolmentStates.add_item
-add('10', _("Requested"),'requested')
-add('20', _("Confirmed"),'confirmed')
-add('30', _("Cancelled"),'cancelled')
-add('40', _("Certified"),'certified')
+add('10', _("Requested"),'requested',invoiceable=False)
+add('20', _("Confirmed"),'confirmed',invoiceable=True)
+add('30', _("Cancelled"),'cancelled',invoiceable=False)
+add('40', _("Certified"),'certified',invoiceable=True)
 #~ add('40', _("Started"),'started')
 #~ add('50', _("Success"),'success')
 #~ add('60', _("Award"),'award')
@@ -613,13 +587,28 @@ class Enrolment(dd.UserAuthored,dd.Printable,sales.Invoiceable):
     class Meta:
         verbose_name = _("Enrolment")
         verbose_name_plural = _('Enrolments')
+        unique_together = ('course','pupil')
 
     #~ teacher = models.ForeignKey(Teacher)
-    course = models.ForeignKey('courses.Course')
-    pupil = models.ForeignKey(Pupil)
+    course = dd.ForeignKey('courses.Course')
+    pupil = dd.ForeignKey(Pupil)
     request_date = models.DateField(_("Date of request"),default=datetime.date.today)
     state = EnrolmentStates.field(default=EnrolmentStates.requested)
+    amount = dd.PriceField(_("Amount"),blank=True)
+    remark = models.CharField(max_length=200,
+          blank=True,
+          verbose_name=_("Remark"))
+  
+    def save(self,*args,**kw):
+        if self.amount is None:
+            self.compute_amount()
+        super(Enrolment,self).save(*args,**kw)
 
+    #~ def before_ui_save(self,ar):
+        #~ if self.amount is None:
+            #~ self.compute_amount()
+        #~ super(Enrolment,self).before_ui_save(ar)
+        
     def get_print_templates(self,bm,action):
         #~ if self.state:
         return [self.state.name + bm.template_ext]
@@ -629,19 +618,43 @@ class Enrolment(dd.UserAuthored,dd.Printable,sales.Invoiceable):
         return "%s / %s" % (self.course,self.pupil)
         
     invoiceable_date_field = 'request_date'
-    invoiceable_partner_field = 'pupil'
+    #~ invoiceable_partner_field = 'pupil'
     
+    @classmethod
+    def get_partner_filter(cls,partner):
+        """
+        Return a dict of filter...
+        """
+        #~ kw.update(pupil=partner)
+        q1 = models.Q(pupil__invoicing_address__isnull=True,pupil=partner)
+        q2 = models.Q(pupil__invoicing_address=partner)
+        return models.Q(q1 | q2,invoice__isnull=True)
+    
+    
+    def pupil_changed(self,ar):
+        self.compute_amount()
+        
+    def compute_amount(self):
+        #~ if self.course is None: 
+            #~ return 
+        if self.course.tariff is None:
+            self.amount = ZERO
+        self.amount = self.course.tariff.sales_price
+            
+    def get_invoiceable_amount(self): 
+        return self.amount
+        
     def get_invoiceable_product(self): 
-        if self.course: 
+        #~ if self.course is not None: 
+        if self.state.invoiceable: 
             return self.course.tariff
             
     def get_invoiceable_title(self): 
-        if self.course: 
-            return self.course
+        #~ if self.course is not None: 
+        return self.course
 
     def get_invoiceable_qty(self): 
-        if self.course: 
-            return Decimal(1)
+        return ONE
     
 
 class Enrolments(dd.Table):
@@ -660,9 +673,15 @@ class Enrolments(dd.Table):
     order_by = ['request_date']
     column_names = 'request_date course pupil workflow_buttons user *'
     hidden_columns = 'id state'
+    insert_layout = """
+    request_date user
+    course pupil
+    remark
+    """
     detail_layout = """
-    course pupil user request_date state
-    # courses.EventsByEnrolment
+    request_date user
+    course pupil 
+    remark amount workflow_buttons
     sales.InvoicingsByInvoiceable
     """
         
@@ -670,8 +689,8 @@ class Enrolments(dd.Table):
     def get_request_queryset(self,ar):
         qs = super(Enrolments,self).get_request_queryset(ar)
         if isinstance(qs,list): return qs
-        if ar.param_values.user is not None:
-            qs = qs.filter(user=ar.param_values.user)
+        if ar.param_values.author is not None:
+            qs = qs.filter(user=ar.param_values.author)
             
         if ar.param_values.state:
             qs = qs.filter(state=ar.param_values.state)
@@ -703,15 +722,38 @@ class Enrolments(dd.Table):
             yield unicode(_("Also ")) + unicode(EnrolmentStates.cancelled.text)
         if ar.param_values.course_state:
             yield unicode(settings.SITE.modules.courses.Course._meta.verbose_name) + ' ' + unicode(ar.param_values.course_state)
-        if ar.param_values.user:
-            yield unicode(ar.param_values.user)
+        if ar.param_values.author:
+            yield unicode(ar.param_values.author)
         
 
+class ConfirmAllEnrolments(dd.Action):
+    label = _("Confirm all")
+    select_rows = False
+    http_method = 'POST'
+    
+    def run_from_ui(self,ar,**kw):
+        obj = ar.selected_rows[0]
+        assert obj is None
+        def ok():
+            for obj in ar:
+                obj.state = EnrolmentStates.confirmed
+                obj.save()
+                kw.update(refresh_all=True)
+            return kw
+        msg = _("This will confirm all %d enrolments in this list.") % ar.get_total_count()
+        return ar.confirm(ok, msg, _("Are you sure?"))
+    
+
+
 class PendingRequestedEnrolments(Enrolments):
     
     label = _("Pending requested enrolments")
     auto_fit_column_widths = True
     params_panel_hidden = True
+    column_names = 'request_date course pupil remark user amount workflow_buttons'
+    hidden_columns = 'id state'
+    
+    confirm_all = ConfirmAllEnrolments()
     
     @classmethod
     def param_defaults(self,ar,**kw):
@@ -792,8 +834,6 @@ def unused_setup_master_menu(site,ui,profile,m):
     #~ m.add_action(Courses)
 
 
-def setup_my_menu(site,ui,profile,m): pass
-  
 def setup_config_menu(site,ui,profile,m):
     m = m.add_menu("courses",MODULE_LABEL)
     #~ m.add_action(Rooms)
@@ -809,6 +849,7 @@ def setup_explorer_menu(site,ui,profile,m):
     #~ m.add_action(Presences)
     #~ m.add_action(Events)
     m.add_action(Enrolments)
+    m.add_action(EnrolmentStates)
   
   
 #~ print '20130219 lino.modlib.courses ok'  
