@@ -31,13 +31,35 @@ from lino import dd
 from lino import mixins
 from django.utils.translation import ugettext_lazy as _
 from lino.modlib.ledger.utils import FiscalYears
+from lino.mixins.printable import model_group
+from lino.utils.xmlgen.html import E
+from lino.utils import iif
+from lino.utils import join_elems
 
-accounts = dd.resolve_app('accounts')
-vat = dd.resolve_app('vat')
+accounts = dd.resolve_app('accounts',strict=True)
+vat = dd.resolve_app('vat',strict=True)
 partner_model = settings.SITE.partners_app_label + '.Partner'
 
 ZERO = Decimal()
 
+
+vat.TradeTypes.purchases.update(
+    #~ price_field_name='sales_price',
+    #~ price_field_label=_("Sales price"),
+    base_account_field_name='purchases_account',
+    base_account_field_label=_("Purchases Base account"),
+    vat_account_field_name='purchases_vat_account',
+    vat_account_field_label=_("Purchases VAT account"),
+    partner_account_field_name='suppliers_account',
+    partner_account_field_label=_("Suppliers account"))
+
+#~ TradeTypes.purchases.update(
+    #~ partner_account_field_name='suppliers_account',
+    #~ partner_account_field_label=_("Suppliers account"))
+
+
+
+
 class VoucherType(dd.Choice):
     def __init__(self,model,table_class):
         self.table_class = table_class
@@ -69,7 +91,44 @@ class VoucherTypes(dd.ChoiceList):
         return cls.add_item_instance(VoucherType(model,table_class))
     
 
-class Journal(dd.BabelNamed,mixins.Sequenced):
+
+
+class DebitOrCreditField(models.BooleanField):
+    pass
+    
+class MatchField(models.CharField):
+    def __init__(self,verbose_name=None,**kw):
+        if verbose_name is None:
+            verbose_name = _("Match")
+        kw.setdefault('max_length',20)
+        models.CharField.__init__(self,verbose_name,**kw)
+    
+class DcAmountField(dd.VirtualField):
+    """
+    An editable virtual field to set both fields `amount` and `dc`
+    """
+    
+    editable = True
+    
+    def __init__(self,dc,*args,**kw):
+        self.dc = dc
+        dd.VirtualField.__init__(self,dd.PriceField(*args,**kw),None)
+        
+    def set_value_in_object(self,request,obj,value):
+        obj.amount = value
+        obj.dc = self.dc
+        
+    def value_from_object(self,obj,ar):
+        if obj.dc == self.dc: return obj.amount
+        return None
+
+
+
+
+
+
+
+class Journal(dd.BabelNamed,mixins.Sequenced,mixins.PrintableType):
   
     class Meta:
         verbose_name = _("Journal")
@@ -83,13 +142,14 @@ class Journal(dd.BabelNamed,mixins.Sequenced):
     voucher_type = VoucherTypes.field() 
     force_sequence = models.BooleanField(default=False)
     #~ total_based = models.BooleanField(_("Voucher entry based on total amount"),default=False)
-    chart = models.ForeignKey('accounts.Chart')
-    #~ chart = models.ForeignKey('accounts.Chart',blank=True,null=True)
-    account = models.ForeignKey('accounts.Account',blank=True,null=True)
+    chart = dd.ForeignKey('accounts.Chart')
+    #~ chart = dd.ForeignKey('accounts.Chart',blank=True,null=True)
+    account = dd.ForeignKey('accounts.Account',blank=True,null=True)
     #~ account = models.CharField(max_length=6,blank=True)
     #~ pos = models.IntegerField()
     #~ printed_name = models.CharField(max_length=100,blank=True)
     printed_name = dd.BabelCharField(max_length=100,blank=True)
+    dc = DebitOrCreditField(default=None)
     
     @dd.chooser()
     def account_choices(self,chart):
@@ -169,6 +229,12 @@ class Journal(dd.BabelNamed,mixins.Sequenced):
         pass
         
     def full_clean(self,*args,**kw):
+        if self.dc is None:
+            if self.trade_type:
+                self.dc = self.trade_type.dc
+            elif self.account:
+                self.dc = self.account.type.dc
+                
         if not self.name:
             self.name = self.id
         #~ if not self.pos:
@@ -183,20 +249,22 @@ class Journal(dd.BabelNamed,mixins.Sequenced):
             if doc.number + 1 != self.get_next_number(doc):
                 return _("%s is not the last voucher in journal" % unicode(doc))
 
+    def get_templates_group(self):
+        return model_group(self.voucher_type.model)
 
 
 class Journals(dd.Table):
     model = Journal
     order_by = ["seqno"]
-    column_names = "seqno id trade_type voucher_type name force_sequence *"
+    column_names = "ref:5 trade_type voucher_type force_sequence name * seqno id"
     detail_layout = """
-    seqno id trade_type voucher_type 
-    force_sequence account
+    ref trade_type voucher_type 
+    force_sequence account seqno id
     name
-    printed_name
+    build_method  printed_name
     """
     insert_layout = dd.FormLayout("""
-    name
+    ref name
     trade_type 
     voucher_type 
     """,window_size=(60,'auto'))
@@ -215,15 +283,15 @@ def VoucherNumber(**kw):
 
 #~ class Voucher(mixins.Controllable):
 #~ class Voucher(mixins.UserAuthored,mixins.ProjectRelated):
-class Voucher(mixins.UserAuthored):
+class Voucher(mixins.UserAuthored,mixins.Registrable):
     """
     A Voucher is a document that represents a monetary transaction.
     Subclasses must define a field `state`.
     This model is subclassed by sales.Invoice, ledger.AccountInvoice, 
     finan.Statement etc...
     
-    It is *not* abstract because we have a ForeignKey to Voucher in Movement,
-    and we want one Movement model for all ledger movements.
+    It is *not* abstract because we have a ForeignKey to Voucher in 
+    Movement and we want one Movement model for all ledger movements.
     
     """
     class Meta:
@@ -250,6 +318,12 @@ class Voucher(mixins.UserAuthored):
         #~ jnl = Journal(doctype=doctype,id=id,**kw)
         #~ return jnl
         
+    def get_due_date(self):
+        return self.date
+        
+    def get_trade_type(self):
+        return self.journal.trade_type
+        
     @classmethod
     def create_journal(cls,trade_type=None,account=None,chart=None,**kw):
     #~ def create_journal(cls,jnl_id,trade_type,**kw):
@@ -278,8 +352,17 @@ class Voucher(mixins.UserAuthored):
         if self.number is None:
             return "%s #%s (not registered)" % (
                 unicode(self.journal.voucher_type.model._meta.verbose_name),self.id)
+        if self.journal.ref:
+            return "%s#%s" % (self.journal.ref,self.number)
         return "#%s (%s %s)" % (self.number,self.journal,self.year)
         
+    def get_default_match(self):
+        #~ return "%s#%s" % (self.journal.ref,self.number)
+        return "%s%s" % (self.id,self.journal.ref)
+        
+    #~ def on_create(self,*args,**kw):
+        #~ super(Voucher,self).on_create(*args,**kw)
+        #~ settings.SITE.on_create_voucher(self)
                 
     def register(self,ar):
         """
@@ -369,10 +452,14 @@ class Voucher(mixins.UserAuthored):
             #~ return False
         #~ return super(Voucher,self).get_row_permission(ar,state,ba)
 
-    def href_to(self,ar):
-        obj = self.journal.voucher_type.model.objects.get(
+    def get_mti_child(self):
+        return self.journal.voucher_type.model.objects.get(
             journal=self.journal,number=self.number,year=self.year)
-        return ar.href_to(obj)
+        #~ m = self.journal.voucher_type.model
+        #~ return m.objects.get(pk=self.pk)
+        
+    def obj2html(self,ar):
+        return ar.obj2html(self.get_mti_child())
         
     
     #~ def add_voucher_item(self,account=None,**kw):
@@ -391,7 +478,7 @@ class Voucher(mixins.UserAuthored):
         return self.items.model(**kw)
         #~ return super(AccountInvoice,self).add_voucher_item(**kw)
         
-
+        
 class Vouchers(dd.Table):
     """
     List of all vouchers
@@ -421,34 +508,54 @@ class ByJournal(dd.Table):
 
 
 
-class DebitOrCreditField(models.BooleanField):
-    pass
-    
-    
-class DcAmountField(dd.VirtualField):
-    """
-    An editable virtual field to set both fields `amount` and `dc`
-    """
-    
-    editable = True
-    
-    def __init__(self,dc,*args,**kw):
-        self.dc = dc
-        dd.VirtualField.__init__(self,dd.PriceField(*args,**kw),None)
         
-    def set_value_in_object(self,request,obj,value):
-        obj.amount = value
-        obj.dc = self.dc
+class Matchable(dd.Model):
+    
+    class Meta: 
+        abstract = True
         
-    def value_from_object(self,obj,ar):
-        if obj.dc == self.dc: return obj.amount
-        return None
+    match = MatchField(blank=True)
+    #~ match = dd.ForeignKey('ledger.Movement',verbose_name=_("Match"),blank=True,null=True)
+    
+    #~ @dd.chooser()
+    #~ def match_choices(cls,partner):
+        #~ d = {}
+        #~ qs = Movement.objects.filter(partner=partner,satisfied=False)
+        #~ for m in qs:
+            #~ i = d.setdefault(m.get_match(),[])
+            #~ i.append(m)
+        #~ choices = []
+        #~ for k,lst in d.items():
+            #~ amount = ZERO
+            #~ text = ", ".join([i.select_text() for i in lst])
+            #~ for i in lst:
+                #~ amount += iif(i.dc,1,-1) * i.amount
+            #~ text += ' ' + unicode(amount)
+            #~ choices.append((k,text))
+        #~ return choices
         
+        
+        
+    @dd.chooser(simple_values=True)
+    def match_choices(cls,partner):
+        #~ DC = voucher.journal.dc
+        #~ choices = []
+        qs = Movement.objects.filter(partner=partner,satisfied=False)
+        qs = qs.order_by('voucher__date')
+        #~ qs = qs.distinct('match')
+        return qs.values_list('match',flat=True)
+        
+        for mvt in qs:
+            choices.append(Match(DC,partner,mvt.match))
+        return choices
+        
+    #~ def get_match_display(self,m):
+        #~ return repr(m)
     
 
 
     
-class Movement(mixins.Sequenced):
+class Movement(mixins.Sequenced,Matchable):
   
     allow_cascaded_delete = ['voucher']
     
@@ -458,14 +565,28 @@ class Movement(mixins.Sequenced):
         
     voucher = models.ForeignKey(Voucher)
     #~ pos = models.IntegerField("Position",blank=True,null=True)
-    account = models.ForeignKey(accounts.Account)
-    partner = models.ForeignKey(partner_model,blank=True,null=True)
+    account = dd.ForeignKey(accounts.Account)
+    partner = dd.ForeignKey(partner_model,blank=True,null=True)
     amount = dd.PriceField(default=0)
     dc = DebitOrCreditField()
+    satisfied = models.BooleanField(_("Satisfied"),default=False)
+    #~ match = dd.ForeignKey('self',verbose_name=_("Match"),blank=True,null=True)
     #~ is_credit = models.BooleanField(_("Credit"),default=False)
     #~ debit = dd.PriceField(default=0)
     #~ credit = dd.PriceField(default=0)
     
+    #~ def full_clean(self,*args,**kw):
+        #~ if not self.match:
+            #~ self.match = self.voucher.get_default_match()
+        #~ super(Matchable,self).full_clean(*args,**kw)
+    
+    #~ def get_default_match(self):
+        #~ return unicode(self.voucher)
+        
+    def select_text(self):
+        v = self.voucher.get_mti_child()
+        return "%s (%s)" % (v,v.date)
+    
     @dd.virtualfield(dd.PriceField(_("Debit")))
     def debit(self,ar):
         if self.dc: 
@@ -482,7 +603,13 @@ class Movement(mixins.Sequenced):
             
     @dd.displayfield(_("Voucher"))
     def voucher_link(self,ar):
-        return self.voucher.href_to(ar)
+        #~ return self.voucher.get_mti_child().obj2html(ar)
+        return ar.obj2html(self.voucher.get_mti_child())
+        
+    #~ @dd.displayfield(_("Matched by"))
+    #~ def matched_by(self,ar):
+        #~ elems = [obj.voucher_link(ar) for obj in Movement.objects.filter(match=self)]
+        #~ return E.div(*elems)
         
     
     def get_siblings(self):
@@ -492,7 +619,6 @@ class Movement(mixins.Sequenced):
     def __unicode__(self):
         return "%s.%d" % (unicode(self.voucher),self.seqno)
         
-    
 
 class Movements(dd.Table): 
     model = Movement
@@ -501,13 +627,127 @@ class Movements(dd.Table):
     
 class MovementsByVoucher(Movements):
     master_key = 'voucher'
-    column_names = 'seqno account debit credit'
+    column_names = 'seqno account debit credit match satisfied'
+    #~ editable = False
+    auto_fit_column_widths = True
     
 class MovementsByPartner(Movements):
     master_key = 'partner'
-    order_by = ['voucher__date']
-    column_names = 'voucher__date voucher_link debit credit account '
+    order_by = ['-voucher__date']
+    column_names = 'voucher__date voucher_link debit credit account match satisfied'
+    #~ editable = False
+    auto_fit_column_widths = True
     
+class Match(object):
+    """
+    Volatile object representing a group of movements
+    """
+    def __init__(self,dc,partner,match):
+        self.dc = dc
+        self.partner = partner
+        self.match = match
+        self.debts = []
+        self.received = []
+        self.balance = ZERO
+        self.due_date = None
+        self.trade_type = None
+        self.has_unsatisfied_movement = False
+        self.has_satisfied_movement = False
+        
+        qs = Movement.objects.filter(partner=partner,match=match)
+        for mvt in qs.order_by('voucher__date'):
+            self.collect(mvt)
+    
+    def collect(self,mvt):
+        if mvt.satisfied:
+            self.has_satisfied_movement = True
+        else:
+            self.has_unsatisfied_movement = True
+        if self.trade_type is None:
+            voucher = mvt.voucher.get_mti_child()
+            self.trade_type = voucher.get_trade_type()
+        if mvt.dc == self.dc:
+            self.debts.append(mvt)
+            self.balance += mvt.amount
+            voucher = mvt.voucher.get_mti_child()
+            due_date = voucher.get_due_date()
+            if self.due_date is None or due_date < self.due_date:
+                self.due_date = due_date
+        else:
+            self.received.append(mvt)
+            self.balance -= mvt.amount
+    
+    def update_satisfied(self):
+        satisfied = self.balance != ZERO
+        if satisfied:
+            if not self.has_unsatisfied_movement: return 
+        else:
+            if not self.has_satisfied_movement: return 
+        for m in self.debts + self.received:
+            if m.satisfied != satisfied:
+                m.satisfied = satisfied
+                m.save()
+                
+def get_partner_matches(partner,dc,**flt):
+    qs = Movement.objects.filter(partner=partner,**flt)
+    qs = qs.order_by('voucher__date')
+    if False: # DISTINCT ON fields not supported
+        qs = qs.distinct('match') 
+        for mvt in qs:
+            yield Match(cls.DUE_DC,partner,mvt.match)
+    else:
+        #~ logger.info("20130921 %s %s",partner,qs)
+        found = set()
+        for mvt in qs:
+            if not mvt.match in found:
+                found.add(mvt.match)
+                yield Match(dc,partner,mvt.match)
+        
+    
+class DuePaymentsByPartner(dd.VirtualTable):
+    """
+    Due Payements is the table to print in a Payment Reminder.
+    Usually this table has one row per sales invoice which is not completely paid.
+    But several invoices ("debts") may be grouped by match.
+    """
+    
+    master = 'contacts.Partner'
+    column_names = 'match due_date balance debts received'
+    auto_fit_column_widths = True
+    
+    DUE_DC = accounts.DEBIT
+    
+    
+    @classmethod
+    def get_data_rows(cls,ar):
+        partner = ar.master_instance
+        if partner is None: return 
+        return get_partner_matches(cls.DUE_DC,partner,satisfied=False)
+        
+    @dd.displayfield(_("Match"))
+    def match(self,row,ar):
+        return row.match
+            
+    @dd.displayfield(_("Due date"))
+    def due_date(self,row,ar):
+        return row.due_date
+            
+    @dd.displayfield(_("Due"))
+    def debts(self,row,ar):
+        #~ return ", ".join([ar.obj2html(i,i.select_text()) for i in row.debts])
+        #~ return join_elems([ar.obj2html(i.voucher,i.select_text()) for i in row.debts])
+        return join_elems([ar.obj2html(i.voucher.get_mti_child()) for i in row.debts])
+            
+    @dd.displayfield(_("Received"))
+    def received(self,row,ar):
+        #~ return ", ".join([ar.obj2html(i,i.select_text()) for i in row.received])
+        #~ return join_elems([ar.obj2html(i.voucher,i.select_text()) for i in row.received])
+        return join_elems([ar.obj2html(i.voucher.get_mti_child()) for i in row.received])
+        
+    @dd.virtualfield(dd.PriceField(_("Balance")))
+    def balance(self,row,ar):
+        return row.balance
+            
 
 
 
@@ -524,7 +764,7 @@ add('40',_("Paid"),'paid',editable=False)
 #~ InvoiceStates.draft.add_transition(_("Deregister"),states='registered paid')
 #~ InvoiceStates.registered.add_transition(_("Register"),states='draft')
     
-class AccountInvoice(vat.VatDocument,Voucher,mixins.Registrable):
+class AccountInvoice(vat.VatDocument,Voucher,Matchable):
     
     class Meta:
         verbose_name = _("Invoice")
@@ -539,8 +779,9 @@ class AccountInvoice(vat.VatDocument,Voucher,mixins.Registrable):
     
     workflow_state_field = 'state'
     
-    def get_trade_type(self):
-        return self.journal.trade_type
+    def get_due_date(self):
+        return self.due_date or self.date
+        
         
     
 
@@ -587,7 +828,7 @@ class InvoiceDetail(dd.FormLayout):
     
     general = dd.Panel("""
     id date partner user 
-    due_date your_ref vat_regime item_vat
+    due_date your_ref vat_regime #item_vat
     ItemsByInvoice:60 totals:20
     """,label=_("General"))
     
@@ -672,6 +913,9 @@ class ItemsByInvoice(dd.Table):
     column_names = "account title vat_class total_base total_vat total_incl seqno"
     master_key = 'voucher'
     order_by = ["seqno"]
+    
+    
+    
 
 
 #~ MODULE_LABEL = _("Ledger")
@@ -705,9 +949,6 @@ def setup_main_menu(site,ui,profile,main):
             #~ label=unicode(jnl),
             #~ params=dict(master_instance=jnl))
     
-def setup_my_menu(site,ui,profile,m): 
-    pass
-  
 def setup_config_menu(site,ui,profile,m): 
     #~ m = m.add_menu("ledger",MODULE_LABEL)
     m = m.add_menu("accounts",MODULE_LABEL)
@@ -733,3 +974,10 @@ def customize_accounts():
             models.BooleanField(verbose_name=tt.text))
 
 customize_accounts()
+
+
+
+def update_partner_satisfied(p):
+    for m in get_partner_matches(accounts.DEBIT,p):
+        m.update_satisfied()
+        
