@@ -37,28 +37,13 @@ from django.core.exceptions import ValidationError
 
 
 from lino import mixins
-#~ from lino.mixins import mails
 from lino import dd
 from lino.core import actions
 
-#~ from lino import reports
-#~ from lino import layouts
-#~ from lino.utils import perms
-from lino.utils.restify import restify
-#~ from lino.utils import printable
 from north import dbutils
-#~ from lino.utils import call_optional_super
-#~ from lino import choices_method, simple_choices_method
-
-#~ from lino.modlib.mails.utils import RecipientType
 
 from lino.utils.html2text import html2text
 from django.core.mail import EmailMultiAlternatives
-#~ from lino.utils.choicelists import ChoiceList
-
-
-#~ from Cheetah.Template import Template as CheetahTemplate
-#~ from lino.utils import dtosl, dtos
 
 
 
@@ -118,30 +103,25 @@ Whether the printable file should be attached to the email
 when creating an email from a mailable of this type.
 """)
     #~ email_as_attachment = models.BooleanField(_("Email as attachment"))
+    
     email_template = models.CharField(max_length=200,
       verbose_name=_("Email template"),
-      blank=True,help_text="""\
-The name of the file to be used as template 
-when creating an email from a mailable of this type.
-""")
+      blank=True,help_text="""The name of the file to be used as 
+template when creating an email from a mailable of this type.""")
     
-    #~ @dd.chooser(simple_values=True)
-    #~ def email_template_choices(cls):
-        #~ return find_template_config_files('.eml.html',cls.templates_group)
-      
     @dd.chooser(simple_values=True)
     def email_template_choices(cls):
-        return list_templates('.eml.html',cls.templates_group)
+        return settings.SITE.list_templates('.eml.html',cls.templates_group)
       
     
 
 
-class CreateMail(dd.RowAction):
+class CreateMail(dd.Action):
     """
     Creates an outbox mail and displays it.
     """
     url_action_name = 'email'
-    icon_name = 'x-tbar-email_add'
+    icon_name = 'email_add'
     help_text = _('Create an email from this')
     #~ label = pgettext_lazy(u'verb',u'Mail')
     label = _('Create email')
@@ -168,7 +148,8 @@ class CreateMail(dd.RowAction):
                 return False
         return super(CreateMail,self).get_action_permission(ar,obj,state)
         
-    def run_from_ui(self,elem,ar,**kw):
+    def run_from_ui(self,ar,**kw):
+        elem = ar.selected_rows[0]
       
         as_attachment = elem.attach_to_email(ar)
         
@@ -192,20 +173,6 @@ class CreateMail(dd.RowAction):
         return ar.success(**kw)
         
 
-def list_templates(ext,group=''):
-    logger.info("20130717 list_templates(%r,%r)",ext,group)
-    if group:
-        #~ prefix = os.path.join(*(group.split('/')))
-        def ff(fn):
-            rv = fn.startswith(group) and fn.endswith(ext)
-            #~ logger.info("20130101 %r -> %s", fn,rv)
-            return rv 
-        lst = settings.SITE.jinja_env.list_templates(filter_func=ff)
-        L = len(group) + 1
-        lst = [i[L:] for i in lst]
-        return lst
-    return settings.SITE.jinja_env.list_templates(extensions=[ext])
-    
 
 class Mailable(dd.Model):
     """
@@ -327,12 +294,12 @@ class RecipientsByMail(Recipients):
 
 
 
-class SendMail(dd.RowAction):
+class SendMail(dd.Action):
     """
     Sends this as an email.
     """
   
-    icon_name = 'x-tbar-email_send'
+    icon_name = 'email_go'
     url_action_name = 'send'
     label = _('Send email')
     callable_from = (actions.GridEdit, 
@@ -346,7 +313,8 @@ class SendMail(dd.RowAction):
             return False
         return super(SendMail,self).get_action_permission(ar,obj,state)
         
-    def run_from_ui(self,elem,ar,**kw):
+    def run_from_ui(self,ar,**kw):
+        elem = ar.selected_rows[0]
         #~ if elem.sent:
             #~ return rr.ui.error(message='Mail has already been sent.')
         #~ subject = elem.subject
@@ -473,10 +441,10 @@ class Mail(mixins.AutoUser,mixins.Printable,mixins.ProjectRelated,mixins.Control
         #~ for x in super(Mail,self).get_model_actions(table): yield x
         #~ yield 'send_mail',SendMail()
         
-    def get_print_language(self,bm):
+    def get_print_language(self):
         if self.user is not None:
             return self.user.language
-        return super(Mail,self).get_print_language(bm)
+        return super(Mail,self).get_print_language()
         
     
     def __unicode__(self):
@@ -611,7 +579,7 @@ class Attachment(mixins.Controllable):
         #~ url = ui.build_url(*parts)
         text = url.split('/')[-1]
         #~ return ui.ext_renderer.href(url,text)
-        return ar.renderer.href(url,text)
+        return [ar.renderer.href(url,text)]
         
         
         
