@@ -123,7 +123,7 @@ def objects():
     yield roletype(**babel_values('name',en="IT Manager",fr=u'Gérant informatique',de=u"EDV-Manager",et=u"IT manager"))
 
 
-    I = Instantiator('lino.HelpText','content_type field help_text').build
+    I = Instantiator('system.HelpText','content_type field help_text').build
     
     Person = resolve_model("partners.Person")
     t = ContentType.objects.get_for_model(Person)
