@@ -13,7 +13,8 @@
 ## along with Lino; if not, see <http://www.gnu.org/licenses/>.
 
 """
-Deserves a docstring
+
+
 """
 
 from __future__ import unicode_literals
@@ -24,6 +25,7 @@ import datetime
 
 from decimal import Decimal
 HUNDRED = Decimal('100')
+ZERO = Decimal(0)
 
 from dateutil.relativedelta import relativedelta
 ONE_DAY = relativedelta(days=1)
@@ -43,27 +45,35 @@ from lino import mixins
 from lino.utils import mti
 #~ from lino.utils.quantities import Duration
 
+from ..vat.utils import add_vat, remove_vat
 
 #~ journals = resolve_app('journals')
 #~ journals = models.get_app('journals')
 #~ auth = resolve_app('auth')
 #~ from lino.modlib.users import models as auth
 partners = dd.resolve_app(settings.SITE.partners_app_label)
-accounts = dd.resolve_app('accounts')
-ledger = dd.resolve_app('ledger')
-vat = dd.resolve_app('vat')
-products = dd.resolve_app('products')
-#~ from lino.modlib.ledger import models as ledger
-#~ from lino.modlib.journals import models as journals
-#~ from lino.modlib.products import models as products
-#~ from lino.modlib.contacts import models as contacts
-#~ from lino.modlib.vat.models import TradeTypes
-#~ products = models.get_app('products')
-#~ ledger = resolve_app('ledger')
-#~ products = resolve_app('products')
-#~ contacts = resolve_app('contacts')
-#~ from lino.modlib.contacts import models as contacts
-#~ from lino.utils.choicelists import ChoiceList  
+#~ accounts = dd.resolve_app('accounts')
+ledger = dd.resolve_app('ledger',strict=True)
+vat = dd.resolve_app('vat',strict=True)
+products = dd.resolve_app('products',strict=True)
+
+
+
+vat.TradeTypes.sales.update(
+    price_field_name='sales_price',
+    price_field_label=_("Sales price"),
+    base_account_field_name='sales_account',
+    base_account_field_label=_("Sales Base account"),
+    vat_account_field_name='sales_vat_account',
+    vat_account_field_label=_("Sales VAT account"),
+    partner_account_field_name='clients_account',
+    partner_account_field_label=_("Clients account"))
+
+dd.inject_field('contacts.Partner','invoicing_address',dd.ForeignKey('contacts.Partner',
+        verbose_name=_("Invoicing address"),
+        blank=True,null=True))
+    
+
 
 #~ class Channel(ChoiceList):
     #~ label = _("Channel")
@@ -253,6 +263,7 @@ class SalesRule(dd.Model):
 class SalesDocument(
       #~ mixins.UserAuthored,
       vat.VatDocument,
+      ledger.Matchable,
       #~ journals.Sendable,
       #~ journals.Journaled,
       #~ contacts.ContactDocument,
@@ -300,6 +311,11 @@ class SalesDocument(
     discount = dd.PercentageField(_("Discount"),blank=True,null=True)
     
     
+    def get_printable_type(self):
+        return self.journal
+
+    def get_print_language(self):
+        return self.language
         
     #~ @dd.chooser()
     #~ def partner_choices(self):
@@ -385,7 +401,8 @@ class SalesDocuments(dd.Table):
         
 
 
-class Invoice(SalesDocument,ledger.Voucher,mixins.Registrable):
+#~ class Invoice(SalesDocument,ledger.Voucher,mixins.Registrable):
+class Invoice(SalesDocument,ledger.Voucher):
     """
     An invoice usually used for selling something.
     """
@@ -403,6 +420,9 @@ class Invoice(SalesDocument,ledger.Voucher,mixins.Registrable):
     
     #~ _registrable_fields = set('date author partner vat_regime payment_term due_date'.split())
     
+    def get_due_date(self):
+        return self.due_date or self.date
+        
     def full_clean(self,*args,**kw):
         if self.due_date is None:
             if self.payment_term is not None:
@@ -428,37 +448,57 @@ class Invoice(SalesDocument,ledger.Voucher,mixins.Registrable):
         yield 'user'
         yield 'partner'
         yield 'vat_regime'
-        yield 'item_vat'
+        #~ yield 'item_vat'
 
 
 
 class ProductDocItem(ledger.VoucherItem,vat.QtyVatItemBase):
+    
+    class Meta:
+        abstract = True
+        
     product = models.ForeignKey('products.Product',blank=True,null=True)
     #~ title = models.CharField(max_length=200,blank=True)
     description = dd.RichTextField(_("Description"),blank=True,null=True)
     #~ discount = models.IntegerField(_("Discount"),default=0)
     discount = dd.PercentageField(_("Discount"),blank=True,null=True)
+    
 
     
     def get_base_account(self,tt):
-        ref = settings.SITE.get_product_base_account(tt,self.product)
-        return self.voucher.journal.chart.get_account_by_ref(ref)
+        return tt.get_product_base_account(self.product)
+        #~ return self.voucher.journal.chart.get_account_by_ref(ref)
         
-    def get_vat_class(self,tt):
-        name = settings.SITE.get_product_vat_class(tt,self.product)
-        return vat.VatClasses.get_by_name(name)
+    #~ def get_vat_class(self,tt):
+        #~ name = settings.SITE.get_product_vat_class(tt,self.product)
+        #~ return vat.VatClasses.get_by_name(name)
         
     #~ def full_clean(self,*args,**kw):
         #~ super(ProductDocItem,self).full_clean(*args,**kw)
 
     def discount_changed(self,ar):
-        if self.product:
-            if self.product.price is not None:
-                if self.discount is None:
-                    self.unit_price = self.product.price 
-                else:
-                    self.unit_price = self.product.price * (HUNDRED - self.discount) / HUNDRED                    
-                self.unit_price_changed(ar)
+        if not self.product:
+            return
+
+        tt = self.voucher.get_trade_type()
+        catalog_price = tt.get_catalog_price(self.product)
+            
+        if catalog_price is not None:
+            #~ assert self.vat_class == self.product.vat_class
+            if self.voucher.vat_regime.item_vat:
+                rate = self.get_vat_rate() # rate of this item
+            else:
+                rate = ZERO
+            catalog_rate = settings.SITE.get_vat_rate(tt,
+                self.vat_class,vat.get_default_vat_regime)
+            if rate != catalog_rate:
+                catalog_price = remove_vat(catalog_price,catalog_rate)
+                catalog_price = add_vat(catalog_price,rate)
+            if self.discount is None:
+                self.unit_price = catalog_price
+            else:
+                self.unit_price = catalog_price * (HUNDRED - self.discount) / HUNDRED                    
+            self.unit_price_changed(ar)
                 
     def product_changed(self,ar):
         if self.product:
@@ -509,10 +549,14 @@ class ItemsByInvoice(ItemsByDocument):
     hidden_columns = "seqno description total_base total_vat"
     
 
+class ItemsByInvoicePrint(ItemsByInvoice):
+    column_names = "title:40 unit_price:10 qty:5 total_incl:10"
+
 class InvoiceItemsByProduct(ItemsByInvoice):
     master_key = 'product'
     column_names = "voucher voucher__partner qty title description:20x1 discount unit_price total_incl total_base total_vat"
     editable = False
+    #~ auto_fit_column_widths = True
 
     
 
@@ -540,7 +584,7 @@ class InvoiceDetail(dd.FormLayout):
     """,label=_("General"))
     
     more = dd.Panel("""
-    id user language #project item_vat
+    id user language #project #item_vat
     intro
     """,label=_("More"))
     
@@ -602,7 +646,8 @@ class InvoicesByJournal(Invoices):
     
 class SignAction(actions.Action):
     label = "Sign"
-    def run_from_ui(self,obj,ar):
+    def run_from_ui(self,ar):
+        obj = ar.selected_rows[0]
         def ok():
             for row in ar.selected_rows:
                 row.instance.user = ar.get_user()
@@ -622,24 +667,9 @@ class DocumentsToSign(Invoices):
                   "subject:10 total_incl total_base total_vat "
     #~ actions = Invoices.actions + [ SignAction() ]
     
-    #~ def get_row_actions(self,renderer):
-        #~ l = super(Invoices,self).get_row_actions(renderer)
-        
-        #~ def sign(renderer):
-            #~ for row in renderer.selected_rows():
-                #~ row.instance.user = renderer.request.user
-                #~ row.instance.save()
-            #~ renderer.must_refresh()
-            
-        #~ l.append( ('sign', sign) )
-        #~ return l 
 
 
 
-    
-    
-
-
 class InvoicesByPartner(Invoices):
     #~ model = 'sales.Invoice'
     order_by = ["date"]
@@ -703,12 +733,12 @@ def customize_partners():
             blank=True,
             help_text=_("The default VAT regime for sales to this customer.")))
             
-    dd.inject_field(partner_model,
-        'item_vat',
-        models.BooleanField(
-            default=False,
-            help_text=_("The default item VAT setting for sales to this customer.")))
-        
+    #~ dd.inject_field(partner_model,
+        #~ 'item_vat',
+        #~ models.BooleanField(
+            #~ default=False,
+            #~ help_text=_("The default item VAT setting for sales to this customer.")))
+        #~ 
     dd.inject_field(partner_model,
         'imode',
         models.ForeignKey(InvoicingMode,blank=True,null=True))
@@ -729,12 +759,15 @@ def site_setup(site):
     #~ for t in (site.modules.partners.Partners,
               #~ site.modules.partners.Persons,
               #~ site.modules.partners.Organisations):
-    site.modules[settings.SITE.partners_app_label].Partners.add_detail_tab("sales",
-        """
-        payment_term vat_regime item_vat imode
-        sales.InvoicesByPartner
-        """,
-        label=MODULE_LABEL)
+    for m in dd.models_by_base(site.modules.contacts.Partner):
+        t = m.get_default_table()
+        if not hasattr(t.detail_layout,'sales'):
+            t.add_detail_tab("sales", """
+            invoicing_address vat_regime imode payment_term 
+            sales.InvoicesByPartner
+            """,
+            label=MODULE_LABEL)
+    #~ site.modules.contacts.Partners.add_detail_tab("sales",
     #~ if site.is_installed('tickets'):
         #~ site.modules.tickets.Projects.add_detail_tab("sales","sales.InvoicesByProject")
     #~ site.modules.lino.SiteConfigs.add_detail_tab("sales","""
@@ -756,9 +789,6 @@ def site_setup(site):
     #~ m.add_action(Invoices)
     #~ m.add_action(DocumentsToSign)
 
-def setup_my_menu(site,ui,profile,m): 
-    pass
-  
 def setup_config_menu(site,ui,profile,m): 
     m = m.add_menu("sales",MODULE_LABEL)
     m.add_action(InvoicingModes)
