@@ -50,18 +50,7 @@ from lino.core import actions
 
 class User(mixins.CreatedModified):
     """
-    Represents a User of this site.
-    
-    This version of the Users table is used on Lino sites with
-    :doc:`/topics/http_auth`. 
-    
-    Only username is required. Other fields are optional.
-    
-    There is no password field since Lino is not responsible for authentication.
-    New users are automatically created in this table when 
-    Lino gets a first request with a username that doesn't yet exist.
-    It is up to the local system administrator to manually fill then 
-    fields like first_name, last_name, email, access rights for the new user.    
+    Represents a :ddref:`users.User` of this site.
     """
     
     preferred_foreignkey_width = 15 
@@ -77,21 +66,21 @@ class User(mixins.CreatedModified):
 
     username = models.CharField(_('Username'), max_length=30, 
         unique=True, 
-        help_text=_("""
-        Required. Must be unique. 
-        """))
+        help_text=_("""Required. Must be unique."""))
         
     password = models.CharField(_('Password'), max_length=128)
         
-    profile = dd.UserProfiles.field(blank=True)
+    profile = dd.UserProfiles.field(blank=True,
+        help_text=_("Users with an empty `profile` field are considered inactive and cannot log in."))
     
+    initials = models.CharField(_('Initials'), max_length=10, blank=True)
     first_name = models.CharField(_('First name'), max_length=30, blank=True)
     last_name = models.CharField(_('Last name'), max_length=30, blank=True)
     email = models.EmailField(_('e-mail address'), blank=True)
     
     remarks = models.TextField(_("Remarks"),blank=True) # ,null=True)
     
-    language = dd.LanguageField(default=models.NOT_PROVIDED)
+    language = dd.LanguageField(default=models.NOT_PROVIDED,blank=True)
     
     if settings.SITE.is_installed('contacts'):
       
@@ -165,9 +154,13 @@ class User(mixins.CreatedModified):
                     setattr(self,n,getattr(p,n))
             #~ self.language = p.language
         if not self.language:
-            self.language = settings.SITE.DEFAULT_LANGUAGE.django_code
+            #~ self.language = settings.SITE.DEFAULT_LANGUAGE.django_code
+            self.language = settings.SITE.get_default_language() 
         if not self.password:
             self.set_unusable_password()
+        if not self.initials:
+            if self.first_name and self.last_name:
+                self.initials = self.first_name[0] + self.last_name[0]
         super(User,self).full_clean(*args,**kw)
         
     #~ def save(self,*args,**kw):
@@ -228,10 +221,10 @@ class User(mixins.CreatedModified):
 class UserDetail(dd.FormLayout):
   
     box1 = """
-    username id profile 
-    first_name last_name partner
+    username profile:20 partner
+    first_name last_name initials
     email language 
-    created modified
+    id created modified
     """
 
     main = """
@@ -252,17 +245,14 @@ class UserInsert(dd.FormLayout):
     
  
 class Users(dd.Table):
-    help_text = _("""
-    Shows the list of all users on this site.
-    
-    A User is somebody who can log into the application.
-    
-    """)
+    help_text = _("""Shows the list of all users on this site.""")
     #~ debug_actions  = True
     required = dict(user_level='manager')
     model = User
     #~ order_by = "last_name first_name".split()
     order_by = ["username"]
+    active_fields = ['partner']
+    
     #~ column_names = 'username first_name last_name is_active is_staff is_expert is_superuser *'
     column_names = 'username profile first_name last_name *'
     detail_layout = UserDetail()
@@ -297,14 +287,10 @@ class MySettings(Users):
     
 class UsersOverview(Users):
     """
-    >> from django.conf import settings
-    >> settings.SITE.startup()
-    >> print settings.SITE.modules.users.UsersOverview.to_rst()
-    
-    
-    $ python manage.py shell
-    >> from lino.modlib.users.models import UsersOverview as T; print T.to_rst()
-    
+    A variant of :ddref:`users.Users` showing only active users
+    and only some fields. 
+    This is used on demo sites in :xfile:`admin_main.py` to display the 
+    list of available users.
     """
     column_names = 'username profile language'
     exclude = dict(profile='')
