@@ -1,4 +1,4 @@
-## Copyright 2009-2012 Luc Saffre
+## Copyright 2009-2013 Luc Saffre
 ## This file is part of the Lino project.
 ## Lino is free software; you can redistribute it and/or modify 
 ## it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ from tempfile import gettempdir
 from os.path import join, abspath, dirname, normpath
 import lino
 
-from lino.ui import Site
+from lino import Site
 
 SITE = Site(globals())
 """
@@ -122,44 +122,10 @@ USE_I18N = True
 # Examples: "http://foo.com/media/", "/media/".
 #~ ADMIN_MEDIA_PREFIX = '/admin-media/'
 
-# Make this unique, and don't share it with anybody.
-SECRET_KEY = 'cqt^18t(Fb#14a@s%mbtdif+ih8fscpf8l9aw+0ivo2!3c(c%&'
-
-#~ ROOT_URLCONF = 'lino.ui.extjs3.urls'
-
-#~ TEMPLATE_DIRS = [
-      #~ join(LINO.project_dir,'templates'),
-      #~ join(abspath(dirname(lino.__file__)),'templates'),
-#~ ]
-#print "baz", __file__
-
-#~ INSTALLED_APPS = [
-  #~ 'lino.modlib.users',
-  #~ 'lino',
-#~ ]
-
-
-#~ INSTALLED_APPS = []
-
-
-
-
-#print "done", __file__
 
 EMAIL_HOST = "mail.example.com"
 #EMAIL_PORT = ""
 
-
-#~ QOOXDOO_PATH = None
-#~ """
-#~ Path to the Qooxdoo SDK. Used by :term:`makeui`
-#~ """
-
-# this setting will be overridden by local settings
-# it is needed for generating docs. Sphinx's autodoc needs to import the modules to introspect them, but Django would complain if no settings is specified.
-
-#~ DATABASES = None
-
 DATABASES = {
       'default': {
           'ENGINE': 'django.db.backends.sqlite3',
@@ -168,3 +134,4 @@ DATABASES = {
   }
 
 
+SECRET_KEY = "20227" # see :djangoticket:`20227`
