@@ -1,4 +1,4 @@
-from north.dbutils import set_language
+#~ from north.dbutils import set_language
 from django.conf import settings
 settings.SITE.startup()
 globals().update(settings.SITE.modules)
