@@ -1,6 +1,6 @@
 SETUP_INFO = dict(name = 'lino',
-  version = '1.6.11',
-  install_requires = ['North==0.1.5','djangosite>=0.1.5','PyYAML','odfpy','jinja2','appy'],
+  version = '1.6.12', # released 20130911
+  install_requires = ['North','djangosite','odfpy','jinja2','appy','pytidylib','PyYAML'],
   description = "A framework for writing desktop-like web applications using Django and ExtJS",
   license = 'GPL',
   include_package_data = True,
@@ -32,6 +32,10 @@ SETUP_INFO.update(long_description = """\
 Lino is a high-level framework for writing desktop-like customized 
 database applications based on `Django <https://www.djangoproject.com/>`_
 and `Sencha ExtJS <http://www.sencha.com/products/extjs/>`_.
+A Lino application is technically just a Django project, 
+but the application developer does not need to write any 
+URLconf, HTML nor CSS (`more <http://lino-framework.org/about>`_)
+
 
 Examples of Lino applications are
 `Lino-Welfare <http://welfare.lino-framework.org>`__,
@@ -39,10 +43,6 @@ Examples of Lino applications are
 or
 `Lino-Faggio <http://faggio.lino-framework.org>`__.
 
-A Lino application is technically just a Django project, 
-but the application developer does not need to write any 
-URLconf, HTML nor CSS (`more <http://lino-framework.org/about>`_)
-
 Lino is based on three other projects by the same author:
 `atelier <http://atelier.lino-framework.org>`__,
 `djangosite <http://site.lino-framework.org>`__
@@ -63,8 +63,11 @@ lino.modlib.contacts
 lino.modlib.contacts.fixtures
 lino.modlib.countries
 lino.modlib.countries.fixtures
+lino.modlib.families
 lino.modlib.finan
 lino.modlib.finan.fixtures
+lino.modlib.languages
+lino.modlib.languages.fixtures
 lino.modlib.ledger
 lino.modlib.ledger.fixtures
 lino.modlib.links
@@ -181,7 +184,7 @@ SETUP_INFO.update(message_extractors = {
         ('**.py',                'python', None),
         ('**/linoweb.js',        'jinja2', None),
         #~ ('**.js',                'javascript', None),
-        ('**/templates_jinja/**.html', 'jinja2', None),
+        ('**/config/**.html', 'jinja2', None),
         #~ ('**/templates/**.txt',  'genshi', {
             #~ 'template_class': 'genshi.template:TextTemplate'
         #~ })
@@ -198,6 +201,7 @@ add_package_data('lino','config/*.odt')
 add_package_data('lino.modlib.cal','config/*.odt')
 add_package_data('lino.modlib.notes','config/notes/Note/*.odt')
 add_package_data('lino.modlib.outbox','config/outbox/Mail/*.odt')
+add_package_data('lino.modlib.languages.fixtures','*.tab')
 
 l = add_package_data('lino')
 for lng in 'de fr et nl'.split():
