@@ -43,7 +43,7 @@ from lino.utils import mti
 from lino.core import choicelists
 from lino.utils.jsgen import py2js, id2js, js_code
 from lino.utils import choosers
-from lino.utils.auth import make_view_permission_handler
+from lino.core.auth import make_view_permission_handler
 
 from lino.utils.xmlgen import etree
 from lino.utils.xmlgen import html as xghtml
@@ -84,14 +84,6 @@ def has_fk_renderer(fld):
 def rpt2url(rpt):
     return '/' + rpt.app_label + '/' + rpt.__name__
 
-#~ def a2btn(a):
-    #~ return dict(
-      #~ opens_a_slave=a.opens_a_slave,
-      #~ handler=js_code("Lino.%s" % a),
-      #~ name=a.name,
-      #~ label=a.label, # 20111111
-    #~ )
-      
 def py2html(obj,name):
     for n in name.split('.'):
         obj = getattr(obj,n,"N/A")
@@ -175,6 +167,8 @@ class GridColumn(jsgen.Component):
         #~ if isinstance(editor,BooleanFieldElement):
             #~ self.editor = None
         #~ else:
+        #~ if editor.field.__class__.__name__ == "DcAmountField":
+            #~ print 20130911, editor.editable
         self.editor = editor
         if editor.grid_column_template is not None:
             self.value_template = editor.grid_column_template
@@ -267,13 +261,22 @@ class GridColumn(jsgen.Component):
               #~ if editor.name == 'user':
                 #~ print 20120919, editor.field.__class__, editor.field.editable
                 #~ print 20120919, editor.field.model, kw['editable']
+                
         else:
             kw.update(editable=False)
         jsgen.Component.__init__(self,editor.name,**kw)
     
+            
         #~ if self.editable:
             #~ editor = self.get_field_options()
         
+    def ext_options(self,**kw):
+        kw = jsgen.Component.ext_options(self,**kw)
+        if self.editor.field is not None:
+            if is_hidden_babel_field(self.editor.field):
+                kw.update(hidden=True)
+        return kw
+    
         
 class Toolbar(jsgen.Component):
     value_template = "new Ext.Toolbar(%s)"
@@ -625,6 +628,15 @@ def add_help_text(kw,help_text,title,datasource,fieldname):
                   py2js(ttt)))
             ))
         
+def is_hidden_babel_field(fld):
+    lng = getattr(fld,'_babel_language',None)
+    if lng is None: return False
+    if jsgen._for_user_profile is None: return False
+    if jsgen._for_user_profile.hidden_languages is None: return False
+    #~ if lng.name == 'nl':
+        #~ raise Exception(20130728)
+    if lng in jsgen._for_user_profile.hidden_languages: return True
+    return False
         
 class FieldElement(LayoutElement):
     """
@@ -680,6 +692,9 @@ class FieldElement(LayoutElement):
         self.add_default_value(kw)
         
         LayoutElement.__init__(self,layout_handle,field.name,**kw)
+        
+        #~ if self.field.__class__.__name__ == "DcAmountField":
+            #~ print 20130911, self.field, self.editable
 
     def value_from_object(self,obj,ar):
         """
@@ -739,6 +754,9 @@ class FieldElement(LayoutElement):
     def get_field_options(self,**kw):
         if self.xtype:
             kw.update(xtype=self.xtype)
+            
+        if is_hidden_babel_field(self.field):
+            kw.update(hidden=True)
         
         # When used as editor of an EditorGridPanel, don't set the name attribute
         # because it is not needed for grids and might conflict with fields of a 
@@ -970,6 +988,12 @@ class ChoiceListFieldElement(ChoicesFieldElement):
     Special case are choicelist fields with blank=True: these 
     must dynamicaly add a blank choice to the the choicelist.
     """
+    def __init__(self,layout_handle,field,**kw):
+        pw = field.choicelist.preferred_foreignkey_width
+        if pw is not None:
+            kw.setdefault('preferred_width',pw)
+        FieldElement.__init__(self,layout_handle,field,**kw)
+      
     def get_field_options(self,**kw):
         kw = ComboFieldElement.get_field_options(self,**kw)
         #~ kw.update(store=js_code('Lino.%s.choices' % self.field.choicelist.actor_id))
@@ -1026,23 +1050,14 @@ class ForeignKeyElement(ComplexRemoteComboFieldElement):
   
     preferred_width = 20
     
-    def __init__(self,layout_handle,field,**kw):
-    #~ def __init__(self,*args,**kw):
-        #~ print 20100903,repr(self.field.rel.to)
-        #~ assert issubclass(self.field.rel.to,dd.Model), "%r is not a model" % self.field.rel.to
-        #~ pw = getattr(field.rel.to,'_lino_preferred_width',None)
+    def unused__init__(self,layout_handle,field,**kw):
         if isinstance(field.rel.to,basestring):
             from lino.core.dbutils import resolve_model
             field.rel.to = resolve_model(field.rel.to)
-            #~ field.lino_resolve_type()
-            #~ raise Exception("20130507 %s : %s" % (field,field.rel.to))
         pw = field.rel.to.preferred_foreignkey_width
         if pw is not None:
             kw.setdefault('preferred_width',pw)
-            #~ kw.update(preferred_width=pw)
         self.actor = dbtables.get_model_report(field.rel.to)
-        #~ if self.actor.model is None:
-            #~ raise Exception("20120621 ForeignKeyElement for %s.%s" % (self.actor,field))
         a = self.actor.detail_action
         if a is not None:
             if not isinstance(layout_handle.layout,layouts.ListLayout):
@@ -1052,22 +1067,34 @@ class ForeignKeyElement(ComplexRemoteComboFieldElement):
                     #~ "Lino.show_fk_detail_handler(this,Lino.%s)}" % a))
         FieldElement.__init__(self,layout_handle,field,**kw)
       
-    #~ def submit_fields(self):
-        #~ return [self.field.name,self.field.name+ext_requests.CHOICES_HIDDEN_SUFFIX]
-        
         
     def get_field_options(self,**kw):
         kw = super(ForeignKeyElement,self).get_field_options(**kw)
-        kw.update(pageSize=self.actor.page_length)
-        if self.actor.model is not None:
-            kw.update(emptyText=_('Select a %s...') % self.actor.model._meta.verbose_name)
+        if isinstance(self.field.rel.to,basestring):
+            raise Exception("20130827 %s.rel.to is %r" % (self.field,self.field.rel.to))
+        pw = self.field.rel.to.preferred_foreignkey_width
+        if pw is not None:
+            kw.setdefault('preferred_width',pw)
+        #~ actor = dbtables.get_model_report(self.field.rel.to)
+        actor = self.field.rel.to.get_default_table()
+        a = actor.detail_action
+        if a is not None:
+            if not isinstance(self.layout_handle.layout,layouts.ListLayout):
+                self.value_template = "new Lino.TwinCombo(%s)"
+                kw.update(onTrigger2Click=js_code(
+                    "function(e){ Lino.show_fk_detail(this,Lino.%s)}" % a.full_name()))
+                    #~ "Lino.show_fk_detail_handler(this,Lino.%s)}" % a))
+        
+        kw.update(pageSize=actor.page_length)
+        if actor.model is not None:
+            kw.update(emptyText=_('Select a %s...') % actor.model._meta.verbose_name)
         return kw
 
     def cell_html(self,ui,row):
         obj = getattr(row,self.field.name)
         if obj is None:
             return ''
-        return ui.href_to(obj)
+        return ui.obj2html(obj)
 
     def value2html(self,ar,v,**cellattrs):
         #~ return ar.renderer.obj2html(ar,v)
@@ -1363,7 +1390,7 @@ class DisplayElement(FieldElement):
         return self.field.value_from_object(obj,ar)
         
     def format_value(self,ar,v):
-        from lino.utils.xmlgen.html import E
+        #~ from lino.utils.xmlgen.html import E
         from lino.utils.xmlgen.html import html2rst
         if etree.iselement(v): 
             return html2rst(v)
@@ -1383,6 +1410,11 @@ class BooleanDisplayElement(BooleanMixin,DisplayElement):
     preferred_width = 20
     preferred_height = 1
 
+    def __init__(self,*args,**kw):
+        # do not call DisplayElement.__init__()
+        #~ kw.setdefault('value','<br/>') # see blog/2012/0527
+        #~ kw.update(always_enabled=True)
+        FieldElement.__init__(self,*args,**kw)
     
                 
 class BooleanFieldElement(BooleanMixin,FieldElement):
@@ -1523,7 +1555,7 @@ class HtmlBoxElement(DisplayElement):
         #~ LayoutElement.__init__(self,layout_handle,name,**kw)
         
     def get_field_options(self,**kw):
-        kw.update(master_panel=js_code("this"))
+        #~ kw.update(master_panel=js_code("this"))
         kw.update(name=self.field.name)
         kw.update(containing_panel=js_code("this"))
         kw.update(layout='fit')
@@ -1533,7 +1565,7 @@ class HtmlBoxElement(DisplayElement):
         # for this trick thanks to Vladimir 
         # <http://forums.ext.net/showthread.php?1513-CLOSED-Autoscroll-on-ext-panel>
         #~ kw.update(bodyStyle="overflow-x:hidden !important;")
-        kw.update(bodyStyle="overflow-x:hidden")
+        kw.update(bodyStyle="overflow-x:hidden;")
         
         #~ if self.field.drop_zone: # testing with drop_zone 'FooBar'
             #~ kw.update(listeners=dict(render=js_code('initialize%sDropZone' % self.field.drop_zone)))
@@ -1696,9 +1728,14 @@ class Container(LayoutElement):
         return False
         
 class Wrapper(VisibleComponent):
+    """
+    """
     #~ label = None
     def __init__(self,e,**kw):
         kw.update(layout='form')
+        #~ if e.value.get('autoHeight',False): # since 20130924
+            #~ kw.update(autoHeight=True)
+        #~ if False: # since 20130924
         if not isinstance(e,TextFieldElement):
             kw.update(autoHeight=True)
         #~ kw.update(labelAlign=e.parent.labelAlign)
@@ -1709,6 +1746,7 @@ class Wrapper(VisibleComponent):
         for n in ('width', 'height', 'preferred_width','preferred_height','vflex','loosen_requirements'):
             setattr(self,n,getattr(e,n))
         #~ e.update(anchor="100%")
+            
         if e.vflex: 
             #~ 20120630 e.update(anchor="100% 100%")
             #~ e.update(anchor="-25 -25")
@@ -1717,6 +1755,7 @@ class Wrapper(VisibleComponent):
             #~ e.update(anchor="100%")
             #~ e.update(anchor="-25")
             e.update(anchor=FULLWIDTH)
+            e.update(autoHeight=True) # 20130924
         #~ e.update(padding=DEFAULT_PADDING)
         #~ self.allow_read = e.allow_read
         #~ self.get_view_permission = e.get_view_permission
@@ -1737,6 +1776,14 @@ class Wrapper(VisibleComponent):
         for chunk in self.wrapped.as_plain_html(ar,obj):
             yield chunk
             
+    def ext_options(self,**kw):
+        kw = super(Wrapper,self).ext_options(**kw)
+        if self.wrapped.field is not None:
+            if is_hidden_babel_field(self.wrapped.field):
+                kw.update(hidden=True)
+                #~ print("20130827 hidden %s" % self.wrapped.field)
+        return kw
+    
 class Panel(Container):
     """
     A vertical Panel is vflex if and only if at least one of its children is vflex.
@@ -1787,50 +1834,6 @@ class Panel(Container):
                 # so we must split this panel into several containers.
                 # vflex elements go into a vbox, the others into a form layout. 
                 
-                if False:
-                    # Rearrange elements into "element groups"
-                    # Each element of egroups is a list of elements who have same vflex
-                    egroups = []
-                    for e in elements:
-                        if len(egroups) and egroups[-1][0].vflex == e.vflex:
-                            egroups[-1].append(e)
-                        else:
-                            egroups.append([e])
-                            
-                    if len(egroups) == 1:
-                        # all elements have same vflex
-                        assert tuple(egroups[0]) == elements, "%r != %r" % (tuple(egroups[0]), elements)
-                        
-                    elements = []
-                    for eg in egroups:
-                        if eg[0].vflex:
-                            #~ for e in eg: e.update(flex=1,align='stretch')
-                            for e in eg: 
-                                e.update(flex=1)
-                                e.collapsible = True
-                                #~ e.update(collapsible=True)
-                            if len(eg) == 1:
-                                g = eg[0]
-                            else:
-                                #~ g = Container(layout_handle,name,*eg,**dict(layout='vbox',flex=1)
-                                g = Panel(layout_handle,name,vertical,*eg,**dict(layout='vbox',
-                                  flex=1,layoutConfig=dict(align='stretch')))
-                                assert g.vflex is True
-                        else:
-                            #~ for e in eg: e.update(align='stretch')
-                            if len(eg) == 1:
-                                g = eg[0]
-                            else:
-                                g = Container(layout_handle,name,*eg,**dict(layout='form',autoHeight=True))
-                                #~ g = Container(layout_handle,name,*eg,**dict(layout='form'))
-                                assert g.vflex is False
-                        #~ if monitorResize:
-                            #~ g.update(monitorResize=True)
-                        #~ g.update(align='stretch')
-                        #~ g.update(layoutConfig=dict(align='stretch'))
-                        elements.append(g)
-                    kw.update(layout='vbox',layoutConfig=dict(align='stretch'))
-                    #~ self.elements = elements
             else: # not self.vertical
                 kw.update(layout='hbox',layoutConfig=dict(align='stretch'))
               
@@ -1846,21 +1849,8 @@ class Panel(Container):
                         self.label_width = w
 
 
-        if False: # not yet converted to new dtl syntax 20120214
-            label = layout_handle.layout.collapsible_elements.get(name,None)
-            if label:
-                self.collapsible = True
-                self.label = label
-            
-        #~ if str(layout_handle.layout) == 'ClientDetail on pcsw.Clients':
-            #~ if name == 'cbss':
-                #~ print '20120925 ext_elems', name, kw
-
         Container.__init__(self,layout_handle,name,*elements,**kw)
         
-        #~ if name == 'cbss':
-            #~ logger.info("20120925 Panel.__init__() 2 %r",self.required)
-        
         w = h = 0
         has_height = False # 20120210
         for e in self.elements:
@@ -1889,6 +1879,13 @@ class Panel(Container):
         if not d.has_key('layout'):
             if len(self.elements) == 1:
                 d.update(layout='fit')
+                #~ """
+                #~ before 20130924 it was always 'fit' here. But 
+                #~ """
+                #~ if self.elements[0].vflex or not self.elements[0].value.get('autoHeight',False):
+                    #~ d.update(layout='fit')
+                #~ else:
+                    #~ d.update(layout='form')
             elif self.vertical:
                 #~ d.update(layout='form')
                 if self.vflex:
@@ -1902,6 +1899,8 @@ class Panel(Container):
                 d.update(layout='hbox',autoHeight=True) # 20101028
                 
         if d['layout'] == 'form':
+            #~ if not self.vflex:
+                #~ d.update(autoHeight=True) # since 20130924
             assert self.vertical
             #~ self.update(labelAlign=self.labelAlign)
             self.update(labelAlign=self.label_align)
@@ -1911,6 +1910,7 @@ class Panel(Container):
                 #~ 20120630 self.elements[0].update(anchor="100% 100%")
                 #~ self.elements[0].update(anchor="-25 -25")
                 self.elements[0].update(anchor=FULLWIDTH + ' ' + FULLHEIGHT)
+                #~ self.update(autoHeight=self.elements[0].autoHeight) # since 20130924
                 
             else:
                 for e in self.elements:
@@ -2042,6 +2042,7 @@ class Panel(Container):
             Panels which are usually not vertical but still want a frame 
             since they are the main panel.
             """
+            #~ d.update(autoHeight=True) # since 20130924
             d.update(frame=True)
             d.update(bodyBorder=False)
             d.update(border=False)
@@ -2098,6 +2099,8 @@ class GridElement(Container):
         #~ kw.update(viewConfig=vc)
         
         kw.setdefault('label',rpt.label)
+        if len(self.columns) == 1:
+            kw.setdefault('hideHeaders',True)
         
         add_help_text(kw,rpt.help_text,rpt.title or rpt.label,rpt.app_label,rpt.actor_id)
         
@@ -2142,56 +2145,6 @@ class GridElement(Container):
             yield th
       
 
-    def row2html(self,ar,columns,row,sums,**cellattrs):
-        #~ logger.info("20130123 row2html %s",fields)
-        #~ for i,fld in enumerate(self.list_fields):
-        has_numeric_value = False
-        cells = []
-        for i,col in enumerate(columns):
-            #~ if fld.name == 'person__gsm':
-            #~ logger.info("20120406 Store.row2list %s -> %s", fld, fld.field)
-            #~ import pdb; pdb.set_trace()
-            v = col.field._lino_atomizer.full_value_from_object(row,ar)
-            if v is None:
-                td = E.td(**cellattrs)
-            else:
-                nv = col.value2num(v)
-                if nv != 0:
-                    try:
-                        sums[i] += nv
-                    except TypeError as e:
-                        raise Exception("Cannot compute %r + %r" % (sums[i],nv))
-                    has_numeric_value = True
-                td = col.value2html(ar,v,**cellattrs)
-            col.apply_cell_format(td)
-            cells.append(td)
-        if ar.actor.hide_zero_rows and not has_numeric_value:
-            return None
-        return cells
-            
-    def row2text(self,ar,fields,row,sums):
-        for i,fld in enumerate(fields):
-            if fld.field is not None:
-                try: # was used to find bug 20130422
-                    v = fld.field._lino_atomizer.full_value_from_object(row,ar)
-                except Exception as e:
-                    v = "%s:\n%s" % (fld.field,e)
-                if v is None:
-                    yield ''
-                else:
-                    sums[i] += fld.value2num(v)
-                    yield fld.format_value(ar,v)
-                
-    def sums2html(self,ar,columns,sums,**cellattrs):
-        #~ return [fld.format_sum(ar,sums,i)
-          #~ for i,fld in enumerate(fields)]
-        return [fld.sum2html(ar,sums,i,**cellattrs)
-          for i,fld in enumerate(columns)]
-      
-        #~ return [fld.sum2html(ar.ui,sums[i])
-          #~ for i,fld in enumerate(fields)]
-            
-            
     def as_plain_html(self,ar,obj):
         sar = ar.spawn(actor=self.actor,action=self.actor.default_action,master_instance=obj)
         yield sar.as_html()
@@ -2236,17 +2189,6 @@ class ParamsPanel(Panel):
     #~ value_template = "new Ext.form.FormPanel(%s)"
     #~ value_template = "new Ext.form.FormPanel({layout:'fit', autoHeight: true, frame: true, items:new Ext.Panel(%s)})"
     value_template = "%s"
-    #~ pass
-    def unused__init__(self,lh,name,vertical,*elements,**kw):
-        Panel.__init__(self,lh,name,vertical,*elements,**kw)
-        
-        #~ fkw = dict(layout='fit', autoHeight= True, frame= True, items=pp)
-        if lh.layout._datasource.params_panel_hidden:
-            self.value_template = "new Ext.form.FormPanel({hidden:true, layout:'fit', autoHeight: true, frame: true, items:new Ext.Panel(%s)})"
-          
-            #~ fkw.update(hidden=True)
-        #~ self.value = 
-        #~ return ext_elems.FormPanel(**fkw)
     
 
 #~ class ParamsForm(Panel):
