@@ -72,15 +72,13 @@ from lino.core import actors
     
 from lino.utils import choosers
 from lino.core import choicelists
-from lino.core import menus
 from lino.utils import jsgen
-from lino.utils.jsgen import py2js, js_code, id2js
 from lino.utils.xmlgen import html as xghtml
-from lino.utils.config import make_dummy_messages_file
+from lino.utils.xmlgen.html import E
 from lino.utils import codetime
 
-from lino.utils.jscompressor import JSCompressor
 if False:
+    from lino.utils.jscompressor import JSCompressor
     jscompress = JSCompressor().compress
 else:    
     def jscompress(s): return s
@@ -107,6 +105,7 @@ def add_user_language(kw,ar):
         kw.setdefault(ext_requests.URL_PARAM_USER_LANGUAGE,lang)
 
 
+class NOT_GIVEN: pass
 
 
 class HtmlRenderer(object):
@@ -114,23 +113,34 @@ class HtmlRenderer(object):
     Deserves more documentation.
     """
     is_interactive = False
+    row_classes_map = {}
     
     def __init__(self,ui):
         self.ui = ui
         
     def href(self,url,text):
-        return '<a href="%s">%s</a>' % (url,text)
+        #~ return '<a href="%s">%s</a>' % (url,text)
+        return E.a(text,href=url)
         
-          
-    def href_button_action(self,ba,*args,**kw):
-        if ba.action.icon_file is not None:
-            kw.update(icon_file=ba.action.icon_file)
-            kw.update(style="vertical-align:-30%;")
-        return self.href_button(*args,**kw)
+    def show(self,ar,**kw):
+        return ar.table2xhtml(**kw)
+        #~ return E.tostring(ar.table2xhtml())
         
-    def href_button(self,url,text,title=None,target=None,icon_file=None,**kw):
+        
+    def href_button_action(self,ba,url,text=None,title=None,icon_name=NOT_GIVEN,**kw):
         """
-        Returns an elementtree object of a "button-like" ``<a href>`` tag.
+        changed 20130905 for "Must read eID card button"
+        but that caused icons to not appear in workflow_buttons.
+        """
+        if icon_name is NOT_GIVEN:
+            icon_name = ba.action.icon_name
+        if icon_name is not None and not kw.has_key('style'):
+            kw.update(style="vertical-align:-30%;")
+        return self.href_button(url,text,title,icon_name=icon_name,**kw)
+        
+    def href_button(self,url,text,title=None,target=None,icon_name=None,**kw):
+        """
+        Returns an etree object of a "button-like" ``<a href>`` tag.
         """
         #~ logger.info('20121002 href_button %r',unicode(text))
         if target:
@@ -141,7 +151,7 @@ class HtmlRenderer(object):
             #~ return xghtml.E.a(text,href=url,title=title)
         kw.update(href=url)
         #~ if icon_name:
-        if icon_file:
+        if icon_name is not None:
             #~ btn = xghtml.E.button(type='button',class_='x-btn-text '+icon_name)
             #~ btn = xghtml.E.button(
                 #~ type='button',
@@ -150,12 +160,28 @@ class HtmlRenderer(object):
             #~ return btn
             #~ return xghtml.E.a(btn,**kw)
             #~ kw.update(class_='x-btn-text '+icon_name)
-            img = xghtml.E.img(src=settings.SITE.build_media_url('lino','extjs','images','mjames',icon_file))
+            img = xghtml.E.img(src=settings.SITE.build_media_url('lino','extjs','images','mjames',icon_name+'.png'))
             return xghtml.E.a(img,**kw)
         else:
             #~ return xghtml.E.span('[',xghtml.E.a(text,**kw),']')
             #~ kw.update(style='border-width:1px; border-color:black; border-style:solid;')
             return xghtml.E.a(text,**kw)
+            
+    def insert_button(self,ar,text,known_values={},**options):
+        """
+        Returns the HTML of a button which will call the `insert_action`.
+        """
+        a = ar.actor.insert_action
+        if a is None:
+            #~ raise Exception("20130924 a is None")
+            return
+        if not a.get_bound_action_permission(ar,ar.master_instance,None):
+            #~ raise Exception("20130924 no permission")
+            return
+        elem = ar.create_instance(**known_values)
+        st = ar.get_status()
+        st.update(data_record=views.elem2rec_insert(ar,ar.ah,elem))
+        return self.window_action_button(ar.request,a,st,text,**options)
         
     def quick_add_buttons(self,ar):
         """
@@ -168,37 +194,23 @@ class HtmlRenderer(object):
         See also :doc:`/tickets/56`.
         
         """
-        s = ''
-        #~ params = dict(base_params=ar.request2kw(self))
-        params = None
-        after_show = ar.get_status(self)
-        
-        #~ params = ar.get_status(self)
-        #~ after_show = dict()
-        #~ a = ar.actor.get_url_action('insert_action')
         buttons = []
-        a = ar.actor.insert_action
-        if a is not None:
-            if a.get_bound_action_permission(ar,ar.master_instance,None):
-                elem = ar.create_instance()
-                after_show.update(data_record=views.elem2rec_insert(ar,ar.ah,elem))
-                #~ after_show.update(record_id=-99999)
-                # see tickets/56
-                #~ s += self.window_action_button(a,after_show,_("New"))
-                buttons.append(self.window_action_button(ar.request,a,after_show,_("New")))
-                #~ buttons.append(self.action_button(ar.request,a,after_show,_("New")))
-                buttons.append(' ')
-                after_show = ar.get_status(self)
+        btn = ar.insert_button(_("New"))
+        if btn is not None:
+            buttons.append(btn)
+            buttons.append(' ')
+                #~ after_show = ar.get_status()
         n = ar.get_total_count()
         #~ print 20120702, [o for o in ar]
         if n > 0:
             obj = ar.data_iterator[n-1]
-            after_show.update(record_id=obj.pk)
+            st = ar.get_status()
+            st.update(record_id=obj.pk)
             #~ a = ar.actor.get_url_action('detail_action')
             a = ar.actor.detail_action
             buttons.append(self.window_action_button(
-                ar.request,a,after_show,_("Show Last"),
-                icon_file = 'application_form.png',
+                ar.request,a,st,_("Show Last"),
+                icon_name = 'application_form',
                 title=_("Show the last record in a detail window")))
             buttons.append(' ')
             #~ s += ' ' + self.window_action_button(
@@ -206,7 +218,7 @@ class HtmlRenderer(object):
             #~ s += ' ' + self.href_to_request(ar,"[%s]" % unicode(_("Show All")))
             buttons.append(self.href_to_request(None,ar,
               _("Show All"),
-              icon_file = 'application_view_list.png',
+              icon_name = 'application_view_list',
               title=_("Show all records in a table window")))
         #~ return '<p>%s</p>' % s
         return xghtml.E.p(*buttons)
@@ -214,36 +226,36 @@ class HtmlRenderer(object):
     def quick_upload_buttons(self,rr):
         """
         Returns a HTML chunk that displays "quick upload buttons":
-        either one button :guilabel:`[Upload]` 
+        either one button :guilabel:`Upload` 
         (if the given :class:`TableTequest <lino.core.dbtables.TableRequest>`
         has no rows)
-        or two buttons :guilabel:`[Show]` and :guilabel:`[Edit]` 
+        or two buttons :guilabel:`Show` and :guilabel:`Edit` 
         if it has one row.
         
         See also :doc:`/tickets/56`.
         
         """
-        #~ params = dict(base_params=rr.request2kw(self))
-        #~ params = rr.get_status(self)
-        params = None
-        after_show = rr.get_status(self)
-        #~ after_show = dict(base_params=rr.get_status(self))
-        #~ after_show = dict()
         if rr.get_total_count() == 0:
-            #~ a = rr.actor.get_url_action('insert_action')
-            a = rr.actor.insert_action
-            if a is not None:
-                elem = rr.create_instance()
-                after_show.update(data_record=views.elem2rec_insert(rr,rr.ah,elem))
-                #~ after_show.update(record_id=-99999)
-                # see tickets/56
-                return self.window_action_button(rr.request,a,after_show,_("Upload"),
-                  #~ icon_file='attach.png',
-                  #~ icon_file='world_add.png',
-                  icon_file='page_add.png',
-                  title=_("Upload a file from your PC to the server."))
-                  #~ icon_name='x-tbar-upload')
+            if True: # after 20130809
+                return rr.insert_button(_("Upload"),
+                    icon_name='page_add',
+                    title=_("Upload a file from your PC to the server."))
+            else:
+                a = rr.actor.insert_action
+                if a is not None:
+                    after_show = rr.get_status()
+                    elem = rr.create_instance()
+                    after_show.update(data_record=views.elem2rec_insert(rr,rr.ah,elem))
+                    #~ after_show.update(record_id=-99999)
+                    # see tickets/56
+                    return self.window_action_button(rr.request,a,after_show,_("Upload"),
+                      #~ icon_file='attach.png',
+                      #~ icon_file='world_add.png',
+                      icon_name='page_add',
+                      title=_("Upload a file from your PC to the server."))
+                      #~ icon_name='x-tbar-upload')
         if rr.get_total_count() == 1:
+            after_show = rr.get_status()
             obj = rr.data_iterator[0]
             chunks = []
             #~ chunks.append(xghtml.E.a(_("show"),
@@ -252,14 +264,15 @@ class HtmlRenderer(object):
                 settings.SITE.build_media_url(obj.file.name),_("show"),
                 target='_blank',
                 #~ icon_file='world_go.png',
-                icon_file='page_go.png',
+                icon_name='page_go',
+                style="vertical-align:-30%;",
                 title=_("Open the uploaded file in a new browser window")))
             chunks.append(' ')
             after_show.update(record_id=obj.pk)
             chunks.append(self.window_action_button(rr.request,
                 rr.ah.actor.detail_action,
                 after_show,
-                _("Edit"),icon_file='application_form.png',title=_("Edit metadata of the uploaded file.")))
+                _("Edit"),icon_name='application_form',title=_("Edit metadata of the uploaded file.")))
             return xghtml.E.p(*chunks)
             
             #~ s = ''
@@ -283,6 +296,15 @@ class HtmlRenderer(object):
                 return xghtml.E.a(text,href=url)
         return xghtml.E.b(text)
         
+    def window_action_button(self,request,ba,after_show={},label=None,title=None,**kw):
+        """
+        Return a HTML chunk for a button that will execute this action.
+        """
+        label = unicode(label or ba.get_button_label())
+        url = 'javascript:'+self.action_call(request,ba,after_show)
+        #~ logger.info('20121002 window_action_button %s %r',a,unicode(label))
+        return self.href_button_action(ba,url,label,title or ba.action.help_text,**kw)
+        
 class TextRenderer(HtmlRenderer):
     user = None
     
@@ -307,17 +329,6 @@ class TextRenderer(HtmlRenderer):
         """
         print ar.to_rst(*args,**kw)
         
-    def unused_request(self,actor=None,**kw):
-        #~ username = kw.pop('username',None)
-        #~ if username:
-            #~ self.login(username)
-        #~ if self.user:
-            #~ kw.setdefault('user',self.user)
-        kw.setdefault('user',settings.SITE.console_user)
-        kw.update(renderer=self)
-        #~ raise Exception(kw)
-        return actor.request(**kw)
-        
   
 class PlainRenderer(HtmlRenderer):
     """
@@ -331,17 +342,17 @@ class PlainRenderer(HtmlRenderer):
     def instance_handler(self,ar,obj,**kw):
         a = getattr(obj,'_detail_action',None)
         if a is None:
-            a = obj.__class__._lino_default_table.detail_action
+            a = obj.__class__.get_default_table().detail_action
         if a is not None:
             if ar is None or a.get_bound_action_permission(ar,obj,None):
                 add_user_language(kw,ar)
                 return self.get_detail_url(obj,**kw)
   
-    def href_to(self,ar,obj,text=None):
-        h = self.instance_handler(ar,obj)
-        if h is None:
-            return cgi.escape(force_unicode(obj))
-        return self.href(url,text or cgi.escape(force_unicode(obj)))
+    #~ def href_to(self,ar,obj,text=None):
+        #~ h = self.instance_handler(ar,obj)
+        #~ if h is None:
+            #~ return cgi.escape(force_unicode(obj))
+        #~ return self.href(url,text or cgi.escape(force_unicode(obj)))
         
     def pk2url(self,ar,pk,**kw):
         if pk is not None:
@@ -357,7 +368,7 @@ class PlainRenderer(HtmlRenderer):
         return settings.SITE.build_plain_url(obj._meta.app_label,obj.__class__.__name__,str(obj.pk),*args,**kw)
         
     def get_request_url(self,ar,*args,**kw):
-        st = ar.get_status(self)
+        st = ar.get_status()
         kw.update(st['base_params'])
         add_user_language(kw,ar)
         #~ since 20121226 kw.setdefault(ext_requests.URL_PARAM_FORMAT,ext_requests.URL_FORMAT_PLAIN)
@@ -382,15 +393,6 @@ class PlainRenderer(HtmlRenderer):
         label = label or ba.action.label
         return label
       
-    def window_action_button(self,request,ba,after_show={},label=None,title=None,**kw):
-        """
-        Return a HTML chunk for a button that will execute this action.
-        """
-        label = unicode(label or ba.get_button_label())
-        url = 'javascript:'+self.action_call(request,ba,after_show)
-        #~ logger.info('20121002 window_action_button %s %r',a,unicode(label))
-        return self.href_button_action(ba,url,label,title or ba.action.help_text,**kw)
-        
     def action_call(self,request,bound_action,after_show):  
         return "oops"
 
