@@ -191,7 +191,8 @@ class StoreField(object):
             
     def format_value(self,ar,v):
         """
-        Return a plain textual representation of this value as a unicode string.
+        Return a plain textual representation as a unicode string
+        of the given value `v`.   Note that `v` might be `None`.
         """
         return force_unicode(v)
         
@@ -273,7 +274,7 @@ class ForeignKeyStoreField(RelatedMixin,ComboStoreField):
         #~ obj = self.full_value_from_object(req,row)
         #~ if obj is None:
             #~ return ''
-        #~ return req.ui.href_to(obj)
+        #~ return req.ui.obj2html(obj)
         
         
     def get_value_text(self,v,obj):
@@ -376,6 +377,8 @@ class RequestStoreField(StoreField):
         #~ d[self.field.name] = v
 
     def format_value(self,ar,v):
+        if v is None:
+            return ''
         return str(v.get_total_count())
         
     #~ def sum2html(self,ar,sums,i,**cellattrs):
@@ -499,6 +502,15 @@ class DisabledActionsStoreField(SpecialStoreField):
     #~ def full_value_from_object(self,request,obj):
         #~ return 
         
+class RowClassStoreField(SpecialStoreField):
+    name = 'row_class'
+        
+    def full_value_from_object(self,obj,ar):
+        return ' '.join([ar.renderer.row_classes_map.get(s,'') 
+            for s in self.store.actor.get_row_classes(obj,ar)])
+        #~ return ar.renderer.row_classes_map.get('x-grid3-row-%s' % s
+            
+        
 class DisableEditingStoreField(SpecialStoreField):
     """
     A field whose value is the result of the `get_row_permission` 
@@ -616,14 +628,14 @@ class GenericForeignKeyField(DisplayStoreField):
         if v is None: return ''
         if ar is None: return unicode(v)
         if ar.renderer is None: return unicode(v)
-        return ar.href_to(v)
+        return ar.obj2html(v)
         #~ owner = getattr(obj,self.name)
         #~ if owner is None: 
             #~ # owner_id = getattr(obj,self.field.fk_field)
             #~ # if owner_id is None:
                 #~ # return ''
             #~ return ''
-        #~ return ar.href_to(owner)
+        #~ return ar.obj2html(owner)
   
 
 class unused_GenericForeignKeyField(StoreField):
@@ -638,7 +650,7 @@ class unused_GenericForeignKeyField(StoreField):
             #~ # if owner_id is None:
                 #~ # return ''
             #~ return ''
-        #~ return ar.href_to(owner)
+        #~ return ar.obj2html(owner)
   
     def value2list(self,ar,v,l,row):
         return l.append(unicode(v))
@@ -830,13 +842,18 @@ def create_atomizer(fld,name):
         """
         sf = create_atomizer(fld.field,fld.name)
         def value_from_object(sf,obj,ar):
+            #~ if fld.name == 'event__when_text':
+                #~ logger.info("20130802 create_atomizer RemoteField value_from_object")
             m = fld.func
-            return m(obj)
+            return m(obj,ar)
             
         def full_value_from_object(sf,obj,ar):
             #~ logger.info("20120406 %s.full_value_from_object(%s)",sf.name,sf)
             m = fld.func
-            return m(obj)
+            v = m(obj,ar)
+            #~ if fld.name == 'event__when_text':
+                #~ logger.info("20130802 full_value_from_object %s",obj)
+            return v
             
         sf.value_from_object = curry(value_from_object,sf)
         sf.full_value_from_object = curry(full_value_from_object,sf)
@@ -928,7 +945,7 @@ class ParameterStore(BaseStore):
             l.append(pv[f.field.name])
         return l
         
-    def pv2dict(self,ui,pv,**d):
+    def pv2dict(self,pv,**d):
         for fld in self.param_fields:
             v = pv.get(fld.name,None)
             fld.value2dict(v,d,None)
@@ -1025,7 +1042,7 @@ class Store(BaseStore):
             dh = form.get_layout_handle(rh.ui)
             self.collect_fields(self.detail_fields,dh)
         
-        if issubclass(rh.actor,dbtables.Table):
+        if self.pk is not None:
             self.pk_index = 0
             found = False
             for fld in self.list_fields:
@@ -1063,6 +1080,9 @@ class Store(BaseStore):
         if rh.actor.editable:
             addfield(DisableEditingStoreField(self))
             
+        if rh.actor.get_row_classes is not None:
+            addfield(RowClassStoreField(self))
+            
         #~ self.fields.append(PropertiesStoreField)
         #~ self.fields_dict = dict([(f.field.name,f) for f in self.fields])
         
@@ -1080,19 +1100,6 @@ class Store(BaseStore):
         self.detail_fields = tuple(self.detail_fields)
         
 
-    def add_field_for(self,fields,df):
-        sf = get_atomizer(df,df.name)
-            
-        if not sf in self.all_fields:
-            self.all_fields.append(sf)
-            
-        #~ sf = self.df2sf.get(df,None)
-        #~ if sf is None:
-            #~ sf = self.create_atomizer(df,df.name)
-            #~ self.all_fields.append(sf)
-            #~ self.df2sf[df] = sf
-        fields.append(sf)
-        
     def collect_fields(self,fields,*layouts):
         """
         `fields` is a pointer to either `self.detail_fields` or `self.list_fields`.
@@ -1109,14 +1116,27 @@ class Store(BaseStore):
                     if self.pk is None:
                         self.pk = df
                 #~ fields.add(fld)
-        #~ if not self.pk in fields:
-        if issubclass(self.actor,dbtables.Table):
-            if self.pk is None:
-                self.pk = self.actor.model._meta.pk
+        
+        if self.pk is None:
+            self.pk = self.actor.get_pk_field()
+        if self.pk is not None:
             if not pk_found:
                 self.add_field_for(fields,self.pk)
                 
 
+    def add_field_for(self,fields,df):
+        sf = get_atomizer(df,df.name)
+            
+        if not sf in self.all_fields:
+            self.all_fields.append(sf)
+            
+        #~ sf = self.df2sf.get(df,None)
+        #~ if sf is None:
+            #~ sf = self.create_atomizer(df,df.name)
+            #~ self.all_fields.append(sf)
+            #~ self.df2sf[df] = sf
+        fields.append(sf)
+        
     def form2obj(self,ar,form_values,instance,is_new):
         disabled_fields = set(self.actor.disabled_fields(instance,ar))
         #~ logger.info("20130107 Store.form2obj(%s)", form_values)
@@ -1185,11 +1205,7 @@ class Store(BaseStore):
         else:
             for fld in self.list_fields:
                 v = fld.full_value_from_object(row,request)
-                #~ if fld.name == 'person__age':
-                #~ logger.info("20120406 Store.row2list %s -> %s", fld, v)
                 fld.value2list(request,v,l,row)
-                #~ if fld.name == 'person__age':
-                    #~ logger.info("20120406 Store.row2list %s -> %s", fld, l)
         #~ logger.info("20130611 Store row2list() --> %r", l)
         return l
       
