@@ -46,10 +46,6 @@ from django.conf.urls import patterns, url, include
 import lino
 from lino.core import constants as ext_requests
 from . import store as ext_store
-#~ from lino.ui.extjs3 import ext_elems
-#~ from lino.ui.extjs3 import ext_store
-#~ from lino.ui.extjs3 import ext_windows
-#~ from lino.ui import requests as ext_requests
 
 from lino.core import actions 
 #~ from lino.core.actions import action2str
@@ -61,19 +57,9 @@ from lino.core import tables
 from lino.ui import base
 from lino.core import actors
     
-from lino.utils import choosers
 from lino.utils.xmlgen import html as xghtml
 from lino.utils.config import make_dummy_messages_file
 
-from lino.utils.jscompressor import JSCompressor
-if False:
-    jscompress = JSCompressor().compress
-else:    
-    def jscompress(s): return s
-      
-from lino.mixins import printable
-
-
 
 #~ from lino.utils.choicelists import DoYouLike, HowWell
 #~ STRENGTH_CHOICES = DoYouLike.get_choices()
@@ -100,7 +86,9 @@ def parse_int(s,default=None):
     
 class ExtUI(base.UI):
     """
-    The central instance of Lino's ExtJS3 User Interface.
+    The central instance of Lino's User Interface.
+    Not only ExtJS (despite the name).
+    
     """
     #~ _handle_attr_name = '_extjs3_handle'
     #~ _handle_attr_name = '_lino_ui_handle'
@@ -126,10 +114,6 @@ class ExtUI(base.UI):
         #~ logger.info("20130221 lino.ui.ExtUI.__init__()")
         pre_ui_build.send(self)
         
-        from lino.utils import codetime
-        self.mtime = codetime()
-        #~ logger.info("20130610 codetime is %s", datetime.datetime.fromtimestamp(self.mtime))
-        
         #~ raise Exception("20120614")
         #~ self.pdf_renderer = PdfRenderer(self) # 20120624
         from lino.ui.render import PlainRenderer, TextRenderer
@@ -156,6 +140,10 @@ class ExtUI(base.UI):
                 #~ if ba.action.parameters:
                     #~ ba.action.params_layout.get_layout_handle(self)
         
+        from lino.utils import codetime
+        self.mtime = codetime()
+        #~ logger.info("20130610 codetime is %s", datetime.datetime.fromtimestamp(self.mtime))
+        
         post_ui_build.send(self)
         
         #~ """
@@ -247,78 +235,16 @@ class ExtUI(base.UI):
             #~ if issubclass(h.report,dbtables.EmptyTable):
                 #~ h.store = ext_store.Store(h)
           
-    def table2xhtml(self,ar,max_row_count=300,**kw):
-        #~ doc = xghtml.Document(force_unicode(ar.get_title()))
-        #~ t = doc.add_table()
-        t = xghtml.Table()
-        self.ar2html(ar,t,ar.data_iterator,**kw)
-        # return xghtml.E.tostring(t.as_element())
-        return t.as_element()
-        
-    def ar2html(self,ar,tble,data_iterator,column_names=None):
-        """
-        Render the given ActionRequest ar to html
-        """
-        tble.attrib.update(cellspacing="3px",bgcolor="#ffffff", width="100%")
-        #~ tble.attrib.update(cellspacing="3px",bgcolor="#d0def0", width="100%")
-        
-        grid = ar.ah.list_layout.main
-        columns = grid.columns
-        fields, headers, cellwidths = ar.get_field_info(column_names)
-        columns = fields
-        #~ print 20130330, cellwidths
-          
-        if ar.renderer.is_interactive and ar.master_instance is None:
-            #~ print 20130527, ar.order_by
-            for i,e in enumerate(columns):
-                if e.sortable and ar.order_by != [e.name]:
-                    kw = {ext_requests.URL_PARAM_SORT:e.name}
-                    url = ar.renderer.get_request_url(ar,**kw)
-                    if url is not None:
-                        headers[i] = xghtml.E.a(headers[i],href=url)
-        
-        #~ cellattrs = dict(align="center",valign="middle",bgcolor="#eeeeee")
-        cellattrs = dict(align="left",valign="top",bgcolor="#eeeeee")
-        #~ cellattrs = dict(align="left",valign="top",bgcolor="#d0def0")
-        #~ cellattrs = dict()
-        
-        headers = [x for x in grid.headers2html(ar,columns,headers,**cellattrs)]
-        sums  = [fld.zero for fld in columns]
-        #~ hr = tble.add_header_row(*headers,**cellattrs)
-        if cellwidths:
-            for i,td in enumerate(headers): 
-                td.attrib.update(width=str(cellwidths[i]))
-        tble.head.append(xghtml.E.tr(*headers))
-        #~ print 20120623, ar.actor
-        recno = 0
-        for row in data_iterator:
-            cells = grid.row2html(ar,columns,row,sums,**cellattrs)
-            if cells is not None:
-                recno += 1
-                tble.body.append(xghtml.E.tr(*cells))
-            
-        if recno == 0:
-            tble.clear()
-            tble.body.append(ar.no_data_text)
-        
-        if not ar.actor.hide_sums:
-            has_sum = False
-            for i in sums:
-                if i:
-                    has_sum = True
-                    break
-            if has_sum:
-                cells = grid.sums2html(ar,columns,sums,**cellattrs)
-                tble.body.append(xghtml.E.tr(*cells))
-                #~ tble.add_body_row(*ar.ah.store.sums2html(ar,fields,sums,**cellattrs))
-            
-            
-    def action_response(self,rv):
+    def render_action_response(self,rv):
         """
         Builds a JSON response from given dict, 
         checking first whether there are only allowed keys 
         (defined in :attr:`ACTION_RESPONSES`)
         """
+        #~ if isinstance(rv,models.Model):
+            #~ js = self.ui.ext_renderer.instance_handler(None,invoice)
+            #~ rv = dict(eval_js=js)
+        
         rv = self.check_action_response(rv)
         return views.json_response(rv)
     
