@@ -40,7 +40,7 @@ from lino.utils import ucsv
 from lino.utils import choosers
 from lino.utils import isiterable
 from lino.utils import dblogger
-from lino.utils import auth
+from lino.core import auth
 
 from lino.core import actions
 from lino.core import actors
@@ -49,6 +49,8 @@ from lino.core import dbtables
 from lino.core import web
 from lino.core.dbutils import navinfo
 
+from lino.utils.media import TmpMediaFile
+
 
 #~ from lino.ui import requests as ext_requests
 from lino.core import constants as ext_requests
@@ -68,14 +70,14 @@ def requested_actor(app_label,actor):
     """
     Utility function which returns the requested actor, 
     either directly or (if specified name is a model) that 
-    model's default actor.
+    model's default table.
     """
     x = getattr(settings.SITE.modules,app_label)
     cl = getattr(x,actor)
     if not isinstance(cl,type):
         raise Exception("%s.%s is not a class" % (app_label,actor))
     if issubclass(cl,models.Model):
-        return cl._lino_default_table
+        return cl.get_default_table()
     if not issubclass(cl,actors.Actor):
         #~ raise http.Http404("%r is not an actor" % cl)
         raise Exception("%r is not an actor" % cl)
@@ -87,7 +89,9 @@ def requested_actor(app_label,actor):
 def action_request(app_label,actor,request,rqdata,is_list,**kw):
     rpt = requested_actor(app_label,actor)
     action_name = rqdata.get(ext_requests.URL_PARAM_ACTION_NAME,None)
-    if action_name is None:
+    #~ if action_name is None:
+        #~ logger.info("20130731 action_name is None")
+    if not action_name:
         if is_list: 
             action_name = rpt.default_list_action_name
         else:
@@ -97,66 +101,15 @@ def action_request(app_label,actor,request,rqdata,is_list,**kw):
         raise http.Http404("%s has no url action %r (possible values are %s)" % (
             rpt,action_name,rpt.get_url_action_names()))
     user = request.subst_user or request.user
-    if not a.get_view_permission(user.profile):
-        #~ raise Http403(_("As %s you have no permission to run this action.") % user.profile)
-        raise exceptions.PermissionDenied(
-            _("As %s you have no permission to run this action.") % user.profile)
-        #~ return http.HttpResponseForbidden(_("As %s you have no permission to run this action.") % user.profile)
+    if False: # 20130829
+        if not a.get_view_permission(user.profile):
+            raise exceptions.PermissionDenied(
+                _("As %s you have no permission to run this action.") % user.profile)
+            #~ return http.HttpResponseForbidden(_("As %s you have no permission to run this action.") % user.profile)
     ar = rpt.request(request=request,action=a,**kw)
     #~ ar.renderer = settings.SITE.ui.ext_renderer
     return ar
     
-def run_action(ar,elem):
-    try:
-        rv = ar.bound_action.action.run_from_ui(elem,ar)
-        if rv is None:
-            rv  = ui.success()
-        return settings.SITE.ui.action_response(rv)
-        #~ return rv
-    #~ except actions.ConfirmationRequired,e:
-        #~ r = dict(
-          #~ success=True,
-          #~ confirm_message='\n'.join([unicode(m) for m in e.messages]),
-          #~ step=e.step)
-        #~ return ar.ui.action_response(r)
-    #~ except actions.DialogRequired,e:
-        #~ r = dict(
-          #~ success=True,
-          #~ dialog_fn=e.dialog,
-          #~ step=e.step)
-        #~ return ar.ui.action_response(r)
-    #~ except actions.DecisionRequired,e:
-        #~ r = dict(
-          #~ success=True,
-          #~ message=unicode(e),
-          #~ decide_id=ar.ui.add_decision(e))
-        #~ return ar.ui.action_response(r)
-    except Warning as e:
-        r = dict(
-          success=False,
-          message=unicode(e),
-          alert=True)
-        return settings.SITE.ui.action_response(r)
-    except Exception as e:
-        if elem is None:
-            msg = unicode(e)
-        else:
-            if isinstance(elem,models.Model):
-                elem = dd.obj2unicode(elem)
-            msg = _(
-              "Action \"%(action)s\" failed for %(record)s:") % dict(
-              action=ar.bound_action.full_name(),
-              record=elem)
-            msg += "\n" + unicode(e)
-        msg += '.\n' + _(
-          "An error report has been sent to the system administrator.")
-        logger.warning(msg)
-        logger.exception(e)
-        r = settings.SITE.ui.error(e,msg,alert=_("Oops!"))
-        return settings.SITE.ui.action_response(r)
-          
-    
-  
     
         
 def json_response_kw(**kw):
@@ -209,7 +162,7 @@ def elem2rec_empty(ar,ah,elem,**rec):
     #~ rec.update(id=elem.pk) or -99999)
     if ar.actor.parameters:
         #~ rec.update(param_values=ar.ah.store.pv2dict(ar.ui,ar.param_values))
-        rec.update(param_values=ar.actor.params_layout.params_store.pv2dict(settings.SITE.ui,ar.param_values))
+        rec.update(param_values=ar.actor.params_layout.params_store.pv2dict(ar.param_values))
     return rec
 
 def elem2rec_detailed(ar,elem,**rec):
@@ -252,7 +205,7 @@ def delete_element(ar,elem):
     msg = ar.actor.disable_delete(elem,ar)
     if msg is not None:
         rv = ar.error(None,msg,alert=True)
-        return settings.SITE.ui.action_response(rv)
+        return settings.SITE.ui.render_action_response(rv)
             
     #~ dblogger.log_deleted(ar.request,elem)
     
@@ -268,7 +221,7 @@ def delete_element(ar,elem):
             ) % dict(record=dd.obj2unicode(elem),error=e)
         #~ msg = "Failed to delete %s." % element_name(elem)
         rv = ar.error(None,msg)
-        return settings.SITE.ui.action_response(rv)
+        return settings.SITE.ui.render_action_response(rv)
         #~ raise Http404(msg)
         
     
@@ -420,7 +373,7 @@ class MainHtml(View):
         settings.SITE.startup()
         ui = settings.SITE.ui
         rv = ui.success(html=settings.SITE.get_main_html(request))
-        return ui.action_response(rv)
+        return ui.render_action_response(rv)
         
 class Authenticate(View):
     """
@@ -435,12 +388,12 @@ class Authenticate(View):
             #~ username = request.session['username']
             #~ del request.session['password']
             rv = dict(success=True,message="User %r logged out." % username)
-            return settings.SITE.ui.action_response(rv)
+            return settings.SITE.ui.render_action_response(rv)
         raise http.Http404()
             
 
     def post(self, request, *args, **kw):
-        from lino.utils import auth
+        #~ from lino.core import auth
         #~ from django.contrib.sessions.backends.db import SessionStore
         #~ ss = SessionStore()
         username = request.POST.get('username')
@@ -448,14 +401,15 @@ class Authenticate(View):
         user = auth.authenticate(username,password)
         if user is None:
             rv = settings.SITE.ui.error("Could not authenticate %r" % username)
-            return settings.SITE.ui.action_response(rv)
+            return settings.SITE.ui.render_action_response(rv)
         request.session['username'] = username
         request.session['password'] = password
         #~ request.session['password'] = request.GET.get('password')
         #~ auth.login(request,request.GET.get('username'), request.GET.get('password'))
         #~ ss.save()
+        #~ logger.info("20130924 Now logged in as %r" % username)
         rv = settings.SITE.ui.success("Now logged in as %r" % username)
-        return settings.SITE.ui.action_response(rv)
+        return settings.SITE.ui.render_action_response(rv)
       
 
 class RunJasmine(View):
@@ -588,7 +542,8 @@ def choices_for_field(request,actor,field):
         
     elif isinstance(field,models.ForeignKey):
         m = field.rel.to
-        t = getattr(m,'_lino_choices_table',m._lino_default_table)
+        #~ t = getattr(m,'_lino_choices_table',m.get_default_table())
+        t = m.get_default_table()
         qs = t.request(request=request).data_iterator
         #~ logger.info('20120710 choices_view(FK) %s --> %s',t,qs)
         def row2dict(obj,d):
@@ -698,7 +653,6 @@ class Restful(View):
     def post(self,request,app_label=None,actor=None,pk=None):
         #~ ui = settings.SITE.ui
         rpt = requested_actor(app_label,actor)
-        #~ a = rpt.default_action
         if pk is None:
             elem = None
         else:
@@ -741,7 +695,9 @@ class Restful(View):
         rows = [ 
           rh.store.row2dict(ar,row,rh.store.list_fields) 
             for row in ar.sliced_data_iterator ]
-        return json_response_kw(count=ar.get_total_count(),rows=rows)
+        kw = dict(count=ar.get_total_count(),rows=rows)
+        kw.update(title=unicode(ar.get_title()))
+        return json_response(kw)
         
     def put(self,request,app_label=None,actor=None,pk=None):
         #~ ui = settings.SITE.ui
@@ -777,25 +733,26 @@ class ApiElement(View):
             #~ msg = "User %s cannot view %s." % (request.user,ah.actor)
             #~ return http.HttpResponseForbidden()
         
-        if pk and pk != '-99999' and pk != '-99998':
-            elem = rpt.get_row_by_pk(pk)
-            if elem is None:
-                raise http.Http404("%s has no row with primary key %r" % (rpt,pk))
-                #~ raise Exception("20120327 %s.get_row_by_pk(%r)" % (rpt,pk))
-        else:
-            elem = None
-        
         action_name = request.GET.get(ext_requests.URL_PARAM_ACTION_NAME,
           rpt.default_elem_action_name)
         ba = rpt.get_url_action(action_name)
         if ba is None:
             raise http.Http404("%s has no action %r" % (rpt,action_name))
             
-        #~ ar = rpt.request(ui,request,a)
         ar = ba.request(request=request)
         ar.renderer = ui.ext_renderer
         ah = ar.ah
         
+        if pk and pk != '-99999' and pk != '-99998':
+            elem = rpt.get_row_by_pk(pk)
+            if elem is None:
+                raise http.Http404("%s has no row with primary key %r" % (rpt,pk))
+                #~ raise Exception("20120327 %s.get_row_by_pk(%r)" % (rpt,pk))
+            ar.selected_rows = [elem]
+        else:
+            elem = None
+        
+        
         fmt = request.GET.get(ext_requests.URL_PARAM_FORMAT,ba.action.default_format)
 
         #~ if isinstance(a,actions.OpenWindowAction):
@@ -803,11 +760,9 @@ class ApiElement(View):
           
             if fmt == ext_requests.URL_FORMAT_JSON:
                 if pk == '-99999':
-                    assert elem is None
                     elem = ar.create_instance()
                     datarec = elem2rec_insert(ar,ah,elem)
                 elif pk == '-99998':
-                    assert elem is None
                     elem = ar.create_instance()
                     datarec = elem2rec_empty(ar,ah,elem)
                 else:
@@ -815,7 +770,7 @@ class ApiElement(View):
                 
                 return json_response(datarec)
                 
-            after_show = ar.get_status(ui,record_id=pk)
+            after_show = ar.get_status(record_id=pk)
             
             tab = request.GET.get(ext_requests.URL_PARAM_TAB,None)
             if tab is not None: 
@@ -831,12 +786,12 @@ class ApiElement(View):
                 raise http.Http404("%s failed for %r" % (ba,elem))
             return http.HttpResponseRedirect(target)
             
-        if isinstance(ba.action,actions.RowAction):
-            if pk == '-99998':
-                assert elem is None
-                elem = ar.create_instance()
-            return run_action(ar,elem)
-        raise NotImplementedError("Action %s is not implemented)" % ba)
+        if pk == '-99998':
+            assert elem is None
+            elem = ar.create_instance()
+            ar.selected_rows = [elem]
+
+        return settings.SITE.ui.run_action(ar)
                 
         
     def post(self,request,app_label=None,actor=None,pk=None):
@@ -845,12 +800,11 @@ class ApiElement(View):
         elem = ar.actor.get_row_by_pk(pk)
         if elem is None:
             raise http.Http404("%s has no row with primary key %r" % (ar.actor,pk))
-        if isinstance(ar.bound_action.action,actions.RowAction):
-            if pk == '-99998':
-                assert elem is None
-                elem = ar.create_instance()
-            return run_action(ar,elem)
-        raise NotImplementedError("Action %s is not implemented)" % ar)
+        if pk == '-99998':
+            assert elem is None
+            elem = ar.create_instance()
+        ar.selected_rows = [elem]
+        return settings.SITE.ui.run_action(ar)
         
     def put(self,request,app_label=None,actor=None,pk=None):
         data = http.QueryDict(request.body) # raw_post_data before Django 1.4
@@ -869,7 +823,7 @@ class ApiElement(View):
             raise http.Http404("%s has no row with primary key %r" % (rpt,pk))
         ar = rpt.request(request=request)
         return delete_element(ar,elem)
-        
+
 
         
   
@@ -900,10 +854,6 @@ class ApiList(View):
         ar.renderer = settings.SITE.ui.ext_renderer
         #~ print 20121116, ar.bound_action.action.action_name
         if ar.bound_action.action.action_name in ['duplicate','post','poststay','insert']:
-        #~ if isinstance(ar.bound_action.action,(
-              #~ actions.InsertRow,
-              #~ actions.DuplicateAction,
-              #~ actions.SubmitInsert)):
             rh = ar.ah
             elem = ar.create_instance()
             if rh.actor.handle_uploaded_files is not None:
@@ -912,9 +862,7 @@ class ApiList(View):
             else:
                 file_upload = False
             return form2obj_and_save(ar,request.POST,elem,True,False,file_upload)
-        return run_action(ar,None)
-        #~ rv = ar.bound_action.action.run(ar)
-        #~ return rv
+        return settings.SITE.ui.run_action(ar)
       
     def get(self,request,app_label=None,actor=None):
         #~ ar = action_request(app_label,actor,request,request.GET,limit=PLAIN_PAGE_LENGTH)
@@ -951,13 +899,13 @@ class ApiList(View):
               #~ gc_choices=[gc.data for gc in ar.actor.grid_configs])
             if ar.actor.parameters:
                 #~ kw.update(param_values=ar.actor.params_layout.params_store.pv2dict(settings.SITE.ui,ar.param_values))
-                kw.update(param_values=ar.actor.params_layout.params_store.pv2dict(settings.SITE.ui,ar.param_values))
+                kw.update(param_values=ar.actor.params_layout.params_store.pv2dict(ar.param_values))
             return json_response(kw) 
                 
         if fmt == ext_requests.URL_FORMAT_HTML:
             #~ ar.renderer = ui.ext_renderer
             #~ after_show = ar.get_status(ar.ui)
-            after_show = ar.get_status(settings.SITE.ui)
+            after_show = ar.get_status()
             
             sp = request.GET.get(ext_requests.URL_PARAM_SHOW_PARAMS_PANEL,None)
             if sp is not None: 
@@ -1004,17 +952,11 @@ class ApiList(View):
                 w.writerow([unicode(v) for v in rh.store.row2list(ar,row)])
             return response
             
-        if fmt in (ext_requests.URL_FORMAT_PDF,ext_requests.URL_FORMAT_ODT):
-          
-            ip = ar.request.META.get('REMOTE_ADDR','unknown_ip')
-            target_parts = ['cache', 'appypdf', ip, str(ar.actor) + '.' + fmt]
-            target_file = os.path.join(settings.MEDIA_ROOT,*target_parts)
-            settings.SITE.makedirs_if_missing(os.path.dirname(target_file))
-            target_url = settings.SITE.build_media_url(*target_parts)
-          
-            ar.appy_render(target_file)
-            
-            return http.HttpResponseRedirect(target_url)
+        #~ if fmt in (ext_requests.URL_FORMAT_PDF,ext_requests.URL_FORMAT_ODT):
+            #~ mf = TmpMediaFile(ar,fmt)
+            #~ settings.SITE.makedirs_if_missing(os.path.dirname(mf.name))
+            #~ ar.appy_render(mf.name)
+            #~ return http.HttpResponseRedirect(mf.url)
             
         if fmt == ext_requests.URL_FORMAT_PRINTER:
             if ar.get_total_count() > MAX_ROW_COUNT:
@@ -1024,11 +966,13 @@ class ApiList(View):
             doc = xghtml.Document(force_unicode(ar.get_title()))
             doc.body.append(E.h1(doc.title))
             t = doc.add_table()
-            settings.SITE.ui.ar2html(ar,t,ar.data_iterator)
+            #~ settings.SITE.ui.ar2html(ar,t,ar.data_iterator)
+            ar.dump2html(t,ar.data_iterator)
             doc.write(response,encoding='utf-8')
             return response
             
-        raise http.Http404("Format %r not supported for GET on %s" % (fmt,ar.actor))
+        return settings.SITE.ui.run_action(ar)
+        #~ raise http.Http404("Format %r not supported for GET on %s" % (fmt,ar.actor))
 
       
 class GridConfig(View):
@@ -1065,7 +1009,7 @@ class GridConfig(View):
                 table=rpt,error=e)
             return settings.SITE.ui.error(None,msg,alert=True)
         #~ logger.info(msg)
-        settings.SITE.ui.build_site_cache(True)
+        settings.SITE.ui.ext_renderer.build_site_cache(True)
         return settings.SITE.ui.success(msg)
         
 MENUS = dict()        
