@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-## Copyright 2009-2012 Luc Saffre
+## Copyright 2009-2013 Luc Saffre
 ## This file is part of the Lino project.
 ## Lino is free software; you can redistribute it and/or modify 
 ## it under the terms of the GNU General Public License as published by
@@ -82,6 +82,30 @@ def join_words(*words):
     join the remaining word using a single space.
     """
     return ' '.join([unicode(x) for x in words if x])
+    
+def join_elems(elems,sep=' '):
+    """
+    Examples::
+    >>> join_elems([1,2,3])
+    [1, ' ', 2, ' ', 3]
+    >>> join_elems([1,2,3],' / ')
+    [1, ' / ', 2, ' / ', 3]
+    >>> join_elems([])
+    []
+    """
+    if not callable(sep):
+        sep_value = sep
+        def sep():
+            return sep_value
+    l = []  
+    s = None
+    for e in elems:
+        if s is not None:
+            l.append(s)
+        s = sep()
+        l.append(e)
+    return l
+    
       
 
 def d2iso(d):
