@@ -13,33 +13,8 @@
 ## along with Lino; if not, see <http://www.gnu.org/licenses/>.
 
 """
-An extended :term:`appy.pod` renderer that installs additional functions
-to be used in `do text|section|table... from ...
-<http://appyframework.org/podWritingAdvancedTemplates.html>`__ 
-statements.
-
-
-- `restify(s)`: 
-  Render a string `s` which contains reStructuredText markup.
-  The string is first passed to 
-  :func:`lino.utils.restify.restify` to convert it to XHTML, 
-  then to `appy.pod`'s built in `xhtml` function.
-  Without this, users would have to write each time something like::
-
-    do text
-    from xhtml(restify(self.body).encode('utf-8'))
-
-- `html(s)` :
-  Render a string that is in HTML (not XHTML).
-
-- `ehtml(e)` :
-  Render an ElementTree HTML object 
-  (generated using :mod:`lino.utils.xmlgen.html`)
-  by passing it to :mod:`lino.utils.html2odf`.
-
-- `table(ar,column_names=None)` : render an :class:`lino.core.tables.TableRequest` 
-  as a table.
-
+Defines :class:`Renderer`
+and :class:`PrintTableAction`
 """
 
 from __future__ import unicode_literals
@@ -51,6 +26,7 @@ import os
 
 from appy.pod.renderer import Renderer as AppyRenderer
 
+from django.utils.translation import ugettext_lazy as _
 from django.utils.encoding import force_unicode
 from django.conf import settings
 
@@ -63,9 +39,16 @@ from odf.style import TableColumnProperties, TableRowProperties, TableCellProper
 from odf import text
 from odf.table import Table, TableColumns, TableColumn, TableHeaderRows, TableRows, TableRow, TableCell
 
+from north import dbutils
+
+#~ from lino.utils import jsgen
+from lino.core import actions
 from lino.utils.restify import restify
 from lino.utils.html2xhtml import html2xhtml
 from lino.utils.html2odf import html2odf, toxml
+from lino.utils.config import find_config_file
+from lino.utils.media import TmpMediaFile
+
 
 
 OAS = '<office:automatic-styles>'
@@ -88,12 +71,47 @@ UL_LIST_STYLE = """\
 
 
 class Renderer(AppyRenderer):
-  
+    """
+    An extended :term:`appy.pod` renderer that installs additional functions
+    to be used in `do text|section|table... from ...
+    <http://appyframework.org/podWritingAdvancedTemplates.html>`__ 
+    statements.
+
+
+    - `jinja(template_name)`: 
+      Render the template named `template_name` using Jinja.
+      I `template_name` contains no dot, then the default filename 
+      extension `.body.html` is added.
+      The template is supposed to contain HTML.
+      
+    - `restify(s)`: 
+      Render a string `s` which contains reStructuredText markup.
+      The string is first passed to 
+      :func:`lino.utils.restify.restify` to convert it to XHTML, 
+      then to `appy.pod`'s built in `xhtml` function.
+      Without this, users would have to write each time something like::
+
+        do text
+        from xhtml(restify(self.body).encode('utf-8'))
+
+    - `html(s)` :
+      Render a string that is in HTML (not XHTML).
+
+    - `ehtml(e)` :
+      Render an ElementTree HTML object 
+      (generated using :mod:`lino.utils.xmlgen.html`)
+      by passing it to :mod:`lino.utils.html2odf`.
+
+    - `table(ar,column_names=None)` : render an :class:`lino.core.tables.TableRequest` 
+      as a table.
+
+    """
     def __init__(self, ar, template, context, result, **kw):
         self.ar = ar
         #~ context.update(appy_renderer=self)
         context.update(restify=self.restify_func)
         context.update(html=self.html_func)
+        context.update(jinja=self.jinja_func)
         context.update(table=self.insert_table)
         context.update(as_odt=self.as_odt)
         #~ context.update(story=self.insert_story)
@@ -106,6 +124,7 @@ class Renderer(AppyRenderer):
         #~ context.update(ui=self.extjs_ui)
         context.update(ui=settings.SITE.ui)
         context.update(settings=settings)
+        context.update(sc=settings.SITE.site_config)
         context.update(settings.SITE.modules)
         kw.update(finalizeFunction=self.finalize_func)
         AppyRenderer.__init__(self,template,context,result, **kw)
@@ -114,6 +133,26 @@ class Renderer(AppyRenderer):
         self.my_automaticstyles = []
         self.my_styles = []
   
+    def jinja_func(self,template_name,**kw):
+        try:
+            if not '.' in template_name:
+                template_name += '.html'
+            #~ printable = self.contentParser.env.context.get('this',None)
+            #~ from lino.mixins.printable import TypedPrintable
+            #~ if isinstance(printable,TypedPrintable):
+                #~ template_name = printable.get_templates_group() + '/' + template_name
+            #~ print 20130910, settings.SITE.jinja_env
+            template = settings.SITE.jinja_env.get_template(template_name)
+            #~ print 20130910, template, dir(self)
+            html = template.render(self.contentParser.env.context)
+            return self.html_func(html)
+            #~ print 20130910, html
+            #~ return self.renderXhtml(html,**kw)
+        except Exception as e:
+            import traceback
+            traceback.print_exc(e)
+        
+        
     def restify_func(self,unicode_string,**kw):
         """
         
@@ -239,7 +278,6 @@ class Renderer(AppyRenderer):
         
     def insert_table_(self,ar,column_names=None,table_width=180):
         ar.setup_from(self.ar)
-        
         columns, headers, widths = ar.get_field_info(column_names)
         widths = map(int,widths)
         tw = sum(widths)
@@ -468,3 +506,122 @@ class Renderer(AppyRenderer):
             #~ doc.save(output_file) # , True)
         #~ return doc
         
+
+
+class PrintTableAction(actions.Action):
+    """
+    
+    """
+    label = _("Table (landscape)")
+    help_text = _('Show this table as a pdf document')
+    icon_name = 'pdf'
+    sort_index = -10
+    select_rows = False
+    default_format = 'ajax'
+    preprocessor = "Lino.get_current_grid_config"
+    MAX_ROW_COUNT = 900
+    template_name = "Table.odt"
+    target_file_format = 'pdf' # can be pdf, odt or rtf
+    #~ target_file_format = 'odt' # write to odt to see error messages for debugging templates
+    combo_group = 'pdf'
+
+    def is_callable_from(self,caller):
+        return isinstance(caller,actions.GridEdit) 
+        
+    def run_from_ui(self,ar,**kw):
+        #~ print 20130912
+        #~ obj = ar.selected_rows[0]
+        mf = TmpMediaFile(ar,self.target_file_format)
+        settings.SITE.makedirs_if_missing(os.path.dirname(mf.name))
+        self.appy_render(ar,mf.name)
+        kw.update(success=True)
+        kw.update(open_url=mf.url)
+        #~ return http.HttpResponseRedirect(mf.url)
+        return kw
+
+    def appy_render(self,ar,target_file):
+        
+        
+        if ar.get_total_count() > self.MAX_ROW_COUNT:
+            raise Exception(_("List contains more than %d rows") % self.MAX_ROW_COUNT)
+    
+        tplgroup = None
+        tplfile = find_config_file(self.template_name,tplgroup)
+        if not tplfile:
+            raise Exception("No file %s / %s" % (tplgroup,self.template_name))
+            
+        ar.renderer = settings.SITE.ui.ext_renderer # 20120624
+        
+        context = self.get_context(ar)
+        if os.path.exists(target_file):
+            os.remove(target_file)
+        logger.info(u"appy.pod render %s -> %s (params=%s",
+            tplfile,target_file,settings.SITE.appy_params)
+        renderer = Renderer(ar,tplfile, context, target_file,**settings.SITE.appy_params)
+        renderer.run()
+        
+    def get_context(self,ar):
+        return dict(
+            ar=ar,
+            title=unicode(ar.get_title()),
+            dtos=dbutils.dtos,
+            dtosl=dbutils.dtosl,
+            dtomy=dbutils.dtomy,
+            babelattr=dbutils.babelattr,
+            babelitem=dbutils.babelitem,
+            tr=dbutils.babelitem,
+            settings=settings,
+            _ = _,
+            #~ knowledge_text=fields.knowledge_text,
+            )
+        
+        
+class PortraitPrintTableAction(PrintTableAction):
+    label = _("Table (portrait)")
+    template_name = "Table-portrait.odt"
+    sort_index = -9
+    
+class PrintLabelsAction(PrintTableAction):
+    """
+    Add this action to your table, which is expected to execute on a 
+    model which implements 
+    :class:`Addressable <lino.mixins.addressable.Addressable>`
+
+    """
+    label = _("Labels")
+    help_text = _('Generate mailing labels for these recipients')
+    #~ icon_name = None
+    template_name = "Labels.odt" 
+    #~ combo_group = 'pdf'
+    sort_index = -8
+    
+    def get_context(self,ar):
+        context = super(PrintLabelsAction,self).get_context(ar)
+        context.update(recipients=self.get_recipients(ar))
+        return context
+
+    def get_recipients(self,ar):
+        """
+        This is here so you can override it. For example::
+        
+            class MyLabelsAction(dd.PrintLabelsAction)
+                # silently ignore all recipients with empty 'street' field 
+                def get_recipients(self,ar):
+                    for obj in ar:
+                        if obj.street:
+                            yield obj
+                            
+        But I personally would rather add a parameters panel so that 
+        users can explicitly say whether they want labels for invalid 
+        addresses or not::
+        
+            class MyTable(dd.Table):
+                parameters = dict(
+                    only_valid_recipients=models.BooleanField(_("only valid recipients")
+                    )
+        
+        """
+        return iter(ar)
+        
+        
+        
